@@ -4,6 +4,8 @@
 // Removes useless (inaccessible or non-coaccessible) states and arcs from an
 // FST.
 
+#include <memory>
+
 #include <fst/script/connect.h>
 
 int main(int argc, char **argv) {
@@ -25,10 +27,10 @@ int main(int argc, char **argv) {
   string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
   string out_name = argc > 2 ? argv[2] : "";
 
-  MutableFstClass *fst = MutableFstClass::Read(in_name, true);
+  std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_name, true));
   if (!fst) return 1;
 
-  s::Connect(fst);
+  s::Connect(fst.get());
 
   fst->Write(out_name);
 
