@@ -3,6 +3,8 @@
 //
 // Epsilon-normalizes an FST.
 
+#include <memory>
+
 #include <fst/script/epsnormalize.h>
 
 DEFINE_bool(eps_norm_output, false, "Normalize output epsilons");
@@ -26,7 +28,7 @@ int main(int argc, char **argv) {
   string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
   string out_name = argc > 2 ? argv[2] : "";
 
-  FstClass *ifst = FstClass::Read(in_name);
+  std::unique_ptr<FstClass> ifst(FstClass::Read(in_name));
   if (!ifst) return 1;
 
   fst::EpsNormalizeType eps_norm_type = FLAGS_eps_norm_output
