@@ -3,6 +3,8 @@
 //
 // Two DFAs are equivalent iff their exit status is zero.
 
+#include <memory>
+
 #include <fst/script/equivalent.h>
 #include <fst/script/randequivalent.h>
 
@@ -42,10 +44,10 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  FstClass *ifst1 = FstClass::Read(in1_name);
+  std::unique_ptr<FstClass> ifst1(FstClass::Read(in1_name));
   if (!ifst1) return 1;
 
-  FstClass *ifst2 = FstClass::Read(in2_name);
+  std::unique_ptr<FstClass> ifst2(FstClass::Read(in2_name));
   if (!ifst2) return 1;
 
   if (!FLAGS_random) {
