@@ -3,6 +3,8 @@
 //
 // Inverts a transduction.
 
+#include <memory>
+
 #include <fst/script/invert.h>
 
 int main(int argc, char **argv) {
@@ -24,10 +26,10 @@ int main(int argc, char **argv) {
   string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
   string out_name = argc > 2 ? argv[2] : "";
 
-  MutableFstClass *fst = MutableFstClass::Read(in_name, true);
+  std::unique_ptr<MutableFstClass> fst(MutableFstClass::Read(in_name, true));
   if (!fst) return 1;
 
-  s::Invert(fst);
+  s::Invert(fst.get());
   fst->Write(out_name);
 
   return 0;
