@@ -5,6 +5,8 @@
 // exit status is zero. FSTs should be deterministic when viewed as unweighted
 // automata.
 
+#include <memory>
+
 #include <fst/script/isomorphic.h>
 
 DEFINE_double(delta, fst::kDelta, "Comparison/quantization delta");
@@ -33,10 +35,10 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  FstClass *ifst1 = FstClass::Read(in1_name);
+  std::unique_ptr<FstClass> ifst1(FstClass::Read(in1_name));
   if (!ifst1) return 1;
 
-  FstClass *ifst2 = FstClass::Read(in2_name);
+  std::unique_ptr<FstClass> ifst2(FstClass::Read(in2_name));
   if (!ifst2) return 1;
 
   bool result = s::Isomorphic(*ifst1, *ifst2, FLAGS_delta);
