@@ -4,6 +4,8 @@
 // Pushes weights and/or output labels in an FST toward the initial or final
 // states.
 
+#include <memory>
+
 #include <fst/script/push.h>
 
 DEFINE_double(delta, fst::kDelta, "Comparison/quantization delta");
@@ -34,7 +36,7 @@ int main(int argc, char **argv) {
   string in_name = (argc > 1 && strcmp(argv[1], "-") != 0) ? argv[1] : "";
   string out_name = argc > 2 ? argv[2] : "";
 
-  FstClass *ifst = FstClass::Read(in_name);
+  std::unique_ptr<FstClass> ifst(FstClass::Read(in_name));
   if (!ifst) return 1;
 
   uint32 flags = 0;
