@@ -1,36 +1,20 @@
-// pdtcompose.cc
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
 // Composes a PDT and an FST.
-//
 
 #include <vector>
-using std::vector;
-#include <utility>
-using std::pair; using std::make_pair;
 
-#include <fst/util.h>
 #include <fst/extensions/pdt/pdtscript.h>
+#include <fst/util.h>
 #include <fst/script/connect.h>
 
 DEFINE_string(pdt_parentheses, "", "PDT parenthesis label pairs.");
 DEFINE_bool(left_pdt, true, "1st arg is PDT (o.w. 2nd arg).");
 DEFINE_bool(connect, true, "Trim output");
+DEFINE_string(compose_filter, "paren",
+              "Composition filter, one of: \"expand\", \"expand_paren\", "
+              "\"paren\"");
 
 int main(int argc, char **argv) {
   namespace s = fst::script;
@@ -66,16 +50,29 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  vector<pair<int64, int64> > parens;
+  std::vector<s::LabelPair> parens;
   fst::ReadLabelPairs(FLAGS_pdt_parentheses, &parens, false);
 
   s::VectorFstClass ofst(ifst1->ArcType());
-  fst::ComposeOptions copts(false);
+
+  fst::PdtComposeFilter compose_filter;
+
+  if (FLAGS_compose_filter == "expand") {
+    compose_filter = fst::EXPAND_FILTER;
+  } else if (FLAGS_compose_filter == "expand_paren") {
+    compose_filter = fst::EXPAND_PAREN_FILTER;
+  } else if (FLAGS_compose_filter == "paren") {
+    compose_filter = fst::PAREN_FILTER;
+  } else {
+    LOG(ERROR) << argv[0]
+               << "Unknown compose filter type: " << FLAGS_compose_filter;
+    return 1;
+  }
+
+  fst::PdtComposeOptions copts(FLAGS_connect, compose_filter);
 
   s::PdtCompose(*ifst1, *ifst2, parens, &ofst, copts, FLAGS_left_pdt);
 
-  if (FLAGS_connect)
-    s::Connect(&ofst);
   ofst.Write(out_name);
 
   return 0;
