@@ -1,24 +1,10 @@
-// pdtinfo.cc
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
+//
+// Prints out various information about a PDT such as number of states, arcs,
+// and parentheses.
 
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
-// Prints out various information about a PDT such as number of
-// states, arcs and parentheses.
-//
+#include <vector>
 
 #include <fst/extensions/pdt/pdtscript.h>
 #include <fst/util.h>
@@ -32,7 +18,6 @@ int main(int argc, char **argv) {
   usage += argv[0];
   usage += " in.pdt\n";
 
-
   std::set_new_handler(FailedNewHandler);
   SET_FLAGS(usage.c_str(), &argc, &argv, true);
   if (argc > 2) {
@@ -50,7 +35,7 @@ int main(int argc, char **argv) {
     return 1;
   }
 
-  vector<pair<int64, int64> > parens;
+  std::vector<s::LabelPair> parens;
   fst::ReadLabelPairs(FLAGS_pdt_parentheses, &parens, false);
 
   s::PrintPdtInfo(*ifst, parens);
