@@ -1,34 +1,16 @@
-// arc-map.h
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
 // Class to map over/transform arcs e.g., change semirings or
 // implement project/invert. Consider using when operation does
 // not change the number of arcs (except possibly superfinal arcs).
 
-#ifndef FST_LIB_ARC_MAP_H__
-#define FST_LIB_ARC_MAP_H__
+#ifndef FST_LIB_ARC_MAP_H_
+#define FST_LIB_ARC_MAP_H_
 
-#include <unordered_map>
-using std::unordered_map;
-using std::unordered_multimap;
 #include <string>
+#include <unordered_map>
 #include <utility>
-using std::pair; using std::make_pair;
 
 #include <fst/cache.h>
 #include <fst/mutable-fst.h>
@@ -105,11 +87,10 @@ enum MapSymbolsAction {
 // decoding. We also include map versions that pass the mapper
 // by value or const reference when this suffices.
 
-
 // Maps an arc type A using a mapper function object C, passed
 // by pointer.  This version modifies its Fst input.
-template<class A, class C>
-void ArcMap(MutableFst<A> *fst, C* mapper) {
+template <class A, class C>
+void ArcMap(MutableFst<A> *fst, C *mapper) {
   typedef typename A::StateId StateId;
   typedef typename A::Weight Weight;
 
@@ -119,8 +100,7 @@ void ArcMap(MutableFst<A> *fst, C* mapper) {
   if (mapper->OutputSymbolsAction() == MAP_CLEAR_SYMBOLS)
     fst->SetOutputSymbols(0);
 
-  if (fst->Start() == kNoStateId)
-    return;
+  if (fst->Start() == kNoStateId) return;
 
   uint64 props = fst->Properties(kFstProperties, false);
 
@@ -132,8 +112,8 @@ void ArcMap(MutableFst<A> *fst, C* mapper) {
   }
 
   for (StateId s = 0; s < fst->NumStates(); ++s) {
-    for (MutableArcIterator< MutableFst<A> > aiter(fst, s);
-         !aiter.Done(); aiter.Next()) {
+    for (MutableArcIterator<MutableFst<A>> aiter(fst, s); !aiter.Done();
+         aiter.Next()) {
       const A &arc = aiter.Value();
       aiter.SetValue((*mapper)(arc));
     }
@@ -143,7 +123,7 @@ void ArcMap(MutableFst<A> *fst, C* mapper) {
       default: {
         A final_arc = (*mapper)(A(0, 0, fst->Final(s), kNoStateId));
         if (final_arc.ilabel != 0 || final_arc.olabel != 0) {
-          FSTERROR() << "ArcMap: non-zero arc labels for superfinal arc";
+          FSTERROR() << "ArcMap: Non-zero arc labels for superfinal arc";
           fst->SetProperties(kError, kError);
         }
 
@@ -175,7 +155,7 @@ void ArcMap(MutableFst<A> *fst, C* mapper) {
               final_arc.weight != Weight::Zero())
             fst->AddArc(s, A(final_arc.ilabel, final_arc.olabel,
                              final_arc.weight, superfinal));
-            fst->SetFinal(s, Weight::Zero());
+          fst->SetFinal(s, Weight::Zero());
         }
         break;
       }
@@ -184,20 +164,18 @@ void ArcMap(MutableFst<A> *fst, C* mapper) {
   fst->SetProperties(mapper->Properties(props), kFstProperties);
 }
 
-
 // Maps an arc type A using a mapper function object C, passed
 // by value.  This version modifies its Fst input.
-template<class A, class C>
+template <class A, class C>
 void ArcMap(MutableFst<A> *fst, C mapper) {
   ArcMap(fst, &mapper);
 }
 
-
 // Maps an arc type A to an arc type B using mapper function
 // object C, passed by pointer. This version writes the mapped
 // input Fst to an output MutableFst.
-template<class A, class B, class C>
-void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C* mapper) {
+template <class A, class B, class C>
+void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C *mapper) {
   typedef typename A::StateId StateId;
   typedef typename A::Weight Weight;
 
@@ -222,12 +200,12 @@ void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C* mapper) {
 
   MapFinalAction final_action = mapper->FinalAction();
   if (ifst.Properties(kExpanded, false)) {
-    ofst->ReserveStates(CountStates(ifst) +
-                        final_action == MAP_NO_SUPERFINAL ? 0 : 1);
+    ofst->ReserveStates(
+        CountStates(ifst) + final_action == MAP_NO_SUPERFINAL ? 0 : 1);
   }
 
   // Add all states.
-  for (StateIterator< Fst<A> > siter(ifst); !siter.Done(); siter.Next())
+  for (StateIterator<Fst<A>> siter(ifst); !siter.Done(); siter.Next())
     ofst->AddState();
 
   StateId superfinal = kNoStateId;
@@ -235,13 +213,12 @@ void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C* mapper) {
     superfinal = ofst->AddState();
     ofst->SetFinal(superfinal, B::Weight::One());
   }
-  for (StateIterator< Fst<A> > siter(ifst); !siter.Done(); siter.Next()) {
+  for (StateIterator<Fst<A>> siter(ifst); !siter.Done(); siter.Next()) {
     StateId s = siter.Value();
-    if (s == ifst.Start())
-      ofst->SetStart(s);
+    if (s == ifst.Start()) ofst->SetStart(s);
 
     ofst->ReserveArcs(s, ifst.NumArcs(s));
-    for (ArcIterator< Fst<A> > aiter(ifst, s); !aiter.Done(); aiter.Next())
+    for (ArcIterator<Fst<A>> aiter(ifst, s); !aiter.Done(); aiter.Next())
       ofst->AddArc(s, (*mapper)(aiter.Value()));
 
     switch (final_action) {
@@ -249,7 +226,7 @@ void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C* mapper) {
       default: {
         B final_arc = (*mapper)(A(0, 0, ifst.Final(s), kNoStateId));
         if (final_arc.ilabel != 0 || final_arc.olabel != 0) {
-          FSTERROR() << "ArcMap: non-zero arc labels for superfinal arc";
+          FSTERROR() << "ArcMap: Non-zero arc labels for superfinal arc";
           ofst->SetProperties(kError, kError);
         }
         ofst->SetFinal(s, final_arc.weight);
@@ -258,7 +235,7 @@ void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C* mapper) {
       case MAP_ALLOW_SUPERFINAL: {
         B final_arc = (*mapper)(A(0, 0, ifst.Final(s), kNoStateId));
         if (final_arc.ilabel != 0 || final_arc.olabel != 0) {
-            // Add a superfinal state if not already done.
+          // Add a superfinal state if not already done.
           if (superfinal == kNoStateId) {
             superfinal = ofst->AddState();
             ofst->SetFinal(superfinal, B::Weight::One());
@@ -289,22 +266,21 @@ void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C* mapper) {
 // Maps an arc type A to an arc type B using mapper function
 // object C, passed by value. This version writes the mapped input
 // Fst to an output MutableFst.
-template<class A, class B, class C>
+template <class A, class B, class C>
 void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C mapper) {
   ArcMap(ifst, ofst, &mapper);
 }
 
-
 struct ArcMapFstOptions : public CacheOptions {
   // ArcMapFst default caching behaviour is to do no caching. Most
   // mappers are cheap and therefore we save memory by not doing
   // caching.
   ArcMapFstOptions() : CacheOptions(true, 0) {}
-  ArcMapFstOptions(const CacheOptions& opts) : CacheOptions(opts) {}
+  ArcMapFstOptions(const CacheOptions &opts) : CacheOptions(opts) {}
 };
 
-
-template <class A, class B, class C> class ArcMapFst;
+template <class A, class B, class C>
+class ArcMapFst;
 
 // Implementation of delayed ArcMapFst.
 template <class A, class B, class C>
@@ -323,14 +299,14 @@ class ArcMapFstImpl : public CacheImpl<B> {
   using CacheImpl<B>::SetFinal;
   using CacheImpl<B>::SetStart;
 
-  friend class StateIterator< ArcMapFst<A, B, C> >;
+  friend class StateIterator<ArcMapFst<A, B, C>>;
 
   typedef B Arc;
   typedef typename B::Weight Weight;
   typedef typename B::StateId StateId;
 
   ArcMapFstImpl(const Fst<A> &fst, const C &mapper,
-                 const ArcMapFstOptions& opts)
+                const ArcMapFstOptions &opts)
       : CacheImpl<B>(opts),
         fst_(fst.Copy()),
         mapper_(new C(mapper)),
@@ -340,8 +316,7 @@ class ArcMapFstImpl : public CacheImpl<B> {
     Init();
   }
 
-  ArcMapFstImpl(const Fst<A> &fst, C *mapper,
-                 const ArcMapFstOptions& opts)
+  ArcMapFstImpl(const Fst<A> &fst, C *mapper, const ArcMapFstOptions &opts)
       : CacheImpl<B>(opts),
         fst_(fst.Copy()),
         mapper_(mapper),
@@ -361,14 +336,13 @@ class ArcMapFstImpl : public CacheImpl<B> {
     Init();
   }
 
-  ~ArcMapFstImpl() {
+  ~ArcMapFstImpl() override {
     delete fst_;
     if (own_mapper_) delete mapper_;
   }
 
   StateId Start() {
-    if (!HasStart())
-      SetStart(FindOState(fst_->Start()));
+    if (!HasStart()) SetStart(FindOState(fst_->Start()));
     return CacheImpl<B>::Start();
   }
 
@@ -377,10 +351,10 @@ class ArcMapFstImpl : public CacheImpl<B> {
       switch (final_action_) {
         case MAP_NO_SUPERFINAL:
         default: {
-          B final_arc = (*mapper_)(A(0, 0, fst_->Final(FindIState(s)),
-                                        kNoStateId));
+          B final_arc =
+              (*mapper_)(A(0, 0, fst_->Final(FindIState(s)), kNoStateId));
           if (final_arc.ilabel != 0 || final_arc.olabel != 0) {
-            FSTERROR() << "ArcMapFst: non-zero arc labels for superfinal arc";
+            FSTERROR() << "ArcMapFst: Non-zero arc labels for superfinal arc";
             SetProperties(kError, kError);
           }
           SetFinal(s, final_arc.weight);
@@ -390,8 +364,8 @@ class ArcMapFstImpl : public CacheImpl<B> {
           if (s == superfinal_) {
             SetFinal(s, Weight::One());
           } else {
-            B final_arc = (*mapper_)(A(0, 0, fst_->Final(FindIState(s)),
-                                          kNoStateId));
+            B final_arc =
+                (*mapper_)(A(0, 0, fst_->Final(FindIState(s)), kNoStateId));
             if (final_arc.ilabel == 0 && final_arc.olabel == 0)
               SetFinal(s, final_arc.weight);
             else
@@ -409,78 +383,75 @@ class ArcMapFstImpl : public CacheImpl<B> {
   }
 
   size_t NumArcs(StateId s) {
-    if (!HasArcs(s))
-      Expand(s);
+    if (!HasArcs(s)) Expand(s);
     return CacheImpl<B>::NumArcs(s);
   }
 
   size_t NumInputEpsilons(StateId s) {
-    if (!HasArcs(s))
-      Expand(s);
+    if (!HasArcs(s)) Expand(s);
     return CacheImpl<B>::NumInputEpsilons(s);
   }
 
   size_t NumOutputEpsilons(StateId s) {
-    if (!HasArcs(s))
-      Expand(s);
+    if (!HasArcs(s)) Expand(s);
     return CacheImpl<B>::NumOutputEpsilons(s);
   }
 
-  uint64 Properties() const { return Properties(kFstProperties); }
+  uint64 Properties() const override { return Properties(kFstProperties); }
 
   // Set error if found; return FST impl properties.
-  uint64 Properties(uint64 mask) const {
-    if ((mask & kError) && (fst_->Properties(kError, false) ||
-                            (mapper_->Properties(0) & kError)))
+  uint64 Properties(uint64 mask) const override {
+    if ((mask & kError) &&
+        (fst_->Properties(kError, false) || (mapper_->Properties(0) & kError)))
       SetProperties(kError, kError);
     return FstImpl<Arc>::Properties(mask);
   }
 
   void InitArcIterator(StateId s, ArcIteratorData<B> *data) {
-    if (!HasArcs(s))
-      Expand(s);
+    if (!HasArcs(s)) Expand(s);
     CacheImpl<B>::InitArcIterator(s, data);
   }
 
   void Expand(StateId s) {
     // Add exiting arcs.
-    if (s == superfinal_) { SetArcs(s); return; }
+    if (s == superfinal_) {
+      SetArcs(s);
+      return;
+    }
 
-    for (ArcIterator< Fst<A> > aiter(*fst_, FindIState(s));
-         !aiter.Done(); aiter.Next()) {
+    for (ArcIterator<Fst<A>> aiter(*fst_, FindIState(s)); !aiter.Done();
+         aiter.Next()) {
       A aarc(aiter.Value());
       aarc.nextstate = FindOState(aarc.nextstate);
-      const B& barc = (*mapper_)(aarc);
+      const B &barc = (*mapper_)(aarc);
       PushArc(s, barc);
     }
 
     // Check for superfinal arcs.
-    if (!HasFinal(s) || Final(s) == Weight::Zero())
-      switch (final_action_) {
+    if (!HasFinal(s) || Final(s) == Weight::Zero()) switch (final_action_) {
         case MAP_NO_SUPERFINAL:
         default:
           break;
         case MAP_ALLOW_SUPERFINAL: {
-          B final_arc = (*mapper_)(A(0, 0, fst_->Final(FindIState(s)),
-                                        kNoStateId));
+          B final_arc =
+              (*mapper_)(A(0, 0, fst_->Final(FindIState(s)), kNoStateId));
           if (final_arc.ilabel != 0 || final_arc.olabel != 0) {
-            if (superfinal_ == kNoStateId)
-              superfinal_ = nstates_++;
+            if (superfinal_ == kNoStateId) superfinal_ = nstates_++;
             final_arc.nextstate = superfinal_;
             PushArc(s, final_arc);
           }
           break;
         }
-      case MAP_REQUIRE_SUPERFINAL: {
-        B final_arc = (*mapper_)(A(0, 0, fst_->Final(FindIState(s)),
-                                      kNoStateId));
-        if (final_arc.ilabel != 0 || final_arc.olabel != 0 ||
-            final_arc.weight != B::Weight::Zero())
-          PushArc(s, B(final_arc.ilabel, final_arc.olabel,
-                      final_arc.weight, superfinal_));
-        break;
+        case MAP_REQUIRE_SUPERFINAL: {
+          B final_arc =
+              (*mapper_)(A(0, 0, fst_->Final(FindIState(s)), kNoStateId));
+          if (final_arc.ilabel != 0 || final_arc.olabel != 0 ||
+              final_arc.weight != B::Weight::Zero())
+            PushArc(s, B(final_arc.ilabel, final_arc.olabel, final_arc.weight,
+                         superfinal_));
+          break;
+        }
       }
-    }
     SetArcs(s);
   }
 
@@ -505,8 +476,7 @@ class ArcMapFstImpl : public CacheImpl<B> {
       final_action_ = mapper_->FinalAction();
       uint64 props = fst_->Properties(kCopyProperties, false);
       SetProperties(mapper_->Properties(props));
-      if (final_action_ == MAP_REQUIRE_SUPERFINAL)
-        superfinal_ = 0;
+      if (final_action_ == MAP_REQUIRE_SUPERFINAL) superfinal_ = 0;
     }
   }
 
@@ -526,15 +496,13 @@ class ArcMapFstImpl : public CacheImpl<B> {
     else
       os = is + 1;
 
-    if (os >= nstates_)
-      nstates_ = os + 1;
+    if (os >= nstates_) nstates_ = os + 1;
 
     return os;
   }
 
-
   const Fst<A> *fst_;
-  C*   mapper_;
+  C *mapper_;
   bool own_mapper_;
   MapFinalAction final_action_;
 
@@ -544,14 +512,13 @@ class ArcMapFstImpl : public CacheImpl<B> {
   void operator=(const ArcMapFstImpl<A, B, C> &);  // disallow
 };
 
-
 // Maps an arc type A to an arc type B using Mapper function object
 // C. This version is a delayed Fst.
 template <class A, class B, class C>
-class ArcMapFst : public ImplToFst< ArcMapFstImpl<A, B, C> > {
+class ArcMapFst : public ImplToFst<ArcMapFstImpl<A, B, C>> {
  public:
-  friend class ArcIterator< ArcMapFst<A, B, C> >;
-  friend class StateIterator< ArcMapFst<A, B, C> >;
+  friend class ArcIterator<ArcMapFst<A, B, C>>;
+  friend class StateIterator<ArcMapFst<A, B, C>>;
 
   typedef B Arc;
   typedef typename B::Weight Weight;
@@ -560,51 +527,56 @@ class ArcMapFst : public ImplToFst< ArcMapFstImpl<A, B, C> > {
   typedef typename Store::State State;
   typedef ArcMapFstImpl<A, B, C> Impl;
 
-  ArcMapFst(const Fst<A> &fst, const C &mapper, const ArcMapFstOptions& opts)
-      : ImplToFst<Impl>(new Impl(fst, mapper, opts)) {}
+  ArcMapFst(const Fst<A> &fst, const C &mapper, const ArcMapFstOptions &opts)
+      : ImplToFst<Impl>(std::make_shared<Impl>(fst, mapper, opts)) {}
 
-  ArcMapFst(const Fst<A> &fst, C* mapper, const ArcMapFstOptions& opts)
-      : ImplToFst<Impl>(new Impl(fst, mapper, opts)) {}
+  ArcMapFst(const Fst<A> &fst, C *mapper, const ArcMapFstOptions &opts)
+      : ImplToFst<Impl>(std::make_shared<Impl>(fst, mapper, opts)) {}
 
   ArcMapFst(const Fst<A> &fst, const C &mapper)
-      : ImplToFst<Impl>(new Impl(fst, mapper, ArcMapFstOptions())) {}
+      : ImplToFst<Impl>(
+            std::make_shared<Impl>(fst, mapper, ArcMapFstOptions())) {}
 
-  ArcMapFst(const Fst<A> &fst, C* mapper)
-      : ImplToFst<Impl>(new Impl(fst, mapper, ArcMapFstOptions())) {}
+  ArcMapFst(const Fst<A> &fst, C *mapper)
+      : ImplToFst<Impl>(
+            std::make_shared<Impl>(fst, mapper, ArcMapFstOptions())) {}
 
   // See Fst<>::Copy() for doc.
   ArcMapFst(const ArcMapFst<A, B, C> &fst, bool safe = false)
-    : ImplToFst<Impl>(fst, safe) {}
+      : ImplToFst<Impl>(fst, safe) {}
 
   // Get a copy of this ArcMapFst. See Fst<>::Copy() for further doc.
-  virtual ArcMapFst<A, B, C> *Copy(bool safe = false) const {
+  ArcMapFst<A, B, C> *Copy(bool safe = false) const override {
     return new ArcMapFst<A, B, C>(*this, safe);
   }
 
-  virtual inline void InitStateIterator(StateIteratorData<B> *data) const;
+  inline void InitStateIterator(StateIteratorData<B> *data) const override;
 
-  virtual void InitArcIterator(StateId s, ArcIteratorData<B> *data) const {
-    GetImpl()->InitArcIterator(s, data);
+  void InitArcIterator(StateId s, ArcIteratorData<B> *data) const override {
+    GetMutableImpl()->InitArcIterator(s, data);
   }
 
- private:
-  // Makes visible to friends.
-  Impl *GetImpl() const { return ImplToFst<Impl>::GetImpl(); }
+ protected:
+  using ImplToFst<Impl>::GetImpl;
+  using ImplToFst<Impl>::GetMutableImpl;
 
+ private:
   void operator=(const ArcMapFst<A, B, C> &fst);  // disallow
 };
 
-
 // Specialization for ArcMapFst.
-template<class A, class B, class C>
-class StateIterator< ArcMapFst<A, B, C> > : public StateIteratorBase<B> {
+template <class A, class B, class C>
+class StateIterator<ArcMapFst<A, B, C>> : public StateIteratorBase<B> {
  public:
   typedef typename B::StateId StateId;
 
   explicit StateIterator(const ArcMapFst<A, B, C> &fst)
-      : impl_(fst.GetImpl()), siter_(*impl_->fst_), s_(0),
-        superfinal_(impl_->final_action_ == MAP_REQUIRE_SUPERFINAL)
-  { CheckSuperfinal(); }
+      : impl_(fst.GetImpl()),
+        siter_(*impl_->fst_),
+        s_(0),
+        superfinal_(impl_->final_action_ == MAP_REQUIRE_SUPERFINAL) {
+    CheckSuperfinal();
+  }
 
   bool Done() const { return siter_.Done() && !superfinal_; }
 
@@ -615,9 +587,9 @@ class StateIterator< ArcMapFst<A, B, C> > : public StateIteratorBase<B> {
     if (!siter_.Done()) {
       siter_.Next();
       CheckSuperfinal();
-    }
-    else if (superfinal_)
+    } else if (superfinal_) {
       superfinal_ = false;
+    }
   }
 
   void Reset() {
@@ -631,55 +603,50 @@ class StateIterator< ArcMapFst<A, B, C> > : public StateIteratorBase<B> {
   // This allows base-class virtual access to non-virtual derived-
   // class members of the same name. It makes the derived class more
   // efficient to use but unsafe to further derive.
-  bool Done_() const { return Done(); }
-  StateId Value_() const { return Value(); }
-  void Next_() { Next(); }
-  void Reset_() { Reset(); }
+  bool Done_() const override { return Done(); }
+  StateId Value_() const override { return Value(); }
+  void Next_() override { Next(); }
+  void Reset_() override { Reset(); }
 
   void CheckSuperfinal() {
-    if (impl_->final_action_ != MAP_ALLOW_SUPERFINAL || superfinal_)
-      return;
+    if (impl_->final_action_ != MAP_ALLOW_SUPERFINAL || superfinal_) return;
     if (!siter_.Done()) {
-      B final_arc = (*impl_->mapper_)(A(0, 0, impl_->fst_->Final(s_),
-                                           kNoStateId));
-      if (final_arc.ilabel != 0 || final_arc.olabel != 0)
-        superfinal_ = true;
+      B final_arc =
+          (*impl_->mapper_)(A(0, 0, impl_->fst_->Final(s_), kNoStateId));
+      if (final_arc.ilabel != 0 || final_arc.olabel != 0) superfinal_ = true;
     }
   }
 
   const ArcMapFstImpl<A, B, C> *impl_;
-  StateIterator< Fst<A> > siter_;
+  StateIterator<Fst<A>> siter_;
   StateId s_;
-  bool superfinal_;    // true if there is a superfinal state and not done
+  bool superfinal_;  // true if there is a superfinal state and not done
 
   DISALLOW_COPY_AND_ASSIGN(StateIterator);
 };
 
-
 // Specialization for ArcMapFst.
 template <class A, class B, class C>
-class ArcIterator< ArcMapFst<A, B, C> >
-    : public CacheArcIterator< ArcMapFst<A, B, C> > {
+class ArcIterator<ArcMapFst<A, B, C>>
+    : public CacheArcIterator<ArcMapFst<A, B, C>> {
  public:
   typedef typename A::StateId StateId;
 
   ArcIterator(const ArcMapFst<A, B, C> &fst, StateId s)
-      : CacheArcIterator< ArcMapFst<A, B, C> >(fst.GetImpl(), s) {
-    if (!fst.GetImpl()->HasArcs(s))
-      fst.GetImpl()->Expand(s);
+      : CacheArcIterator<ArcMapFst<A, B, C>>(fst.GetMutableImpl(), s) {
+    if (!fst.GetImpl()->HasArcs(s)) fst.GetMutableImpl()->Expand(s);
   }
 
  private:
   DISALLOW_COPY_AND_ASSIGN(ArcIterator);
 };
 
-template <class A, class B, class C> inline
-void ArcMapFst<A, B, C>::InitStateIterator(StateIteratorData<B> *data)
-    const {
-  data->base = new StateIterator< ArcMapFst<A, B, C> >(*this);
+template <class A, class B, class C>
+inline void ArcMapFst<A, B, C>::InitStateIterator(
+    StateIteratorData<B> *data) const {
+  data->base = new StateIterator<ArcMapFst<A, B, C>>(*this);
 }
 
-
 //
 // Utility Mappers
 //
@@ -696,11 +663,52 @@ struct IdentityArcMapper {
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const { return props; }
 };
 
+// Mapper that converts all input symbols to epsilon.
+template <class A>
+struct InputEpsilonMapper {
+  typedef A FromArc;
+  typedef A ToArc;
+
+  A operator()(const A &arc) const {
+    return ToArc(0, arc.olabel, arc.weight, arc.nextstate);
+  }
+
+  MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
+
+  MapSymbolsAction InputSymbolsAction() const { return MAP_CLEAR_SYMBOLS; }
+
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
+
+  uint64 Properties(uint64 props) const {
+    return (props & kSetArcProperties) | kIEpsilons;
+  }
+};
+
+// Mapper that converts all output symbols to epsilon.
+template <class A>
+struct OutputEpsilonMapper {
+  typedef A FromArc;
+  typedef A ToArc;
+
+  A operator()(const A &arc) const {
+    return ToArc(arc.ilabel, 0, arc.weight, arc.nextstate);
+  }
+
+  MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
+
+  MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
+
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_CLEAR_SYMBOLS; }
+
+  uint64 Properties(uint64 props) const {
+    return (props & kSetArcProperties) | kOEpsilons;
+  }
+};
 
 // Mapper that returns its input with final states redirected to
 // a single super-final state.
@@ -715,14 +723,13 @@ struct SuperFinalMapper {
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const {
     return props & kAddSuperFinalProperties;
   }
 };
 
-
 // Mapper that leaves labels and nextstate unchanged and constructs a new weight
 // from the underlying value of the arc weight.  Requires that there is a
 // WeightConvert class specialization that converts the weights.
@@ -735,15 +742,15 @@ class WeightConvertMapper {
   typedef typename ToArc::Weight ToWeight;
 
   ToArc operator()(const FromArc &arc) const {
-    return ToArc(arc.ilabel, arc.olabel,
-                 convert_weight_(arc.weight), arc.nextstate);
+    return ToArc(arc.ilabel, arc.olabel, convert_weight_(arc.weight),
+                 arc.nextstate);
   }
 
   MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const { return props; }
 
@@ -778,29 +785,28 @@ struct ToGallicMapper {
       return ToArc(0, 0, GW(SW::One(), arc.weight), kNoStateId);
     // 'Super-non-final' arc.
     else if (arc.nextstate == kNoStateId)
-      return ToArc(0, 0, GW(SW::Zero(), arc.weight), kNoStateId);
+      return ToArc(0, 0, GW::Zero(), kNoStateId);
     // Epsilon label.
     else if (arc.olabel == 0)
-      return ToArc(arc.ilabel, arc.ilabel,
-                   GW(SW::One(), arc.weight), arc.nextstate);
+      return ToArc(arc.ilabel, arc.ilabel, GW(SW::One(), arc.weight),
+                   arc.nextstate);
     // Regular label.
     else
-      return ToArc(arc.ilabel, arc.ilabel,
-                   GW(SW(arc.olabel), arc.weight), arc.nextstate);
+      return ToArc(arc.ilabel, arc.ilabel, GW(SW(arc.olabel), arc.weight),
+                   arc.nextstate);
   }
 
   MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_CLEAR_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_CLEAR_SYMBOLS; }
 
   uint64 Properties(uint64 props) const {
     return ProjectProperties(props, true) & kWeightInvariantProperties;
   }
 };
 
-
 // Mapper from GallicArc<A> to A.
 template <class A, GallicType G = GALLIC_LEFT>
 struct FromGallicMapper {
@@ -808,7 +814,6 @@ struct FromGallicMapper {
   typedef A ToArc;
 
   typedef typename A::Label Label;
-  typedef StringWeight<Label, GALLIC_STRING_TYPE(G)> SW;
   typedef typename A::Weight AW;
   typedef typename GallicArc<A, G>::Weight GW;
 
@@ -820,44 +825,67 @@ struct FromGallicMapper {
     if (arc.nextstate == kNoStateId && arc.weight == GW::Zero())
       return A(arc.ilabel, 0, AW::Zero(), kNoStateId);
 
-    SW w1 = arc.weight.Value1();
-    AW w2 = arc.weight.Value2();
-    StringWeightIterator<Label, GALLIC_STRING_TYPE(G)> iter1(w1);
-
-    Label l = w1.Size() == 1 ? iter1.Value() : 0;
-
-    if (l == kStringInfinity || l == kStringBad ||
-        arc.ilabel != arc.olabel || w1.Size() > 1) {
-      FSTERROR() << "FromGallicMapper: unrepresentable weight";
+    Label l = kNoLabel;
+    AW w;
+    if (!Extract(arc.weight, &w, &l) || arc.ilabel != arc.olabel) {
+      FSTERROR() << "FromGallicMapper: Unrepresentable weight: " << arc.weight
+                 << " for arc with ilabel = " << arc.ilabel
+                 << ", olabel = " << arc.olabel
+                 << ", nextstate = " << arc.nextstate;
       error_ = true;
     }
 
     if (arc.ilabel == 0 && l != 0 && arc.nextstate == kNoStateId)
-      return A(superfinal_label_, l, w2, arc.nextstate);
+      return A(superfinal_label_, l, w, arc.nextstate);
     else
-      return A(arc.ilabel, l, w2, arc.nextstate);
+      return A(arc.ilabel, l, w, arc.nextstate);
   }
 
   MapFinalAction FinalAction() const { return MAP_ALLOW_SUPERFINAL; }
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_CLEAR_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_CLEAR_SYMBOLS; }
 
   uint64 Properties(uint64 inprops) const {
     uint64 outprops = inprops & kOLabelInvariantProperties &
-        kWeightInvariantProperties & kAddSuperFinalProperties;
-    if (error_)
-      outprops |= kError;
+                      kWeightInvariantProperties & kAddSuperFinalProperties;
+    if (error_) outprops |= kError;
     return outprops;
   }
 
+ private:
+  template <GallicType GT>
+  static bool Extract(const GallicWeight<Label, AW, GT> &gallic_weight,
+                      typename A::Weight *weight, typename A::Label *label) {
+    StringWeight<Label, GALLIC_STRING_TYPE(GT)> w1 = gallic_weight.Value1();
+    AW w2 = gallic_weight.Value2();
+    StringWeightIterator<Label, GALLIC_STRING_TYPE(GT)> iter1(w1);
+
+    Label l = w1.Size() == 1 ? iter1.Value() : 0;
+    if (l == kStringInfinity || l == kStringBad || w1.Size() > 1) return false;
+
+    *label = l;
+    *weight = w2;
+    return true;
+  }
+
+  static bool Extract(const GallicWeight<Label, AW, GALLIC> &gallic_weight,
+                      typename A::Weight *weight, typename A::Label *label) {
+    if (gallic_weight.Size() > 1) return false;
+    if (gallic_weight.Size() == 0) {
+      *label = 0;
+      *weight = A::Weight::Zero();
+      return true;
+    }
+    return Extract<GALLIC_RESTRICT>(gallic_weight.Back(), weight, label);
+  }
+
  private:
   Label superfinal_label_;
   mutable bool error_;
 };
 
-
 // Mapper from GallicArc<A> to A.
 template <class A, GallicType G = GALLIC_LEFT>
 struct GallicToNewSymbolsMapper {
@@ -871,8 +899,11 @@ struct GallicToNewSymbolsMapper {
   typedef typename GallicArc<A, G>::Weight GW;
 
   GallicToNewSymbolsMapper(MutableFst<ToArc> *fst)
-      : fst_(fst), lmax_(0), osymbols_(fst->OutputSymbols()),
-        isymbols_(0), error_(false) {
+      : fst_(fst),
+        lmax_(0),
+        osymbols_(fst->OutputSymbols()),
+        isymbols_(0),
+        error_(false) {
     fst_->DeleteStates();
     state_ = fst_->AddState();
     fst_->SetStart(state_);
@@ -881,7 +912,7 @@ struct GallicToNewSymbolsMapper {
       string name = osymbols_->Name() + "_from_gallic";
       fst_->SetInputSymbols(new SymbolTable(name));
       isymbols_ = fst_->MutableInputSymbols();
-      isymbols_->AddSymbol(osymbols_->Find((int64) 0), 0);
+      isymbols_->AddSymbol(osymbols_->Find((int64)0), 0);
     } else {
       fst_->SetInputSymbols(0);
     }
@@ -899,18 +930,17 @@ struct GallicToNewSymbolsMapper {
     if (w1.Size() == 0) {
       l = 0;
     } else {
-      typename Map::iterator miter = map_.find(w1);
+      auto miter = map_.find(w1);
       if (miter != map_.end()) {
         l = (*miter).second;
       } else {
         l = ++lmax_;
-        map_.insert(pair<const SW, Label>(w1, l));
+        map_.insert(std::pair<const SW, Label>(w1, l));
         StringWeightIterator<Label, GALLIC_STRING_TYPE(G)> iter1(w1);
         StateId n;
         string s;
-        for(size_t i = 0, p = state_;
-            i < w1.Size();
-            ++i, iter1.Next(), p = n) {
+        for (size_t i = 0, p = state_; i < w1.Size();
+             ++i, iter1.Next(), p = n) {
           n = i == w1.Size() - 1 ? state_ : fst_->AddState();
           fst_->AddArc(p, ToArc(i ? 0 : l, iter1.Value(), AW::One(), n));
           if (isymbols_) {
@@ -918,13 +948,12 @@ struct GallicToNewSymbolsMapper {
             s = s + osymbols_->Find(iter1.Value());
           }
         }
-        if (isymbols_)
-          isymbols_->AddSymbol(s, l);
+        if (isymbols_) isymbols_->AddSymbol(s, l);
       }
     }
 
     if (l == kStringInfinity || l == kStringBad || arc.ilabel != arc.olabel) {
-      FSTERROR() << "GallicToNewSymbolMapper: unrepresentable weight";
+      FSTERROR() << "GallicToNewSymbolMapper: Unrepresentable weight: " << l;
       error_ = true;
     }
 
@@ -939,21 +968,18 @@ struct GallicToNewSymbolsMapper {
 
   uint64 Properties(uint64 inprops) const {
     uint64 outprops = inprops & kOLabelInvariantProperties &
-        kWeightInvariantProperties & kAddSuperFinalProperties;
-    if (error_)
-      outprops |= kError;
+                      kWeightInvariantProperties & kAddSuperFinalProperties;
+    if (error_) outprops |= kError;
     return outprops;
   }
 
  private:
   class StringKey {
    public:
-    size_t operator()(const SW &x) const {
-      return x.Hash();
-    }
+    size_t operator()(const SW &x) const { return x.Hash(); }
   };
 
-  typedef unordered_map<SW, Label, StringKey> Map;
+  typedef std::unordered_map<SW, Label, StringKey> Map;
 
   MutableFst<ToArc> *fst_;
   Map map_;
@@ -966,7 +992,6 @@ struct GallicToNewSymbolsMapper {
   DISALLOW_COPY_AND_ASSIGN(GallicToNewSymbolsMapper);
 };
 
-
 // Mapper to add a constant to all weights.
 template <class A>
 struct PlusMapper {
@@ -977,8 +1002,7 @@ struct PlusMapper {
   explicit PlusMapper(Weight w) : weight_(w) {}
 
   A operator()(const A &arc) const {
-    if (arc.weight == Weight::Zero())
-      return arc;
+    if (arc.weight == Weight::Zero()) return arc;
     Weight w = Plus(arc.weight, weight_);
     return A(arc.ilabel, arc.olabel, w, arc.nextstate);
   }
@@ -987,20 +1011,16 @@ struct PlusMapper {
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
   }
 
  private:
-
-
-
   Weight weight_;
 };
 
-
 // Mapper to (right) multiply a constant to all weights.
 template <class A>
 struct TimesMapper {
@@ -1011,8 +1031,7 @@ struct TimesMapper {
   explicit TimesMapper(Weight w) : weight_(w) {}
 
   A operator()(const A &arc) const {
-    if (arc.weight == Weight::Zero())
-      return arc;
+    if (arc.weight == Weight::Zero()) return arc;
     Weight w = Times(arc.weight, weight_);
     return A(arc.ilabel, arc.olabel, w, arc.nextstate);
   }
@@ -1021,7 +1040,7 @@ struct TimesMapper {
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
@@ -1031,7 +1050,6 @@ struct TimesMapper {
   Weight weight_;
 };
 
-
 // Mapper to reciprocate all non-Zero() weights.
 template <class A>
 struct InvertWeightMapper {
@@ -1040,8 +1058,7 @@ struct InvertWeightMapper {
   typedef typename A::Weight Weight;
 
   A operator()(const A &arc) const {
-    if (arc.weight == Weight::Zero())
-      return arc;
+    if (arc.weight == Weight::Zero()) return arc;
     Weight w = Divide(Weight::One(), arc.weight);
     return A(arc.ilabel, arc.olabel, w, arc.nextstate);
   }
@@ -1050,14 +1067,13 @@ struct InvertWeightMapper {
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
   }
 };
 
-
 // Mapper to map all non-Zero() weights to One().
 template <class A, class B = A>
 struct RmWeightMapper {
@@ -1067,8 +1083,8 @@ struct RmWeightMapper {
   typedef typename ToArc::Weight ToWeight;
 
   B operator()(const A &arc) const {
-    ToWeight w = arc.weight != FromWeight::Zero() ?
-                   ToWeight::One() : ToWeight::Zero();
+    ToWeight w =
+        arc.weight != FromWeight::Zero() ? ToWeight::One() : ToWeight::Zero();
     return B(arc.ilabel, arc.olabel, w, arc.nextstate);
   }
 
@@ -1076,14 +1092,13 @@ struct RmWeightMapper {
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const {
     return (props & kWeightInvariantProperties) | kUnweighted;
   }
 };
 
-
 // Mapper to quantize all weights.
 template <class A, class B = A>
 struct QuantizeMapper {
@@ -1105,7 +1120,7 @@ struct QuantizeMapper {
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
@@ -1115,7 +1130,6 @@ struct QuantizeMapper {
   float delta_;
 };
 
-
 // Mapper from A to B under the assumption:
 //    B::Weight = A::Weight::ReverseWeight
 //    B::Label == A::Label
@@ -1135,11 +1149,11 @@ struct ReverseWeightMapper {
 
   MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS;}
+  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
 
   uint64 Properties(uint64 props) const { return props; }
 };
 
 }  // namespace fst
 
-#endif  // FST_LIB_ARC_MAP_H__
+#endif  // FST_LIB_ARC_MAP_H_
