@@ -17,10 +17,8 @@
 // \file
 // Google compatibility declarations and inline definitions.
 
-#ifndef FST_LIB_COMPAT_H__
-#define FST_LIB_COMPAT_H__
-
-#include <dlfcn.h>
+#ifndef FST_LIB_COMPAT_H_
+#define FST_LIB_COMPAT_H_
 
 #include <climits>
 #include <cstdlib>
@@ -41,18 +39,12 @@
 #include <fst/log.h>
 #include <fst/icu.h>
 
-using std::cin;
-using std::cout;
-using std::cerr;
-using std::endl;
 using std::string;
 
 void FailedNewHandler();
 
 namespace fst {
 
-using namespace std;
-
 // Downcasting
 template<typename To, typename From>
 inline To down_cast(From* f) {
@@ -109,4 +101,4 @@ class CheckSummer {
 
 }  // namespace fst
 
-#endif  // FST_LIB_COMPAT_H__
+#endif  // FST_LIB_COMPAT_H_
