@@ -1,36 +1,23 @@
-// complement.h
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
+//
+// Class to complement an FST.
 
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
-// Class to complement an Fst.
-
-#ifndef FST_LIB_COMPLEMENT_H__
-#define FST_LIB_COMPLEMENT_H__
+#ifndef FST_LIB_COMPLEMENT_H_
+#define FST_LIB_COMPLEMENT_H_
 
 #include <algorithm>
 #include <string>
 #include <vector>
-using std::vector;
+
 #include <fst/fst.h>
 #include <fst/test-properties.h>
 
+
 namespace fst {
 
-template <class A> class ComplementFst;
+template <class A>
+class ComplementFst;
 
 // Implementation of delayed ComplementFst. The algorithm used
 // completes the (deterministic) FSA and then exchanges final and
@@ -47,12 +34,11 @@ class ComplementFstImpl : public FstImpl<A> {
  public:
   using FstImpl<A>::SetType;
   using FstImpl<A>::SetProperties;
-  using FstImpl<A>::Properties;
   using FstImpl<A>::SetInputSymbols;
   using FstImpl<A>::SetOutputSymbols;
 
-  friend class StateIterator< ComplementFst<A> >;
-  friend class ArcIterator< ComplementFst<A> >;
+  friend class StateIterator<ComplementFst<A>>;
+  friend class ArcIterator<ComplementFst<A>>;
 
   typedef A Arc;
   typedef typename A::Label Label;
@@ -75,9 +61,11 @@ class ComplementFstImpl : public FstImpl<A> {
     SetOutputSymbols(impl.OutputSymbols());
   }
 
-  ~ComplementFstImpl() { delete fst_; }
+  ~ComplementFstImpl() override { delete fst_; }
 
   StateId Start() const {
+    if (Properties(kError)) return kNoStateId;
+
     StateId start = fst_->Start();
     if (start != kNoStateId)
       return start + 1;
@@ -108,13 +96,21 @@ class ComplementFstImpl : public FstImpl<A> {
     return s == 0 ? 0 : fst_->NumOutputEpsilons(s - 1);
   }
 
+  uint64 Properties() const override { return Properties(kFstProperties); }
+
+  // Set error if found; return FST impl properties.
+  uint64 Properties(uint64 mask) const override {
+    if ((mask & kError) && fst_->Properties(kError, false))
+      SetProperties(kError, kError);
+    return FstImpl<Arc>::Properties(mask);
+  }
+
  private:
   const Fst<A> *fst_;
 
   void operator=(const ComplementFstImpl<A> &fst);  // Disallow
 };
 
-
 // Complements an automaton. This is a library-internal operation that
 // introduces a (negative) 'rho' label; use Difference/DifferenceFst in
 // user code, which will not see this label. This version is a delayed Fst.
@@ -122,10 +118,10 @@ class ComplementFstImpl : public FstImpl<A> {
 // This class attaches interface to implementation and handles
 // reference counting, delegating most methods to ImplToFst.
 template <class A>
-class ComplementFst : public ImplToFst< ComplementFstImpl<A> > {
+class ComplementFst : public ImplToFst<ComplementFstImpl<A>> {
  public:
-  friend class StateIterator< ComplementFst<A> >;
-  friend class ArcIterator< ComplementFst<A> >;
+  friend class StateIterator<ComplementFst<A>>;
+  friend class ArcIterator<ComplementFst<A>>;
 
   typedef A Arc;
   typedef typename A::StateId StateId;
@@ -133,11 +129,13 @@ class ComplementFst : public ImplToFst< ComplementFstImpl<A> > {
   typedef ComplementFstImpl<A> Impl;
 
   explicit ComplementFst(const Fst<A> &fst)
-      : ImplToFst<Impl>(new Impl(fst)) {
+      : ImplToFst<Impl>(std::make_shared<Impl>(fst)) {
     uint64 props = kUnweighted | kNoEpsilons | kIDeterministic | kAcceptor;
-    if (fst.Properties(props, true) != props)
-      LOG(FATAL) << "ComplementFst: argument not an unweighted"
-                 << " epsilon-free deterministic acceptor";
+    if (fst.Properties(props, true) != props) {
+      FSTERROR() << "ComplementFst: Argument not an unweighted "
+                 << "epsilon-free deterministic acceptor";
+      GetImpl()->SetProperties(kError, kError);
+    }
   }
 
   // See Fst<>::Copy() for doc.
@@ -145,47 +143,45 @@ class ComplementFst : public ImplToFst< ComplementFstImpl<A> > {
       : ImplToFst<Impl>(fst, safe) {}
 
   // Get a copy of this ComplementFst. See Fst<>::Copy() for further doc.
-  virtual ComplementFst<A> *Copy(bool safe = false) const {
+  ComplementFst<A> *Copy(bool safe = false) const override {
     return new ComplementFst<A>(*this, safe);
   }
 
-  virtual inline void InitStateIterator(StateIteratorData<A> *data) const;
+  inline void InitStateIterator(StateIteratorData<A> *data) const override;
 
-  virtual inline void InitArcIterator(StateId s,
-                                      ArcIteratorData<A> *data) const;
+  inline void InitArcIterator(StateId s,
+                              ArcIteratorData<A> *data) const override;
 
   // Label that represents the rho transition.
   // We use a negative value, which is thus private to the library and
   // which will preserve FST label sort order.
   static const Label kRhoLabel = -2;
+
  private:
-  // Makes visible to friends.
-  Impl *GetImpl() const { return ImplToFst<Impl>::GetImpl(); }
+  using ImplToFst<Impl>::GetImpl;
 
   void operator=(const ComplementFst<A> &fst);  // disallow
 };
 
-template <class A> const typename A::Label ComplementFst<A>::kRhoLabel;
-
+template <class A>
+const typename A::Label ComplementFst<A>::kRhoLabel;
 
 // Specialization for ComplementFst.
 template <class A>
-class StateIterator< ComplementFst<A> > : public StateIteratorBase<A> {
+class StateIterator<ComplementFst<A>> : public StateIteratorBase<A> {
  public:
   typedef typename A::StateId StateId;
   typedef typename A::Label Label;
 
   explicit StateIterator(const ComplementFst<A> &fst)
-      : siter_(*fst.GetImpl()->fst_), s_(0) {
-  }
+      : siter_(*fst.GetImpl()->fst_), s_(0) {}
 
   bool Done() const { return s_ > 0 && siter_.Done(); }
 
   StateId Value() const { return s_; }
 
   void Next() {
-    if (s_ != 0)
-      siter_.Next();
+    if (s_ != 0) siter_.Next();
     ++s_;
   }
 
@@ -198,21 +194,20 @@ class StateIterator< ComplementFst<A> > : public StateIteratorBase<A> {
   // This allows base class virtual access to non-virtual derived-
   // class members of the same name. It makes the derived class more
   // efficient to use but unsafe to further derive.
-  virtual bool Done_() const { return Done(); }
-  virtual StateId Value_() const { return Value(); }
-  virtual void Next_() { Next(); }
-  virtual void Reset_() { Reset(); }
+  bool Done_() const override { return Done(); }
+  StateId Value_() const override { return Value(); }
+  void Next_() override { Next(); }
+  void Reset_() override { Reset(); }
 
-  StateIterator< Fst<A> > siter_;
+  StateIterator<Fst<A>> siter_;
   StateId s_;
 
   DISALLOW_COPY_AND_ASSIGN(StateIterator);
 };
 
-
 // Specialization for ComplementFst.
 template <class A>
-class ArcIterator< ComplementFst<A> > : public ArcIteratorBase<A> {
+class ArcIterator<ComplementFst<A>> : public ArcIteratorBase<A> {
  public:
   typedef typename A::StateId StateId;
   typedef typename A::Label Label;
@@ -220,11 +215,10 @@ class ArcIterator< ComplementFst<A> > : public ArcIteratorBase<A> {
 
   ArcIterator(const ComplementFst<A> &fst, StateId s)
       : aiter_(0), s_(s), pos_(0) {
-    if (s_ != 0)
-      aiter_ = new ArcIterator< Fst<A> >(*fst.GetImpl()->fst_, s - 1);
+    if (s_ != 0) aiter_ = new ArcIterator<Fst<A>>(*fst.GetImpl()->fst_, s - 1);
   }
 
-  virtual ~ArcIterator() { delete aiter_; }
+  ~ArcIterator() override { delete aiter_; }
 
   bool Done() const {
     if (s_ != 0)
@@ -234,7 +228,7 @@ class ArcIterator< ComplementFst<A> > : public ArcIteratorBase<A> {
   }
 
   // Adds the rho label to the rho destination state.
-  const A& Value() const {
+  const A &Value() const {
     if (pos_ == 0) {
       arc_.ilabel = arc_.olabel = ComplementFst<A>::kRhoLabel;
       arc_.weight = Weight::One();
@@ -247,18 +241,14 @@ class ArcIterator< ComplementFst<A> > : public ArcIteratorBase<A> {
   }
 
   void Next() {
-    if (s_ != 0 && pos_ > 0)
-      aiter_->Next();
+    if (s_ != 0 && pos_ > 0) aiter_->Next();
     ++pos_;
   }
 
-  size_t Position() const {
-    return pos_;
-  }
+  size_t Position() const { return pos_; }
 
   void Reset() {
-    if (s_ != 0)
-      aiter_->Reset();
+    if (s_ != 0) aiter_->Reset();
     pos_ = 0;
   }
 
@@ -273,9 +263,7 @@ class ArcIterator< ComplementFst<A> > : public ArcIteratorBase<A> {
     pos_ = a;
   }
 
-  uint32 Flags() const {
-    return kArcValueFlags;
-  }
+  uint32 Flags() const { return kArcValueFlags; }
 
   void SetFlags(uint32 f, uint32 m) {}
 
@@ -283,37 +271,37 @@ class ArcIterator< ComplementFst<A> > : public ArcIteratorBase<A> {
   // This allows base class virtual access to non-virtual derived-
   // class members of the same name. It makes the derived class more
   // efficient to use but unsafe to further derive.
-  virtual bool Done_() const { return Done(); }
-  virtual const A& Value_() const { return Value(); }
-  virtual void Next_() { Next(); }
-  virtual size_t Position_() const { return Position(); }
-  virtual void Reset_() { Reset(); }
-  virtual void Seek_(size_t a) { Seek(a); }
-  uint32 Flags_() const { return Flags(); }
-  void SetFlags_(uint32 f, uint32 m) { SetFlags(f, m); }
+  bool Done_() const override { return Done(); }
+  const A &Value_() const override { return Value(); }
+  void Next_() override { Next(); }
+  size_t Position_() const override { return Position(); }
+  void Reset_() override { Reset(); }
+  void Seek_(size_t a) override { Seek(a); }
+  uint32 Flags_() const override { return Flags(); }
+  void SetFlags_(uint32 f, uint32 m) override { SetFlags(f, m); }
 
-  ArcIterator< Fst<A> > *aiter_;
+  ArcIterator<Fst<A>> *aiter_;
   StateId s_;
   size_t pos_;
   mutable A arc_;
   DISALLOW_COPY_AND_ASSIGN(ArcIterator);
 };
 
-
-template <class A> inline void
-ComplementFst<A>::InitStateIterator(StateIteratorData<A> *data) const {
-  data->base = new StateIterator< ComplementFst<A> >(*this);
+template <class A>
+inline void ComplementFst<A>::InitStateIterator(
+    StateIteratorData<A> *data) const {
+  data->base = new StateIterator<ComplementFst<A>>(*this);
 }
 
-template <class A> inline void
-ComplementFst<A>::InitArcIterator(StateId s, ArcIteratorData<A> *data) const {
-  data->base = new ArcIterator< ComplementFst<A> >(*this, s);
+template <class A>
+inline void ComplementFst<A>::InitArcIterator(StateId s,
+                                              ArcIteratorData<A> *data) const {
+  data->base = new ArcIterator<ComplementFst<A>>(*this, s);
 }
 
-
 // Useful alias when using StdArc.
 typedef ComplementFst<StdArc> StdComplementFst;
 
 }  // namespace fst
 
-#endif  // FST_LIB_COMPLEMENT_H__
+#endif  // FST_LIB_COMPLEMENT_H_
