@@ -1,36 +1,20 @@
-// compose-filter.h
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
 // Classes for filtering the composition matches, e.g. for correct epsilon
 // handling.
 
-#ifndef FST_LIB_COMPOSE_FILTER_H__
-#define FST_LIB_COMPOSE_FILTER_H__
+#ifndef FST_LIB_COMPOSE_FILTER_H_
+#define FST_LIB_COMPOSE_FILTER_H_
 
-#include <fst/fst.h>
-#include <fst/fst-decl.h>  // For optional argument declarations
 #include <fst/filter-state.h>
+#include <fst/fst-decl.h>  // For optional argument declarations
+#include <fst/fst.h>
 #include <fst/matcher.h>
 
 
 namespace fst {
 
-
 // COMPOSITION FILTERS - these determine which matches are allowed to
 // proceed. The filter's state is represented by the type
 // ComposeFilter::FilterState. The basic filters handle correct
@@ -82,7 +66,6 @@ namespace fst {
 //   uint64 Properties(uint64 props) const;
 // };
 
-
 // This filter allows only exact matching of symbols from FST1 with on
 // FST2; e.g. no special interpretation of epsilons.  (Template arg
 // default declared in fst-decl.h.)
@@ -92,7 +75,7 @@ class NullComposeFilter {
   typedef typename M1::FST FST1;
   typedef typename M2::FST FST2;
   typedef typename FST1::Arc Arc;
-  typedef CharFilterState FilterState;
+  typedef TrivialFilterState FilterState;
   typedef M1 Matcher1;
   typedef M2 Matcher2;
 
@@ -100,15 +83,14 @@ class NullComposeFilter {
   typedef typename Arc::Label Label;
   typedef typename Arc::Weight Weight;
 
-  NullComposeFilter(const FST1 &fst1, const FST2 &fst2,
-                         M1 *matcher1 = 0, M2 *matcher2 = 0)
+  NullComposeFilter(const FST1 &fst1, const FST2 &fst2, M1 *matcher1 = 0,
+                    M2 *matcher2 = 0)
       : matcher1_(matcher1 ? matcher1 : new M1(fst1, MATCH_OUTPUT)),
         matcher2_(matcher2 ? matcher2 : new M2(fst2, MATCH_INPUT)),
         fst1_(matcher1_->GetFst()),
         fst2_(matcher2_->GetFst()) {}
 
-  NullComposeFilter(const NullComposeFilter<M1, M2> &filter,
-                         bool safe = false)
+  NullComposeFilter(const NullComposeFilter<M1, M2> &filter, bool safe = false)
       : matcher1_(filter.matcher1_->Copy(safe)),
         matcher2_(filter.matcher2_->Copy(safe)),
         fst1_(matcher1_->GetFst()),
@@ -119,13 +101,14 @@ class NullComposeFilter {
     delete matcher2_;
   }
 
-  FilterState Start() const { return FilterState(0); }
+  FilterState Start() const { return FilterState(true); }
 
-  void SetState(StateId, StateId, const FilterState &) { }
+  void SetState(StateId, StateId, const FilterState &) {}
 
   FilterState FilterArc(Arc *arc1, Arc *arc2) const {
-    return (arc1->olabel == kNoLabel || arc2->ilabel == kNoLabel) ?
-        FilterState::NoState() : FilterState(0);
+    return (arc1->olabel == kNoLabel || arc2->ilabel == kNoLabel)
+               ? FilterState::NoState()
+               : FilterState(true);
   }
 
   void FilterFinal(Weight *, Weight *) const {}
@@ -145,6 +128,72 @@ class NullComposeFilter {
   void operator=(const NullComposeFilter<M1, M2> &);  // disallow
 };
 
+// This filter allows all epsilon matches, potentially resulting in
+// redundant epsilon paths. The use of this filter gives correct results iff
+// one of the following conditions hold:
+//  (1) The semiring is idempotent,
+//  (2) 'fst1' is output-epsilon free,
+//  (3) 'fst2' is input-epsilon free.
+// For (1), redundant epsilon paths may be created but won't hurt correctness.
+// For (2) and (3), no redundant paths are created.
+// (Template arg default declared in fst-decl.h).
+template <class M1, class M2 /* = M1 */>
+class TrivialComposeFilter {
+ public:
+  typedef typename M1::FST FST1;
+  typedef typename M2::FST FST2;
+  typedef typename FST1::Arc Arc;
+  typedef TrivialFilterState FilterState;
+  typedef M1 Matcher1;
+  typedef M2 Matcher2;
+
+  typedef typename Arc::StateId StateId;
+  typedef typename Arc::Label Label;
+  typedef typename Arc::Weight Weight;
+
+  TrivialComposeFilter(const FST1 &fst1, const FST2 &fst2, M1 *matcher1 = 0,
+                       M2 *matcher2 = 0)
+      : matcher1_(matcher1 ? matcher1 : new M1(fst1, MATCH_OUTPUT)),
+        matcher2_(matcher2 ? matcher2 : new M2(fst2, MATCH_INPUT)),
+        fst1_(matcher1_->GetFst()),
+        fst2_(matcher2_->GetFst()) {}
+
+  TrivialComposeFilter(const TrivialComposeFilter<M1, M2> &filter,
+                       bool safe = false)
+      : matcher1_(filter.matcher1_->Copy(safe)),
+        matcher2_(filter.matcher2_->Copy(safe)),
+        fst1_(matcher1_->GetFst()),
+        fst2_(matcher2_->GetFst()) {}
+
+  ~TrivialComposeFilter() {
+    delete matcher1_;
+    delete matcher2_;
+  }
+
+  FilterState Start() const { return FilterState(true); }
+
+  void SetState(StateId, StateId, const FilterState &) {}
+
+  FilterState FilterArc(Arc *arc1, Arc *arc2) const {
+    return FilterState(true);
+  }
+
+  void FilterFinal(Weight *, Weight *) const {}
+
+  // Return resp matchers. Ownership stays with filter.
+  Matcher1 *GetMatcher1() { return matcher1_; }
+  Matcher2 *GetMatcher2() { return matcher2_; }
+
+  uint64 Properties(uint64 props) const { return props; }
+
+ private:
+  Matcher1 *matcher1_;
+  Matcher2 *matcher2_;
+  const FST1 &fst1_;
+  const FST2 &fst2_;
+
+  void operator=(const TrivialComposeFilter<M1, M2> &);  // disallow
+};
 
 // This filter requires epsilons on FST1 to be read before epsilons on FST2.
 // (Template arg default declared in fst-decl.h.)
@@ -162,8 +211,8 @@ class SequenceComposeFilter {
   typedef typename Arc::Label Label;
   typedef typename Arc::Weight Weight;
 
-  SequenceComposeFilter(const FST1 &fst1, const FST2 &fst2,
-                        M1 *matcher1 = 0, M2 *matcher2 = 0)
+  SequenceComposeFilter(const FST1 &fst1, const FST2 &fst2, M1 *matcher1 = 0,
+                        M2 *matcher2 = 0)
       : matcher1_(matcher1 ? matcher1 : new M1(fst1, MATCH_OUTPUT)),
         matcher2_(matcher2 ? matcher2 : new M2(fst2, MATCH_INPUT)),
         fst1_(matcher1_->GetFst()),
@@ -188,8 +237,7 @@ class SequenceComposeFilter {
   FilterState Start() const { return FilterState(0); }
 
   void SetState(StateId s1, StateId s2, const FilterState &f) {
-    if (s1_ == s1 && s2_ == s2 && f == f_)
-      return;
+    if (s1_ == s1 && s2_ == s2 && f == f_) return;
     s1_ = s1;
     s2_ = s2;
     f_ = f;
@@ -202,8 +250,8 @@ class SequenceComposeFilter {
 
   FilterState FilterArc(Arc *arc1, Arc *arc2) const {
     if (arc1->olabel == kNoLabel)
-      return alleps1_ ? FilterState::NoState() :
-        noeps1_ ? FilterState(0) : FilterState(1);
+      return alleps1_ ? FilterState::NoState() : noeps1_ ? FilterState(0)
+                                                         : FilterState(1);
     else if (arc2->ilabel == kNoLabel)
       return f_ != FilterState(0) ? FilterState::NoState() : FilterState(0);
     else
@@ -231,7 +279,6 @@ class SequenceComposeFilter {
   void operator=(const SequenceComposeFilter<M1, M2> &);  // disallow
 };
 
-
 // This filter requires epsilons on FST2 to be read before epsilons on FST1.
 // (Template arg default declared in fst-decl.h.)
 template <class M1, class M2 /* = M1 */>
@@ -248,8 +295,8 @@ class AltSequenceComposeFilter {
   typedef typename Arc::Label Label;
   typedef typename Arc::Weight Weight;
 
-  AltSequenceComposeFilter(const FST1 &fst1, const FST2 &fst2,
-                        M1 *matcher1 = 0, M2 *matcher2 = 0)
+  AltSequenceComposeFilter(const FST1 &fst1, const FST2 &fst2, M1 *matcher1 = 0,
+                           M2 *matcher2 = 0)
       : matcher1_(matcher1 ? matcher1 : new M1(fst1, MATCH_OUTPUT)),
         matcher2_(matcher2 ? matcher2 : new M2(fst2, MATCH_INPUT)),
         fst2_(matcher2_->GetFst()),
@@ -274,8 +321,7 @@ class AltSequenceComposeFilter {
   FilterState Start() const { return FilterState(0); }
 
   void SetState(StateId s1, StateId s2, const FilterState &f) {
-    if (s1_ == s1 && s2_ == s2 && f == f_)
-      return;
+    if (s1_ == s1 && s2_ == s2 && f == f_) return;
     s1_ = s1;
     s2_ = s2;
     f_ = f;
@@ -288,8 +334,8 @@ class AltSequenceComposeFilter {
 
   FilterState FilterArc(Arc *arc1, Arc *arc2) const {
     if (arc2->ilabel == kNoLabel)
-      return alleps2_ ? FilterState::NoState() :
-        noeps2_ ? FilterState(0) : FilterState(1);
+      return alleps2_ ? FilterState::NoState() : noeps2_ ? FilterState(0)
+                                                         : FilterState(1);
     else if (arc1->olabel == kNoLabel)
       return f_ == FilterState(1) ? FilterState::NoState() : FilterState(0);
     else
@@ -314,10 +360,9 @@ class AltSequenceComposeFilter {
   bool alleps2_;   // Only epsilons (and non-final) leaving s2_?
   bool noeps2_;    // No epsilons leaving s2_?
 
-void operator=(const AltSequenceComposeFilter<M1, M2> &);  // disallow
+  void operator=(const AltSequenceComposeFilter<M1, M2> &);  // disallow
 };
 
-
 // This filter requires epsilons on FST1 to be matched with epsilons on FST2
 // whenever possible. (Template arg default declared in fst-decl.h.)
 template <class M1, class M2 /* = M1 */>
@@ -334,8 +379,8 @@ class MatchComposeFilter {
   typedef typename Arc::Label Label;
   typedef typename Arc::Weight Weight;
 
-  MatchComposeFilter(const FST1 &fst1, const FST2 &fst2,
-                     M1 *matcher1 = 0, M2 *matcher2 = 0)
+  MatchComposeFilter(const FST1 &fst1, const FST2 &fst2, M1 *matcher1 = 0,
+                     M2 *matcher2 = 0)
       : matcher1_(matcher1 ? matcher1 : new M1(fst1, MATCH_OUTPUT)),
         matcher2_(matcher2 ? matcher2 : new M2(fst2, MATCH_INPUT)),
         fst1_(matcher1_->GetFst()),
@@ -362,8 +407,7 @@ class MatchComposeFilter {
   FilterState Start() const { return FilterState(0); }
 
   void SetState(StateId s1, StateId s2, const FilterState &f) {
-    if (s1_ == s1 && s2_ == s2 && f == f_)
-      return;
+    if (s1_ == s1 && s2_ == s2 && f == f_) return;
     s1_ = s1;
     s2_ = s2;
     f_ = f;
@@ -381,15 +425,19 @@ class MatchComposeFilter {
 
   FilterState FilterArc(Arc *arc1, Arc *arc2) const {
     if (arc2->ilabel == kNoLabel)  // Epsilon on Fst1
-      return f_ == FilterState(0) ?
-          (noeps2_ ? FilterState(0) :
-           (alleps2_ ? FilterState::NoState(): FilterState(1))) :
-          (f_ == FilterState(1) ? FilterState(1) : FilterState::NoState());
+      return f_ == FilterState(0)
+                 ? (noeps2_
+                        ? FilterState(0)
+                        : (alleps2_ ? FilterState::NoState() : FilterState(1)))
+                 : (f_ == FilterState(1) ? FilterState(1)
+                                         : FilterState::NoState());
     else if (arc1->olabel == kNoLabel)  // Epsilon on Fst2
-      return f_ == FilterState(0) ?
-          (noeps1_ ? FilterState(0) :
-           (alleps1_ ? FilterState::NoState() : FilterState(2))) :
-          (f_ == FilterState(2) ? FilterState(2) : FilterState::NoState());
+      return f_ == FilterState(0)
+                 ? (noeps1_
+                        ? FilterState(0)
+                        : (alleps1_ ? FilterState::NoState() : FilterState(2)))
+                 : (f_ == FilterState(2) ? FilterState(2)
+                                         : FilterState::NoState());
     else if (arc1->olabel == 0)  // Epsilon on both
       return f_ == FilterState(0) ? FilterState(0) : FilterState::NoState();
     else  // Both are non-epsilons
@@ -418,7 +466,6 @@ class MatchComposeFilter {
   void operator=(const MatchComposeFilter<M1, M2> &);  // disallow
 };
 
-
 // This filter works with the MultiEpsMatcher to determine if
 // 'multi-epsilons' are preserved in the composition output
 // (rather than rewritten as 0) and ensures correct properties.
@@ -437,9 +484,8 @@ class MultiEpsFilter {
   typedef typename Arc::Label Label;
   typedef typename Arc::Weight Weight;
 
-  MultiEpsFilter(const FST1 &fst1, const FST2 &fst2,
-                 Matcher1 *matcher1 = 0,  Matcher2 *matcher2 = 0,
-                 bool keep_multi_eps = false)
+  MultiEpsFilter(const FST1 &fst1, const FST2 &fst2, Matcher1 *matcher1 = 0,
+                 Matcher2 *matcher2 = 0, bool keep_multi_eps = false)
       : filter_(fst1, fst2, matcher1, matcher2),
         keep_multi_eps_(keep_multi_eps) {}
 
@@ -456,10 +502,8 @@ class MultiEpsFilter {
   FilterState FilterArc(Arc *arc1, Arc *arc2) const {
     FilterState f = filter_.FilterArc(arc1, arc2);
     if (keep_multi_eps_) {
-      if (arc1->olabel == kNoLabel)
-        arc1->ilabel = arc2->ilabel;
-      if (arc2->ilabel == kNoLabel)
-        arc2->olabel = arc1->olabel;
+      if (arc1->olabel == kNoLabel) arc1->ilabel = arc2->ilabel;
+      if (arc2->ilabel == kNoLabel) arc2->olabel = arc1->olabel;
     }
     return f;
   }
@@ -484,5 +528,4 @@ class MultiEpsFilter {
 
 }  // namespace fst
 
-
 #endif  // FST_LIB_COMPOSE_FILTER_H__
