@@ -1,30 +1,14 @@
-// const-fst.h
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
+//
+// Simple concrete immutable FST whose states and arcs are each stored in
+// single arrays.
 
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
-// Simple concrete immutable FST whose states and arcs are each stored
-// in single arrays.
-
-#ifndef FST_LIB_CONST_FST_H__
-#define FST_LIB_CONST_FST_H__
+#ifndef FST_LIB_CONST_FST_H_
+#define FST_LIB_CONST_FST_H_
 
 #include <string>
 #include <vector>
-using std::vector;
 
 #include <fst/expanded-fst.h>
 #include <fst/fst-decl.h>  // For optional argument declarations
@@ -35,8 +19,10 @@ using std::vector;
 
 namespace fst {
 
-template <class A, class U> class ConstFst;
-template <class F, class G> void Cast(const F &, G *);
+template <class A, class U>
+class ConstFst;
+template <class F, class G>
+void Cast(const F &, G *);
 
 // States and arcs each implemented by single arrays, templated on the
 // Arc definition. The unsigned type U is used to represent indices into
@@ -56,8 +42,13 @@ class ConstFstImpl : public FstImpl<A> {
   typedef U Unsigned;
 
   ConstFstImpl()
-      : states_region_(0), arcs_region_(0), states_(0), arcs_(0), nstates_(0),
-        narcs_(0), start_(kNoStateId) {
+      : states_region_(0),
+        arcs_region_(0),
+        states_(0),
+        arcs_(0),
+        nstates_(0),
+        narcs_(0),
+        start_(kNoStateId) {
     string type = "const";
     if (sizeof(U) != sizeof(uint32)) {
       string size;
@@ -70,7 +61,7 @@ class ConstFstImpl : public FstImpl<A> {
 
   explicit ConstFstImpl(const Fst<A> &fst);
 
-  ~ConstFstImpl() {
+  ~ConstFstImpl() override {
     delete arcs_region_;
     delete states_region_;
   }
@@ -87,9 +78,10 @@ class ConstFstImpl : public FstImpl<A> {
 
   size_t NumOutputEpsilons(StateId s) const { return states_[s].noepsilons; }
 
-  static ConstFstImpl<A, U> *Read(istream &strm, const FstReadOptions &opts);
+  static ConstFstImpl<A, U> *Read(std::istream &strm,
+                                  const FstReadOptions &opts);
 
-  A *Arcs(StateId s) { return arcs_ + states_[s].pos; }
+  const A *Arcs(StateId s) const { return arcs_ + states_[s].pos; }
 
   // Provide information needed for generic state iterator
   void InitStateIterator(StateIteratorData<A> *data) const {
@@ -110,11 +102,11 @@ class ConstFstImpl : public FstImpl<A> {
 
   // States implemented by array *states_ below, arcs by (single) *arcs_.
   struct State {
-    Weight final;                // Final weight
-    Unsigned pos;                // Start of state's arcs in *arcs_
-    Unsigned narcs;              // Number of arcs (per state)
-    Unsigned niepsilons;         // # of input epsilons
-    Unsigned noepsilons;         // # of output epsilons
+    Weight final;         // Final weight
+    Unsigned pos;         // Start of state's arcs in *arcs_
+    Unsigned narcs;       // Number of arcs (per state)
+    Unsigned niepsilons;  // # of input epsilons
+    Unsigned noepsilons;  // # of output epsilons
     State() : final(Weight::Zero()), niepsilons(0), noepsilons(0) {}
   };
 
@@ -128,13 +120,13 @@ class ConstFstImpl : public FstImpl<A> {
   // Minimum file format version supported
   static const int kMinFileVersion = 1;
 
-  MappedFile *states_region_;    // Mapped file for states
-  MappedFile *arcs_region_;      // Mapped file for arcs
-  State *states_;                // States represenation
-  A *arcs_;                      // Arcs representation
-  StateId nstates_;              // Number of states
-  size_t narcs_;                 // Number of arcs (per FST)
-  StateId start_;                // Initial state
+  MappedFile *states_region_;  // Mapped file for states
+  MappedFile *arcs_region_;    // Mapped file for arcs
+  State *states_;              // States represenation
+  A *arcs_;                    // Arcs representation
+  StateId nstates_;            // Number of states
+  size_t narcs_;               // Number of arcs (per FST)
+  StateId start_;              // Initial state
 
   DISALLOW_COPY_AND_ASSIGN(ConstFstImpl);
 };
@@ -148,8 +140,7 @@ const int ConstFstImpl<A, U>::kAlignedFileVersion;
 template <class A, class U>
 const int ConstFstImpl<A, U>::kMinFileVersion;
 
-
-template<class A, class U>
+template <class A, class U>
 ConstFstImpl<A, U>::ConstFstImpl(const Fst<A> &fst) : nstates_(0), narcs_(0) {
   string type = "const";
   if (sizeof(U) != sizeof(uint32)) {
@@ -163,20 +154,16 @@ ConstFstImpl<A, U>::ConstFstImpl(const Fst<A> &fst) : nstates_(0), narcs_(0) {
   start_ = fst.Start();
 
   // Count # of states and arcs.
-  for (StateIterator< Fst<A> > siter(fst);
-       !siter.Done();
-       siter.Next()) {
+  for (StateIterator<Fst<A>> siter(fst); !siter.Done(); siter.Next()) {
     ++nstates_;
     StateId s = siter.Value();
-    for (ArcIterator< Fst<A> > aiter(fst, s);
-         !aiter.Done();
-         aiter.Next())
+    for (ArcIterator<Fst<A>> aiter(fst, s); !aiter.Done(); aiter.Next())
       ++narcs_;
   }
   states_region_ = MappedFile::Allocate(nstates_ * sizeof(*states_));
   arcs_region_ = MappedFile::Allocate(narcs_ * sizeof(*arcs_));
-  states_ = reinterpret_cast<State*>(states_region_->mutable_data());
-  arcs_ = reinterpret_cast<A*>(arcs_region_->mutable_data());
+  states_ = reinterpret_cast<State *>(states_region_->mutable_data());
+  arcs_ = reinterpret_cast<A *>(arcs_region_->mutable_data());
   size_t pos = 0;
   for (StateId s = 0; s < nstates_; ++s) {
     states_[s].final = fst.Final(s);
@@ -184,24 +171,25 @@ ConstFstImpl<A, U>::ConstFstImpl(const Fst<A> &fst) : nstates_(0), narcs_(0) {
     states_[s].narcs = 0;
     states_[s].niepsilons = 0;
     states_[s].noepsilons = 0;
-    for (ArcIterator< Fst<A> > aiter(fst, s);
-         !aiter.Done();
-         aiter.Next()) {
+    for (ArcIterator<Fst<A>> aiter(fst, s); !aiter.Done(); aiter.Next()) {
       const A &arc = aiter.Value();
       ++states_[s].narcs;
-      if (arc.ilabel == 0)
-        ++states_[s].niepsilons;
-      if (arc.olabel == 0)
-        ++states_[s].noepsilons;
+      if (arc.ilabel == 0) ++states_[s].niepsilons;
+      if (arc.olabel == 0) ++states_[s].noepsilons;
       arcs_[pos++] = arc;
     }
   }
-  SetProperties(fst.Properties(kCopyProperties, true) | kStaticProperties);
+
+  uint64 props = fst.Properties(kMutable, false) ?
+      fst.Properties(kCopyProperties, true) :
+      CheckProperties(fst,
+                      kCopyProperties & ~kWeightedCycles & ~kUnweightedCycles,
+                      kCopyProperties);
+  SetProperties(props | kStaticProperties);
 }
 
-
-template<class A, class U>
-ConstFstImpl<A, U> *ConstFstImpl<A, U>::Read(istream &strm,
+template <class A, class U>
+ConstFstImpl<A, U> *ConstFstImpl<A, U>::Read(std::istream &strm,
                                              const FstReadOptions &opts) {
   ConstFstImpl<A, U> *impl = new ConstFstImpl<A, U>;
   FstHeader hdr;
@@ -224,15 +212,15 @@ ConstFstImpl<A, U> *ConstFstImpl<A, U>::Read(istream &strm,
   }
 
   size_t b = impl->nstates_ * sizeof(typename ConstFstImpl<A, U>::State);
-  impl->states_region_ = MappedFile::Map(
-      &strm, opts.mode == FstReadOptions::MAP, opts.source, b);
-  if (!strm || impl->states_region_ == NULL) {
+  impl->states_region_ =
+      MappedFile::Map(&strm, opts.mode == FstReadOptions::MAP, opts.source, b);
+  if (!strm || impl->states_region_ == nullptr) {
     LOG(ERROR) << "ConstFst::Read: Read failed: " << opts.source;
     delete impl;
     return 0;
   }
-  impl->states_ = reinterpret_cast<State*>(
-      impl->states_region_->mutable_data());
+  impl->states_ =
+      reinterpret_cast<State *>(impl->states_region_->mutable_data());
   if ((hdr.GetFlags() & FstHeader::IS_ALIGNED) && !AlignInput(strm)) {
     LOG(ERROR) << "ConstFst::Read: Alignment failed: " << opts.source;
     delete impl;
@@ -240,14 +228,14 @@ ConstFstImpl<A, U> *ConstFstImpl<A, U>::Read(istream &strm,
   }
 
   b = impl->narcs_ * sizeof(A);
-  impl->arcs_region_ = MappedFile::Map(
-      &strm, opts.mode == FstReadOptions::MAP, opts.source, b);
-  if (!strm || impl->arcs_region_ == NULL) {
+  impl->arcs_region_ =
+      MappedFile::Map(&strm, opts.mode == FstReadOptions::MAP, opts.source, b);
+  if (!strm || impl->arcs_region_ == nullptr) {
     LOG(ERROR) << "ConstFst::Read: Read failed: " << opts.source;
     delete impl;
     return 0;
   }
-  impl->arcs_ = reinterpret_cast<A*>(impl->arcs_region_->mutable_data());
+  impl->arcs_ = reinterpret_cast<A *>(impl->arcs_region_->mutable_data());
   return impl;
 }
 
@@ -257,81 +245,79 @@ ConstFstImpl<A, U> *ConstFstImpl<A, U>::Read(istream &strm,
 // represent indices into the arc array (default declared
 // in fst-decl.h).
 template <class A, class U /* = uint32 */>
-class ConstFst : public ImplToExpandedFst< ConstFstImpl<A, U> > {
+class ConstFst : public ImplToExpandedFst<ConstFstImpl<A, U>> {
  public:
-  friend class StateIterator< ConstFst<A, U> >;
-  friend class ArcIterator< ConstFst<A, U> >;
-  template <class F, class G> void friend Cast(const F &, G *);
+  friend class StateIterator<ConstFst<A, U>>;
+  friend class ArcIterator<ConstFst<A, U>>;
+  template <class F, class G>
+  void friend Cast(const F &, G *);
 
   typedef A Arc;
   typedef typename A::StateId StateId;
   typedef ConstFstImpl<A, U> Impl;
   typedef U Unsigned;
 
-  ConstFst() : ImplToExpandedFst<Impl>(new Impl()) {}
+  ConstFst() : ImplToExpandedFst<Impl>(std::make_shared<Impl>()) {}
 
   explicit ConstFst(const Fst<A> &fst)
-      : ImplToExpandedFst<Impl>(new Impl(fst)) {}
+      : ImplToExpandedFst<Impl>(std::make_shared<Impl>(fst)) {}
 
-  ConstFst(const ConstFst<A, U> &fst) : ImplToExpandedFst<Impl>(fst) {}
+  ConstFst(const ConstFst<A, U> &fst, bool safe = false)
+      : ImplToExpandedFst<Impl>(fst) {}
 
   // Get a copy of this ConstFst. See Fst<>::Copy() for further doc.
-  virtual ConstFst<A, U> *Copy(bool safe = false) const {
-    return new ConstFst<A, U>(*this);
+  ConstFst<A, U> *Copy(bool safe = false) const override {
+    return new ConstFst<A, U>(*this, safe);
   }
 
-  // Read a ConstFst from an input stream; return NULL on error
-  static ConstFst<A, U> *Read(istream &strm, const FstReadOptions &opts) {
-    Impl* impl = Impl::Read(strm, opts);
-    return impl ? new ConstFst<A, U>(impl) : 0;
+  // Read a ConstFst from an input stream; return nullptr on error
+  static ConstFst<A, U> *Read(std::istream &strm, const FstReadOptions &opts) {
+    Impl *impl = Impl::Read(strm, opts);
+    return impl ? new ConstFst<A, U>(std::shared_ptr<Impl>(impl)) : nullptr;
   }
 
-  // Read a ConstFst from a file; return NULL on error
+  // Read a ConstFst from a file; return nullptr on error
   // Empty filename reads from standard input
   static ConstFst<A, U> *Read(const string &filename) {
-    Impl* impl = ImplToExpandedFst<Impl>::Read(filename);
-    return impl ? new ConstFst<A, U>(impl) : 0;
+    Impl *impl = ImplToExpandedFst<Impl>::Read(filename);
+    return impl ? new ConstFst<A, U>(std::shared_ptr<Impl>(impl)) : nullptr;
   }
 
-  virtual bool Write(ostream &strm, const FstWriteOptions &opts) const {
+  bool Write(std::ostream &strm, const FstWriteOptions &opts) const override {
     return WriteFst(*this, strm, opts);
   }
 
-  virtual bool Write(const string &filename) const {
+  bool Write(const string &filename) const override {
     return Fst<A>::WriteFile(filename);
   }
 
   template <class F>
-  static bool WriteFst(const F &fst, ostream &strm,
+  static bool WriteFst(const F &fst, std::ostream &strm,
                        const FstWriteOptions &opts);
 
-  virtual void InitStateIterator(StateIteratorData<Arc> *data) const {
+  void InitStateIterator(StateIteratorData<Arc> *data) const override {
     GetImpl()->InitStateIterator(data);
   }
 
-  virtual void InitArcIterator(StateId s, ArcIteratorData<Arc> *data) const {
+  void InitArcIterator(StateId s, ArcIteratorData<Arc> *data) const override {
     GetImpl()->InitArcIterator(s, data);
   }
 
  private:
-  explicit ConstFst(Impl *impl) : ImplToExpandedFst<Impl>(impl) {}
+  explicit ConstFst(std::shared_ptr<Impl> impl)
+      : ImplToExpandedFst<Impl>(impl) {}
 
-  // Makes visible to friends.
-  Impl *GetImpl() const { return ImplToFst<Impl, ExpandedFst<A> >::GetImpl(); }
-
-  void SetImpl(Impl *impl, bool own_impl = true) {
-    ImplToFst< Impl, ExpandedFst<A> >::SetImpl(impl, own_impl);
-  }
+  using ImplToFst<Impl, ExpandedFst<A>>::GetImpl;
 
   // Use overloading to extract the type of the argument.
-  static Impl* GetImplIfConstFst(const ConstFst &const_fst) {
+  static const Impl *GetImplIfConstFst(const ConstFst &const_fst) {
     return const_fst.GetImpl();
   }
 
   // Note that this does not give privileged treatment to subtypes of ConstFst.
-  template<typename NonConstFst>
-  static Impl* GetImplIfConstFst(const NonConstFst& fst) {
-    return NULL;
+  template <typename NonConstFst>
+  static Impl *GetImplIfConstFst(const NonConstFst &fst) {
+    return nullptr;
   }
 
   void operator=(const ConstFst<A, U> &fst);  // disallow
@@ -342,18 +328,18 @@ class ConstFst : public ImplToExpandedFst< ConstFstImpl<A, U> > {
 //
 template <class A, class U>
 template <class F>
-bool ConstFst<A, U>::WriteFst(const F &fst, ostream &strm,
+bool ConstFst<A, U>::WriteFst(const F &fst, std::ostream &strm,
                               const FstWriteOptions &opts) {
-  int file_version = opts.align ? ConstFstImpl<A, U>::kAlignedFileVersion :
-      ConstFstImpl<A, U>::kFileVersion;
+  int file_version = opts.align ? ConstFstImpl<A, U>::kAlignedFileVersion
+                                : ConstFstImpl<A, U>::kFileVersion;
   size_t num_arcs = -1, num_states = -1;
   size_t start_offset = 0;
   bool update_header = true;
-  if (Impl* impl = GetImplIfConstFst(fst)) {
+  if (const Impl *impl = GetImplIfConstFst(fst)) {
     num_arcs = impl->narcs_;
     num_states = impl->nstates_;
     update_header = false;
-  } else if ((start_offset = strm.tellp()) == -1) {
+  } else if (opts.stream_write || (start_offset = strm.tellp()) == -1) {
     // precompute values needed for header when we cannot seek to rewrite it.
     num_arcs = 0;
     num_states = 0;
@@ -374,7 +360,7 @@ bool ConstFst<A, U>::WriteFst(const F &fst, ostream &strm,
     type += size;
   }
   uint64 properties = fst.Properties(kCopyProperties, true) |
-      ConstFstImpl<A, U>::kStaticProperties;
+                      ConstFstImpl<A, U>::kStaticProperties;
   FstImpl<A>::WriteFstHeader(fst, strm, opts, file_version, type, properties,
                              &hdr);
   if (opts.align && !AlignOutput(strm)) {
@@ -407,7 +393,7 @@ bool ConstFst<A, U>::WriteFst(const F &fst, ostream &strm,
   }
   strm.flush();
   if (!strm) {
-    LOG(ERROR) << "ConstFst Write write failed: " << opts.source;
+    LOG(ERROR) << "ConstFst::WriteFst: write failed: " << opts.source;
     return false;
   }
   if (update_header) {
@@ -430,7 +416,7 @@ bool ConstFst<A, U>::WriteFst(const F &fst, ostream &strm,
 // for sample usage (but use the ConstFst type!). This version
 // should inline.
 template <class A, class U>
-class StateIterator< ConstFst<A, U> > {
+class StateIterator<ConstFst<A, U>> {
  public:
   typedef typename A::StateId StateId;
 
@@ -452,22 +438,22 @@ class StateIterator< ConstFst<A, U> > {
   DISALLOW_COPY_AND_ASSIGN(StateIterator);
 };
 
-
 // Specialization for ConstFst; see generic version in fst.h
 // for sample usage (but use the ConstFst type!). This version
 // should inline.
 template <class A, class U>
-class ArcIterator< ConstFst<A, U> > {
+class ArcIterator<ConstFst<A, U>> {
  public:
   typedef typename A::StateId StateId;
 
   ArcIterator(const ConstFst<A, U> &fst, StateId s)
       : arcs_(fst.GetImpl()->Arcs(s)),
-        narcs_(fst.GetImpl()->NumArcs(s)), i_(0) {}
+        narcs_(fst.GetImpl()->NumArcs(s)),
+        i_(0) {}
 
   bool Done() const { return i_ >= narcs_; }
 
-  const A& Value() const { return arcs_[i_]; }
+  const A &Value() const { return arcs_[i_]; }
 
   void Next() { ++i_; }
 
@@ -477,9 +463,7 @@ class ArcIterator< ConstFst<A, U> > {
 
   void Seek(size_t a) { i_ = a; }
 
-  uint32 Flags() const {
-    return kArcValueFlags;
-  }
+  uint32 Flags() const { return kArcValueFlags; }
 
   void SetFlags(uint32 f, uint32 m) {}
 
@@ -496,4 +480,4 @@ typedef ConstFst<StdArc> StdConstFst;
 
 }  // namespace fst
 
-#endif  // FST_LIB_CONST_FST_H__
+#endif  // FST_LIB_CONST_FST_H_
