@@ -1,26 +1,10 @@
-// disambiguate.h
-
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
 // Functions and classes to disambiguate an FST.
 
-#ifndef FST_LIB_DISAMBIGUATE_H__
-#define FST_LIB_DISAMBIGUATE_H__
+#ifndef FST_LIB_DISAMBIGUATE_H_
+#define FST_LIB_DISAMBIGUATE_H_
 
 #include <algorithm>
 #include <climits>
@@ -28,7 +12,6 @@
 #include <set>
 #include <string>
 #include <vector>
-using std::vector;
 
 #include <fst/arcsort.h>
 #include <fst/compose.h>
@@ -52,12 +35,12 @@ struct DisambiguateOptions : public DeterminizeOptions<Arc> {
   typedef typename Arc::Label Label;
 
   explicit DisambiguateOptions(float d = kDelta, Weight w = Weight::Zero(),
-                              StateId n = kNoStateId, Label l = 0)
-      : DeterminizeOptions<Arc>(d, w, n, l) {}
+                               StateId n = kNoStateId, Label l = 0)
+      : DeterminizeOptions<Arc>(d, w, n, l, DETERMINIZE_FUNCTIONAL) {}
 };
 
 // A determinize filter based on a subset element relation.
-// The relation is assumed to be reflexive.
+// The relation is assumed to be reflexive and symmetric.
 template <class Arc, class R>
 class RelationDeterminizeFilter {
  public:
@@ -69,41 +52,32 @@ class RelationDeterminizeFilter {
   typedef DeterminizeStateTuple<Arc, FilterState> StateTuple;
   typedef typename StateTuple::Subset Subset;
   typedef typename StateTuple::Element Element;
-  typedef multimap<Label, DeterminizeArc<StateTuple> > LabelMap;
+  typedef std::multimap<Label, DeterminizeArc<StateTuple>> LabelMap;
 
   // This is needed e.g. to go into the gallic domain for transducers.
   // No need to rebind the relation since its use here only depends
   // on the state Ids.
   template <class A>
-  struct rebind { typedef RelationDeterminizeFilter<A, R> other; };
+  struct rebind {
+    typedef RelationDeterminizeFilter<A, R> other;
+  };
 
   RelationDeterminizeFilter(const Fst<Arc> &fst)
-      : fst_(fst.Copy()),
-        r_(new R()),
-        s_(kNoStateId),
-        head_(0) {
-  }
+      : fst_(fst.Copy()), r_(new R()), s_(kNoStateId), head_(0) {}
 
   // Ownership of the relation is given to this class.
   RelationDeterminizeFilter(const Fst<Arc> &fst, R *r)
-      : fst_(fst.Copy()),
-        r_(r),
-        s_(kNoStateId),
-        head_(0) {
-  }
+      : fst_(fst.Copy()), r_(r), s_(kNoStateId), head_(0) {}
 
   // Ownership of the relation is given to this class. Returns head states.
-  RelationDeterminizeFilter(const Fst<Arc> &fst, R *r, vector<StateId> *head)
-      : fst_(fst.Copy()),
-        r_(r),
-        s_(kNoStateId),
-        head_(head) {
-  }
+  RelationDeterminizeFilter(const Fst<Arc> &fst, R *r,
+                            std::vector<StateId> *head)
+      : fst_(fst.Copy()), r_(r), s_(kNoStateId), head_(head) {}
 
   // This is needed e.g. to go into the gallic domain for transducers.
   // Ownership of the templated filter argument is given to this class.
   template <class F>
-  RelationDeterminizeFilter(const Fst<Arc> &fst, F* filter)
+  RelationDeterminizeFilter(const Fst<Arc> &fst, F *filter)
       : fst_(fst.Copy()),
         r_(new R(filter->GetRelation())),
         s_(kNoStateId),
@@ -113,13 +87,11 @@ class RelationDeterminizeFilter {
 
   // Copy ctr. The FST can be passed if it has been e.g. (deep) copied.
   explicit RelationDeterminizeFilter(
-      const RelationDeterminizeFilter<Arc, R> &filter,
-      const Fst<Arc> *fst = 0)
+      const RelationDeterminizeFilter<Arc, R> &filter, const Fst<Arc> *fst = 0)
       : fst_(fst ? fst->Copy() : filter.fst_->Copy()),
         r_(new R(*filter.r_)),
         s_(kNoStateId),
-        head_() {
-  }
+        head_() {}
 
   ~RelationDeterminizeFilter() {
     delete fst_;
@@ -135,8 +107,7 @@ class RelationDeterminizeFilter {
       StateId head = tuple.filter_state.GetState();
       is_final_ = fst_->Final(head) != Weight::Zero();
       if (head_) {
-        if (head_->size() <= s)
-          head_->resize(s + 1, kNoStateId);
+        if (head_->size() <= s) head_->resize(s + 1, kNoStateId);
         (*head_)[s] = head;
       }
     }
@@ -144,10 +115,8 @@ class RelationDeterminizeFilter {
 
   // Filters transition, possibly modifying label map. Returns
   // true if arc is added to label map.
-  bool FilterArc(const Arc &arc,
-                 const Element &src_element,
-                 const Element &dest_element,
-                 LabelMap *label_map) const;
+  bool FilterArc(const Arc &arc, const Element &src_element,
+                 const Element &dest_element, LabelMap *label_map) const;
 
   // Filters super-final transition, returning new final weight
   Weight FilterFinal(const Weight final_weight, const Element &element) const {
@@ -159,7 +128,7 @@ class RelationDeterminizeFilter {
   }
 
   const R &GetRelation() { return *r_; }
-  vector<StateId> *GetHeadStates() { return head_; }
+  std::vector<StateId> *GetHeadStates() { return head_; }
 
  private:
   // Pairs arc labels with state tuples with possible heads and
@@ -168,27 +137,25 @@ class RelationDeterminizeFilter {
 
   Fst<Arc> *fst_;  // Input FST
   R *r_;           // Relation compatible with the inverse trans. function
-  StateId s_;                    // Current state
-  const StateTuple *tuple_;      // Current tuple
-  bool is_final_;                // Is the current head state final?
-  vector<StateId> *head_;        // Head state for a given state.
+  StateId s_;      // Current state
+  const StateTuple *tuple_;  // Current tuple
+  bool is_final_;            // Is the current head state final?
+  std::vector<StateId> *head_;  // Head state for a given state.
 
-  DISALLOW_COPY_AND_ASSIGN(RelationDeterminizeFilter);
+  void operator=(const RelationDeterminizeFilter<Arc, R> &filt);  // disallow
 };
 
-template <class Arc, class R> bool
-RelationDeterminizeFilter<Arc, R>::FilterArc(const Arc &arc,
-                                             const Element &src_element,
-                                             const Element &dest_element,
-                                             LabelMap *label_map) const {
+template <class Arc, class R>
+bool RelationDeterminizeFilter<Arc, R>::FilterArc(const Arc &arc,
+                                                  const Element &src_element,
+                                                  const Element &dest_element,
+                                                  LabelMap *label_map) const {
   bool added = false;
-  if (label_map->empty())
-    InitLabelMap(label_map);
+  if (label_map->empty()) InitLabelMap(label_map);
 
   // Adds element to state tuple if element state is related to tuple head.
-  for (typename LabelMap::iterator liter = label_map->lower_bound(arc.ilabel);
-       liter != label_map->end() && liter->first == arc.ilabel;
-       ++liter) {
+  for (auto liter = label_map->lower_bound(arc.ilabel);
+       liter != label_map->end() && liter->first == arc.ilabel; ++liter) {
     StateTuple *dest_tuple = liter->second.dest_tuple;
     StateId dest_head = dest_tuple->filter_state.GetState();
     if ((*r_)(dest_element.state_id, dest_head)) {
@@ -199,20 +166,20 @@ RelationDeterminizeFilter<Arc, R>::FilterArc(const Arc &arc,
   return added;
 }
 
-template <class Arc, class R> void
-RelationDeterminizeFilter<Arc, R>::InitLabelMap(LabelMap *label_map) const {
+template <class Arc, class R>
+void RelationDeterminizeFilter<Arc, R>::InitLabelMap(
+    LabelMap *label_map) const {
   StateId src_head = tuple_->filter_state.GetState();
   Label label = kNoLabel;
   StateId nextstate = kNoStateId;
-  for (ArcIterator< Fst<Arc> > aiter(*fst_, src_head);
-       !aiter.Done();
+  for (ArcIterator<Fst<Arc>> aiter(*fst_, src_head); !aiter.Done();
        aiter.Next()) {
     const Arc &arc = aiter.Value();
     if (arc.ilabel == label && arc.nextstate == nextstate)
       continue;  // multiarc
     DeterminizeArc<StateTuple> det_arc(arc);
     det_arc.dest_tuple->filter_state = FilterState(arc.nextstate);
-    pair<Label, DeterminizeArc<StateTuple> > pr(arc.ilabel, det_arc);
+    std::pair<Label, DeterminizeArc<StateTuple>> pr(arc.ilabel, det_arc);
     label_map->insert(pr);
     label = arc.ilabel;
     nextstate = arc.nextstate;
@@ -228,14 +195,15 @@ class Disambiguator {
   typedef typename Arc::Label Label;
   // Ids arc with state id and arc position. Arc pos of -1 means
   // final (super-final transition).
-  typedef pair<StateId, ssize_t> ArcId;
+  typedef std::pair<StateId, ssize_t> ArcId;
 
-  Disambiguator() : candidates_(0), merge_(0), error_(false) { }
+  Disambiguator() : candidates_(0), merge_(0), error_(false) {}
 
-  void Disambiguate(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
-                    const DisambiguateOptions<Arc> &opts =
-                    DisambiguateOptions<Arc>()) {
+  void Disambiguate(
+      const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
+      const DisambiguateOptions<Arc> &opts = DisambiguateOptions<Arc>()) {
     VectorFst<Arc> sfst(ifst);
+    Connect(&sfst);
     ArcSort(&sfst, ArcCompare());
     PreDisambiguate(sfst, ofst, opts);
     ArcSort(ofst, ArcCompare());
@@ -251,14 +219,14 @@ class Disambiguator {
   // This sort order facilitates the predisambiguation.
   class ArcCompare {
    public:
-    bool operator() (Arc arc1, Arc arc2) const {
+    bool operator()(Arc arc1, Arc arc2) const {
       return arc1.ilabel < arc2.ilabel ||
-          (arc1.ilabel == arc2.ilabel && arc1.nextstate < arc2.nextstate);
+             (arc1.ilabel == arc2.ilabel && arc1.nextstate < arc2.nextstate);
     }
 
     uint64 Properties(uint64 props) const {
       return (props & kArcSortProperties) | kILabelSorted |
-          (props & kAcceptor ? kOLabelSorted : 0);
+             (props & kAcceptor ? kOLabelSorted : 0);
     }
   };
 
@@ -266,9 +234,7 @@ class Disambiguator {
   // This sort order facilitates ambiguity detection.
   class ArcIdCompare {
    public:
-    explicit ArcIdCompare(const vector<StateId> &head)
-        : head_(head) {
-    }
+    explicit ArcIdCompare(const std::vector<StateId> &head) : head_(head) {}
 
     bool operator()(const ArcId &a1, const ArcId &a2) const {
       StateId src1 = a1.first;
@@ -291,32 +257,32 @@ class Disambiguator {
     }
 
    private:
-    const vector<StateId> &head_;
+    const std::vector<StateId> &head_;
   };
 
   // A relation that determines if two states share a common future.
   class CommonFuture {
    public:
-    typedef GenericComposeStateTable<Arc, CharFilterState> T;
+    typedef GenericComposeStateTable<Arc, TrivialFilterState> T;
     typedef typename T::StateTuple StateTuple;
 
     // Needed for compilation with DeterminizeRelationFilter
     CommonFuture() {
-      FSTERROR() << "Disambiguate::CommonFuture: FST not provided";
+      FSTERROR() << "Disambiguate::CommonFuture: Fst not provided";
     }
 
     explicit CommonFuture(const Fst<Arc> &ifst) {
-      typedef Matcher< Fst<Arc> > M;
-      ComposeFstOptions<Arc, M, NullComposeFilter<M> > opts;
+      typedef Matcher<Fst<Arc>> M;
+      ComposeFstOptions<Arc, M, NullComposeFilter<M>> opts;
 
       // Ensures composition is between acceptors.
       bool trans = ifst.Properties(kNotAcceptor, true);
-      const Fst<Arc> *fsa = trans ?
-          new ProjectFst<Arc>(ifst, PROJECT_INPUT) : &ifst;
+      const Fst<Arc> *fsa =
+          trans ? new ProjectFst<Arc>(ifst, PROJECT_INPUT) : &ifst;
 
       opts.state_table = new T(*fsa, *fsa);
       ComposeFst<Arc> cfst(*fsa, *fsa, opts);
-      vector<bool> coaccess;
+      std::vector<bool> coaccess;
       uint64 props = 0;
       SccVisitor<Arc> scc_visitor(0, 0, &coaccess, &props);
       DfsVisit(cfst, &scc_visitor);
@@ -326,12 +292,11 @@ class Disambiguator {
           related_.insert(tuple.StatePair());
         }
       }
-      if (trans)
-        delete fsa;
+      if (trans) delete fsa;
     }
 
     bool operator()(const StateId s1, StateId s2) const {
-      pair<StateId, StateId> pr(s1, s2);
+      std::pair<StateId, StateId> pr(s1, s2);
       return related_.count(pr) > 0;
     }
 
@@ -339,10 +304,10 @@ class Disambiguator {
     // States s1 and s2 resp. are in this relation iff they there is a
     // path from s1 to a final state that has the same label as some
     // path from s2 to a final state.
-    set< pair<StateId, StateId> > related_;
+    std::set<std::pair<StateId, StateId>> related_;
   };
 
-  typedef multimap<ArcId, ArcId, ArcIdCompare> ArcIdMap;
+  typedef std::multimap<ArcId, ArcId, ArcIdCompare> ArcIdMap;
 
   // Returns the arc corresponding to ArcId a
   static Arc GetArc(const Fst<Arc> &fst, ArcId a) {
@@ -350,7 +315,7 @@ class Disambiguator {
       Arc arc(kNoLabel, kNoLabel, fst.Final(a.first), kNoStateId);
       return arc;
     } else {
-      ArcIterator< Fst<Arc> > aiter(fst, a.first);
+      ArcIterator<Fst<Arc>> aiter(fst, a.first);
       aiter.Seek(a.second);
       return aiter.Value();
     }
@@ -382,19 +347,19 @@ class Disambiguator {
   // from the initial state to s1 that has the same label as some path
   // from the initial state to s2.  We store only state pairs s1, s2
   // such that s1 <= s2.
-  set< pair<StateId, StateId> > coreachable_;
+  std::set<std::pair<StateId, StateId>> coreachable_;
   // Queue of disambiguation-related states to be processed. We store
   // only state pairs s1, s2 such that s1 <= s2.
-  list < pair<StateId, StateId> > queue_;
+  std::list<std::pair<StateId, StateId>> queue_;
   // Head state in the pre-disambiguation for a given state.
-  vector<StateId> head_;
+  std::vector<StateId> head_;
   // Maps from a candidate ambiguous arc A to each ambiguous candidate arc
   // B with the same label and destination state as A, whose source state s'
   // is coreachable with the source state s of A, and for which
   // head(s') < head(s).
   ArcIdMap *candidates_;
   // Set of ambiguous transitions to be removed.
-  set<ArcId> ambiguous_;
+  std::set<ArcId> ambiguous_;
   // States to merge due to quantization issues.
   UnionFind<StateId> *merge_;
   // Marks error condition.
@@ -402,10 +367,10 @@ class Disambiguator {
   DISALLOW_COPY_AND_ASSIGN(Disambiguator);
 };
 
-template <class Arc> void
-Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
-                                    MutableFst<Arc> *ofst,
-                                    const DisambiguateOptions<Arc> &opts) {
+template <class Arc>
+void Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
+                                         MutableFst<Arc> *ofst,
+                                         const DisambiguateOptions<Arc> &opts) {
   typedef DefaultCommonDivisor<Weight> Div;
   typedef RelationDeterminizeFilter<Arc, CommonFuture> Filt;
 
@@ -421,16 +386,17 @@ Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
   nopts.gc_limit = 0;  // Cache only the last state for fastest copy.
   if (opts.weight_threshold != Weight::Zero() ||
       opts.state_threshold != kNoStateId) {
+    /* TODO(riley): fails regression test; understand why
     if (ifst.Properties(kAcceptor, true)) {
-      vector<Weight> idistance, odistance;
+      std::vector<Weight> idistance, odistance;
       ShortestDistance(ifst, &idistance, true);
       DeterminizeFst<Arc> dfst(ifst, &idistance, &odistance, nopts);
-      PruneOptions< Arc, AnyArcFilter<Arc> > popts(opts.weight_threshold,
+      PruneOptions< Arc, AnyArcFilter<Arc>> popts(opts.weight_threshold,
                                                    opts.state_threshold,
                                                    AnyArcFilter<Arc>(),
                                                    &odistance);
       Prune(dfst, ofst, popts);
-    } else {
+      } else */ {
       *ofst = DeterminizeFst<Arc>(ifst, nopts);
       Prune(ofst, opts.weight_threshold, opts.state_threshold);
     }
@@ -440,18 +406,17 @@ Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
   head_.resize(ofst->NumStates(), kNoStateId);
 }
 
-template <class Arc> void
-Disambiguator<Arc>::FindAmbiguities(const ExpandedFst<Arc> &fst) {
-  if (fst.Start() == kNoStateId)
-    return;
+template <class Arc>
+void Disambiguator<Arc>::FindAmbiguities(const ExpandedFst<Arc> &fst) {
+  if (fst.Start() == kNoStateId) return;
 
   candidates_ = new ArcIdMap(ArcIdCompare(head_));
 
-  pair<StateId, StateId> start_pr(fst.Start(), fst.Start());
+  std::pair<StateId, StateId> start_pr(fst.Start(), fst.Start());
   coreachable_.insert(start_pr);
   queue_.push_back(start_pr);
   while (!queue_.empty()) {
-    const pair<StateId, StateId>  &pr = queue_.front();
+    const std::pair<StateId, StateId> &pr = queue_.front();
     StateId s1 = pr.first;
     StateId s2 = pr.second;
     queue_.pop_front();
@@ -462,14 +427,11 @@ Disambiguator<Arc>::FindAmbiguities(const ExpandedFst<Arc> &fst) {
 template <class Arc>
 void Disambiguator<Arc>::FindAmbiguousPairs(const ExpandedFst<Arc> &fst,
                                             StateId s1, StateId s2) {
-  if (fst.NumArcs(s2) > fst.NumArcs(s1))
-    FindAmbiguousPairs(fst, s2, s1);
+  if (fst.NumArcs(s2) > fst.NumArcs(s1)) FindAmbiguousPairs(fst, s2, s1);
 
-  SortedMatcher < Fst<Arc> > matcher(fst, MATCH_INPUT);
+  SortedMatcher<Fst<Arc>> matcher(fst, MATCH_INPUT);
   matcher.SetState(s2);
-  for (ArcIterator< Fst<Arc> > aiter(fst, s1);
-       !aiter.Done();
-       aiter.Next()) {
+  for (ArcIterator<Fst<Arc>> aiter(fst, s1); !aiter.Done(); aiter.Next()) {
     const Arc &arc1 = aiter.Value();
     ArcId a1(s1, aiter.Position());
     if (matcher.Find(arc1.ilabel)) {
@@ -481,20 +443,20 @@ void Disambiguator<Arc>::FindAmbiguousPairs(const ExpandedFst<Arc> &fst,
 
         // Actual transition is ambiguous
         if (s1 != s2 && arc1.nextstate == arc2.nextstate) {
-          pair<ArcId, ArcId> apr;
+          std::pair<ArcId, ArcId> apr;
           if (head_[s1] > head_[s2]) {
-            apr = make_pair(a1, a2);
+            apr = std::make_pair(a1, a2);
           } else {
-            apr = make_pair(a2, a1);
+            apr = std::make_pair(a2, a1);
           }
           candidates_->insert(apr);
         }
 
-        pair<StateId, StateId> spr;
+        std::pair<StateId, StateId> spr;
         if (arc1.nextstate <= arc2.nextstate) {
-          spr = make_pair(arc1.nextstate, arc2.nextstate);
+          spr = std::make_pair(arc1.nextstate, arc2.nextstate);
         } else {
-          spr = make_pair(arc2.nextstate, arc1.nextstate);
+          spr = std::make_pair(arc2.nextstate, arc1.nextstate);
         }
         // Not already marked as coreachable?
         if (coreachable_.find(spr) == coreachable_.end()) {
@@ -516,15 +478,15 @@ void Disambiguator<Arc>::FindAmbiguousPairs(const ExpandedFst<Arc> &fst,
   }
 
   // Super-final transition is ambiguous
-  if (s1 != s2 &&
-      fst.Final(s1) != Weight::Zero() && fst.Final(s2) != Weight::Zero()) {
+  if (s1 != s2 && fst.Final(s1) != Weight::Zero() &&
+      fst.Final(s2) != Weight::Zero()) {
     ArcId a1(s1, -1);
     ArcId a2(s2, -1);
-    pair<ArcId, ArcId> apr;
+    std::pair<ArcId, ArcId> apr;
     if (head_[s1] > head_[s2]) {
-      apr = make_pair(a1, a2);
+      apr = std::make_pair(a1, a2);
     } else {
-      apr = make_pair(a2, a1);
+      apr = std::make_pair(a2, a1);
     }
     candidates_->insert(apr);
   }
@@ -532,11 +494,9 @@ void Disambiguator<Arc>::FindAmbiguousPairs(const ExpandedFst<Arc> &fst,
 
 template <class Arc>
 void Disambiguator<Arc>::MarkAmbiguities() {
-  if (!candidates_)
-    return;
+  if (!candidates_) return;
 
-  for (typename ArcIdMap::iterator it = candidates_->begin();
-       it != candidates_->end(); ++it) {
+  for (auto it = candidates_->begin(); it != candidates_->end(); ++it) {
     ArcId a = it->first;
     ArcId b = it->second;
     if (ambiguous_.count(b) == 0)  // if b is not to be removed
@@ -549,13 +509,11 @@ void Disambiguator<Arc>::MarkAmbiguities() {
 
 template <class Arc>
 void Disambiguator<Arc>::RemoveSplits(MutableFst<Arc> *ofst) {
-  if (!merge_)
-    return;
+  if (!merge_) return;
 
   // 'Merges' split states to remove spurious ambiguities
   for (StateId s = 0; s < ofst->NumStates(); ++s) {
-    for (MutableArcIterator< MutableFst<Arc> > aiter(ofst, s);
-         !aiter.Done();
+    for (MutableArcIterator<MutableFst<Arc>> aiter(ofst, s); !aiter.Done();
          aiter.Next()) {
       Arc arc = aiter.Value();
       StateId nextstate = merge_->FindSet(arc.nextstate);
@@ -574,7 +532,7 @@ void Disambiguator<Arc>::RemoveSplits(MutableFst<Arc> *ofst) {
   candidates_ = 0;
   FindAmbiguities(*ofst);
   if (merge_) {  // shouldn't get here; sanity test
-    FSTERROR() << "Disambiguate: unable to remove spurious ambiguities";
+    FSTERROR() << "Disambiguate: Unable to remove spurious ambiguities";
     error_ = true;
     return;
   }
@@ -582,22 +540,19 @@ void Disambiguator<Arc>::RemoveSplits(MutableFst<Arc> *ofst) {
 
 template <class Arc>
 void Disambiguator<Arc>::RemoveAmbiguities(MutableFst<Arc> *ofst) {
-  if (ambiguous_.empty())
-    return;
+  if (ambiguous_.empty()) return;
   // Adds dead state to redirect ambiguous transitions to be removed
   StateId dead = ofst->AddState();
-  for (typename set<ArcId>::iterator it = ambiguous_.begin();
-       it != ambiguous_.end();
-       ++it) {
+  for (auto it = ambiguous_.begin(); it != ambiguous_.end(); ++it) {
     StateId s = it->first;
     ssize_t pos = it->second;
     if (pos >= 0) {  // actual transition
-      MutableArcIterator< MutableFst<Arc> > aiter(ofst, s);
+      MutableArcIterator<MutableFst<Arc>> aiter(ofst, s);
       aiter.Seek(pos);
       Arc arc = aiter.Value();
       arc.nextstate = dead;
       aiter.SetValue(arc);
-    } else {        // super-final transition
+    } else {  // super-final transition
       ofst->SetFinal(s, Weight::Zero());
     }
   }
@@ -622,16 +577,17 @@ void Disambiguator<Arc>::RemoveAmbiguities(MutableFst<Arc> *ofst) {
 // transducers that are unweighted or that are acyclic or that are unambiguous.
 //
 // References:
-// - Mehryar Mohri. "A Disambiguation Algorithm for Finite Automata and
-//   Functional Transducers". Proceedings CIAA 2012. Porto, Portugal, 2012.
+// - Mehryar Mohri and Michael Riley, "On the Disambiguation
+//   of Weighted Automata," ArXiv e-prints, cs-FL/1405.0500, 2014,
+//   http://arxiv.org/abs/1405.0500.
 template <class Arc>
-void Disambiguate(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
-                  const DisambiguateOptions<Arc> &opts
-                 = DisambiguateOptions<Arc>()) {
+void Disambiguate(
+    const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
+    const DisambiguateOptions<Arc> &opts = DisambiguateOptions<Arc>()) {
   Disambiguator<Arc> disamb;
   disamb.Disambiguate(ifst, ofst, opts);
 }
 
 }  // namespace fst
 
-#endif  // FST_LIB_DISAMBIGUATE_H__
+#endif  // FST_LIB_DISAMBIGUATE_H_
