@@ -1,21 +1,6 @@
-// collection.h
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
 // Class to store a collection of ordered (multi-)sets with elements of type T.
 
 #ifndef FST_EXTENSIONS_PDT_COLLECTION_H__
@@ -23,7 +8,6 @@
 
 #include <algorithm>
 #include <vector>
-using std::vector;
 
 #include <fst/bi-table.h>
 
@@ -44,7 +28,7 @@ class Collection {
     Node() : node_id(kNoNodeId), element(T()) {}
     Node(I i, const T &t) : node_id(i), element(t) {}
 
-    bool operator==(const Node& n) const {
+    bool operator==(const Node &n) const {
       return n.node_id == node_id && n.element == element;
     }
   };
@@ -60,7 +44,7 @@ class Collection {
   class SetIterator {
    public:
     SetIterator(I id, Node node, NodeTable *node_table)
-        :id_(id), node_(node), node_table_(node_table) {}
+        : id_(id), node_(node), node_table_(node_table) {}
 
     bool Done() const { return id_ == kNoNodeId; }
 
@@ -68,13 +52,12 @@ class Collection {
 
     void Next() {
       id_ = node_.node_id;
-      if (id_ != kNoNodeId)
-        node_ = node_table_->FindEntry(id_);
+      if (id_ != kNoNodeId) node_ = node_table_->FindEntry(id_);
     }
 
    private:
-    I id_;                     // Iterator set node id
-    Node node_;                // Iterator set node
+    I id_;       // Iterator set node id
+    Node node_;  // Iterator set node
     NodeTable *node_table_;
   };
 
@@ -82,7 +65,7 @@ class Collection {
 
   // Lookups integer ID from ordered multi-set. If it doesn't exist
   // and 'insert' is true, then adds it. Otherwise returns -1.
-  I FindId(const vector<T> &set, bool insert = true) {
+  I FindId(const std::vector<T> &set, bool insert = true) {
     I node_id = kNoNodeId;
     for (ssize_t i = set.size() - 1; i >= 0; --i) {
       Node node(node_id, set[i]);
@@ -114,11 +97,14 @@ class Collection {
   DISALLOW_COPY_AND_ASSIGN(Collection);
 };
 
-template<class I, class T> const I Collection<I, T>::kNoNodeId = -1;
+template <class I, class T>
+const I Collection<I, T>::kNoNodeId = -1;
 
-template <class I, class T> const size_t Collection<I, T>::kPrime = 7853;
+template <class I, class T>
+const size_t Collection<I, T>::kPrime = 7853;
 
-template <class I, class T> std::hash<T> Collection<I, T>::hash_;
+template <class I, class T>
+hash<T> Collection<I, T>::hash_;
 
 }  // namespace fst
 
