@@ -1,21 +1,7 @@
-// paren.h
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// Common classes for PDT parentheses
+// Common classes for PDT parentheses.
 
 // \file
 
@@ -23,18 +9,14 @@
 #define FST_EXTENSIONS_PDT_PAREN_H_
 
 #include <algorithm>
-#include <unordered_map>
-using std::unordered_map;
-using std::unordered_multimap;
-#include <unordered_set>
-using std::unordered_set;
-using std::unordered_multiset;
 #include <set>
+#include <unordered_map>
+#include <unordered_set>
 
-#include <fst/extensions/pdt/pdt.h>
 #include <fst/extensions/pdt/collection.h>
-#include <fst/fst.h>
+#include <fst/extensions/pdt/pdt.h>
 #include <fst/dfs-visit.h>
+#include <fst/fst.h>
 
 
 namespace fst {
@@ -56,16 +38,15 @@ class ParenState {
     }
   };
 
-  Label paren_id;     // ID of open (close) paren
-  StateId state_id;   // destination (source) state of open (close) paren
+  Label paren_id;    // ID of open (close) paren
+  StateId state_id;  // destination (source) state of open (close) paren
 
   ParenState() : paren_id(kNoLabel), state_id(kNoStateId) {}
 
   ParenState(Label p, StateId s) : paren_id(p), state_id(s) {}
 
   bool operator==(const ParenState<A> &p) const {
-    if (&p == this)
-      return true;
+    if (&p == this) return true;
     return p.paren_id == this->paren_id && p.state_id == this->state_id;
   }
 
@@ -73,7 +54,7 @@ class ParenState {
 
   bool operator<(const ParenState<A> &p) const {
     return paren_id < this->paren.id ||
-        (p.paren_id == this->paren.id && p.state_id < this->state_id);
+           (p.paren_id == this->paren.id && p.state_id < this->state_id);
   }
 
  private:
@@ -83,7 +64,6 @@ class ParenState {
 template <class A>
 const size_t ParenState<A>::kPrime = 7853;
 
-
 // Creates an FST-style iterator from STL map and iterator.
 template <class M>
 class MapIterator {
@@ -119,18 +99,19 @@ class PdtParenReachable {
  public:
   typedef typename A::StateId StateId;
   typedef typename A::Label Label;
+
  public:
   // Maps from state ID to reachable paren IDs from (to) that state.
-  typedef unordered_multimap<StateId, Label> ParenMultiMap;
+  typedef std::unordered_multimap<StateId, Label> ParenMultiMap;
 
   // Maps from paren ID and state ID to reachable state set ID
-  typedef unordered_map<ParenState<A>, ssize_t,
-                   typename ParenState<A>::Hash> StateSetMap;
+  typedef std::unordered_map<ParenState<A>, ssize_t,
+                             typename ParenState<A>::Hash> StateSetMap;
 
   // Maps from paren ID and state ID to arcs exiting that state with that
   // Label.
-  typedef unordered_multimap<ParenState<A>, A,
-                        typename ParenState<A>::Hash> ParenArcMultiMap;
+  typedef std::unordered_multimap<
+      ParenState<A>, A, typename ParenState<A>::Hash> ParenArcMultiMap;
 
   typedef MapIterator<ParenMultiMap> ParenIterator;
 
@@ -138,30 +119,27 @@ class PdtParenReachable {
 
   typedef typename Collection<ssize_t, StateId>::SetIterator SetIterator;
 
-  // Computes close (open) parenthesis reachabilty information for
+  // Computes close (open) parenthesis reachability information for
   // a PDT with bounded stack.
   PdtParenReachable(const Fst<A> &fst,
-                    const vector<pair<Label, Label> > &parens, bool close)
-      : fst_(fst),
-        parens_(parens),
-        close_(close),
-        error_(false) {
+                    const std::vector<std::pair<Label, Label>> &parens,
+                    bool close)
+      : fst_(fst), parens_(parens), close_(close), error_(false) {
     for (Label i = 0; i < parens.size(); ++i) {
-      const pair<Label, Label>  &p = parens[i];
+      const std::pair<Label, Label> &p = parens[i];
       paren_id_map_[p.first] = i;
       paren_id_map_[p.second] = i;
     }
 
     if (close_) {
       StateId start = fst.Start();
-      if (start == kNoStateId)
-        return;
+      if (start == kNoStateId) return;
       if (!DFSearch(start)) {
         FSTERROR() << "PdtReachable: Underlying cyclicity not supported";
         error_ = true;
       }
     } else {
-      FSTERROR() << "PdtParenReachable: open paren info not implemented";
+      FSTERROR() << "PdtParenReachable: Open paren info not implemented";
       error_ = true;
     }
   }
@@ -207,18 +185,18 @@ class PdtParenReachable {
   void ComputeStateSet(StateId s);
 
   // Gather state set(s) from state 'nexts'.
-  void UpdateStateSet(StateId nexts, set<Label> *paren_set,
-                      vector< set<StateId> > *state_sets) const;
+  void UpdateStateSet(StateId nexts, std::set<Label> *paren_set,
+                      std::vector<std::set<StateId>> *state_sets) const;
 
   const Fst<A> &fst_;
-  const vector<pair<Label, Label> > &parens_;         // Paren ID -> Labels
-  bool close_;                                        // Close/open paren info?
-  unordered_map<Label, Label> paren_id_map_;               // Paren labels -> ID
-  ParenMultiMap paren_multimap_;                      // Paren reachability
-  ParenArcMultiMap paren_arc_multimap_;               // Paren Arcs
-  vector<char> state_color_;                          // DFS state
-  mutable Collection<ssize_t, StateId> state_sets_;   // Reachable states -> ID
-  StateSetMap set_map_;                               // ID -> Reachable states
+  const std::vector<std::pair<Label, Label>> &parens_;  // Paren ID -> Labels
+  bool close_;                                       // Close/open paren info?
+  std::unordered_map<Label, Label> paren_id_map_;      // Paren labels -> ID
+  ParenMultiMap paren_multimap_;                     // Paren reachability
+  ParenArcMultiMap paren_arc_multimap_;              // Paren Arcs
+  std::vector<char> state_color_;                    // DFS state
+  mutable Collection<ssize_t, StateId> state_sets_;  // Reachable states -> ID
+  StateSetMap set_map_;                              // ID -> Reachable states
   bool error_;
   DISALLOW_COPY_AND_ASSIGN(PdtParenReachable);
 };
@@ -226,44 +204,35 @@ class PdtParenReachable {
 // DFS that gathers paren and state set information.
 template <class A>
 bool PdtParenReachable<A>::DFSearch(StateId s) {
-  if (s >= state_color_.size())
-    state_color_.resize(s + 1, kDfsWhite);
+  if (s >= state_color_.size()) state_color_.resize(s + 1, kDfsWhite);
 
-  if (state_color_[s] == kDfsBlack)
-    return true;
+  if (state_color_[s] == kDfsBlack) return true;
 
-  if (state_color_[s] == kDfsGrey)
-    return false;
+  if (state_color_[s] == kDfsGrey) return false;
 
   state_color_[s] = kDfsGrey;
 
-  for (ArcIterator<Fst<A> > aiter(fst_, s);
-       !aiter.Done();
-       aiter.Next()) {
+  for (ArcIterator<Fst<A>> aiter(fst_, s); !aiter.Done(); aiter.Next()) {
     const A &arc = aiter.Value();
 
-    typename unordered_map<Label, Label>::const_iterator pit
-        = paren_id_map_.find(arc.ilabel);
-    if (pit != paren_id_map_.end()) {               // paren?
+    typename std::unordered_map<Label, Label>::const_iterator pit =
+        paren_id_map_.find(arc.ilabel);
+    if (pit != paren_id_map_.end()) {  // paren?
       Label paren_id = pit->second;
       if (arc.ilabel == parens_[paren_id].first) {  // open paren
-        if (!DFSearch(arc.nextstate))
-          return false;
+        if (!DFSearch(arc.nextstate)) return false;
         for (SetIterator set_iter = FindStates(paren_id, arc.nextstate);
              !set_iter.Done(); set_iter.Next()) {
           for (ParenArcIterator paren_arc_iter =
                    FindParenArcs(paren_id, set_iter.Element());
-               !paren_arc_iter.Done();
-               paren_arc_iter.Next()) {
+               !paren_arc_iter.Done(); paren_arc_iter.Next()) {
             const A &cparc = paren_arc_iter.Value();
-            if (!DFSearch(cparc.nextstate))
-              return false;
+            if (!DFSearch(cparc.nextstate)) return false;
           }
         }
       }
-    } else {                                       // non-paren
-      if(!DFSearch(arc.nextstate))
-        return false;
+    } else {  // non-paren
+      if (!DFSearch(arc.nextstate)) return false;
     }
   }
   ComputeStateSet(s);
@@ -274,50 +243,44 @@ bool PdtParenReachable<A>::DFSearch(StateId s) {
 // Unions state sets together gathered by the DFS.
 template <class A>
 void PdtParenReachable<A>::ComputeStateSet(StateId s) {
-  set<Label> paren_set;
-  vector< set<StateId> > state_sets(parens_.size());
-  for (ArcIterator< Fst<A> > aiter(fst_, s);
-       !aiter.Done();
-       aiter.Next()) {
+  std::set<Label> paren_set;
+  std::vector<std::set<StateId>> state_sets(parens_.size());
+  for (ArcIterator<Fst<A>> aiter(fst_, s); !aiter.Done(); aiter.Next()) {
     const A &arc = aiter.Value();
 
-    typename unordered_map<Label, Label>::const_iterator pit
-        = paren_id_map_.find(arc.ilabel);
-    if (pit != paren_id_map_.end()) {               // paren?
+    typename std::unordered_map<Label, Label>::const_iterator pit =
+        paren_id_map_.find(arc.ilabel);
+    if (pit != paren_id_map_.end()) {  // paren?
       Label paren_id = pit->second;
       if (arc.ilabel == parens_[paren_id].first) {  // open paren
-        for (SetIterator set_iter =
-                 FindStates(paren_id, arc.nextstate);
+        for (SetIterator set_iter = FindStates(paren_id, arc.nextstate);
              !set_iter.Done(); set_iter.Next()) {
           for (ParenArcIterator paren_arc_iter =
                    FindParenArcs(paren_id, set_iter.Element());
-               !paren_arc_iter.Done();
-               paren_arc_iter.Next()) {
+               !paren_arc_iter.Done(); paren_arc_iter.Next()) {
             const A &cparc = paren_arc_iter.Value();
             UpdateStateSet(cparc.nextstate, &paren_set, &state_sets);
           }
         }
-      } else {                                      // close paren
+      } else {  // close paren
         paren_set.insert(paren_id);
         state_sets[paren_id].insert(s);
         ParenState<A> paren_state(paren_id, s);
-        paren_arc_multimap_.insert(make_pair(paren_state, arc));
+        paren_arc_multimap_.insert(std::make_pair(paren_state, arc));
       }
-    } else {                                        // non-paren
+    } else {  // non-paren
       UpdateStateSet(arc.nextstate, &paren_set, &state_sets);
     }
   }
 
-  vector<StateId> state_set;
-  for (typename set<Label>::iterator paren_iter = paren_set.begin();
-       paren_iter != paren_set.end(); ++paren_iter) {
+  std::vector<StateId> state_set;
+  for (auto paren_iter = paren_set.begin(); paren_iter != paren_set.end();
+       ++paren_iter) {
     state_set.clear();
     Label paren_id = *paren_iter;
-    paren_multimap_.insert(make_pair(s, paren_id));
-    for (typename set<StateId>::iterator state_iter
-             = state_sets[paren_id].begin();
-         state_iter != state_sets[paren_id].end();
-         ++state_iter) {
+    paren_multimap_.insert(std::make_pair(s, paren_id));
+    for (auto state_iter = state_sets[paren_id].begin();
+         state_iter != state_sets[paren_id].end(); ++state_iter) {
       state_set.push_back(*state_iter);
     }
     ParenState<A> paren_state(paren_id, s);
@@ -328,20 +291,19 @@ void PdtParenReachable<A>::ComputeStateSet(StateId s) {
 // Gather state set(s) from state 'nexts'.
 template <class A>
 void PdtParenReachable<A>::UpdateStateSet(
-    StateId nexts, set<Label> *paren_set,
-    vector< set<StateId> > *state_sets) const {
-  for(ParenIterator paren_iter = FindParens(nexts);
-      !paren_iter.Done(); paren_iter.Next()) {
+    StateId nexts, std::set<Label> *paren_set,
+    std::vector<std::set<StateId>> *state_sets) const {
+  for (ParenIterator paren_iter = FindParens(nexts); !paren_iter.Done();
+       paren_iter.Next()) {
     Label paren_id = paren_iter.Value();
     paren_set->insert(paren_id);
-    for (SetIterator set_iter = FindStates(paren_id, nexts);
-         !set_iter.Done(); set_iter.Next()) {
+    for (SetIterator set_iter = FindStates(paren_id, nexts); !set_iter.Done();
+         set_iter.Next()) {
       (*state_sets)[paren_id].insert(set_iter.Element());
     }
   }
 }
 
-
 // Store balancing parenthesis data for a PDT. Allows on-the-fly
 // construction (e.g. in PdtShortestPath) unlike PdtParenReachable above.
 template <class A>
@@ -351,18 +313,19 @@ class PdtBalanceData {
   typedef typename A::Label Label;
 
   // Hash set for open parens
-  typedef unordered_set<ParenState<A>, typename ParenState<A>::Hash> OpenParenSet;
+  typedef std::unordered_set<ParenState<A>, typename ParenState<A>::Hash>
+      OpenParenSet;
 
   // Maps from open paren destination state to parenthesis ID.
-  typedef unordered_multimap<StateId, Label> OpenParenMap;
+  typedef std::unordered_multimap<StateId, Label> OpenParenMap;
 
   // Maps from open paren state to source states of matching close parens
-  typedef unordered_multimap<ParenState<A>, StateId,
-                        typename ParenState<A>::Hash> CloseParenMap;
+  typedef std::unordered_multimap<ParenState<A>, StateId,
+                                  typename ParenState<A>::Hash> CloseParenMap;
 
   // Maps from open paren state to close source set ID
-  typedef unordered_map<ParenState<A>, ssize_t,
-                   typename ParenState<A>::Hash> CloseSourceMap;
+  typedef std::unordered_map<ParenState<A>, ssize_t,
+                             typename ParenState<A>::Hash> CloseSourceMap;
 
   typedef typename Collection<ssize_t, StateId>::SetIterator SetIterator;
 
@@ -378,7 +341,7 @@ class PdtBalanceData {
     ParenState<A> key(paren_id, open_dest);
     if (!open_paren_set_.count(key)) {
       open_paren_set_.insert(key);
-      open_paren_map_.insert(make_pair(open_dest, paren_id));
+      open_paren_map_.insert(std::make_pair(open_dest, paren_id));
     }
   }
 
@@ -389,7 +352,7 @@ class PdtBalanceData {
   void CloseInsert(Label paren_id, StateId open_dest, StateId close_source) {
     ParenState<A> key(paren_id, open_dest);
     if (open_paren_set_.count(key))
-      close_paren_map_.insert(make_pair(key, close_source));
+      close_paren_map_.insert(std::make_pair(key, close_source));
   }
 
   // Find close paren source states matching an open parenthesis.
@@ -411,21 +374,20 @@ class PdtBalanceData {
   // before Find(open_dest). Stores close paren source state sets
   // efficiently.
   void FinishInsert(StateId open_dest) {
-    vector<StateId> close_sources;
-    for (typename OpenParenMap::iterator oit = open_paren_map_.find(open_dest);
+    std::vector<StateId> close_sources;
+    for (auto oit = open_paren_map_.find(open_dest);
          oit != open_paren_map_.end() && oit->first == open_dest;) {
       Label paren_id = oit->second;
       close_sources.clear();
       ParenState<A> okey(paren_id, open_dest);
       open_paren_set_.erase(open_paren_set_.find(okey));
-      for (typename CloseParenMap::iterator cit = close_paren_map_.find(okey);
+      for (auto cit = close_paren_map_.find(okey);
            cit != close_paren_map_.end() && cit->first == okey;) {
         close_sources.push_back(cit->second);
         close_paren_map_.erase(cit++);
       }
-      sort(close_sources.begin(), close_sources.end());
-      typename vector<StateId>::iterator unique_end =
-          unique(close_sources.begin(), close_sources.end());
+      std::sort(close_sources.begin(), close_sources.end());
+      auto unique_end = std::unique(close_sources.begin(), close_sources.end());
       close_sources.resize(unique_end - close_sources.begin());
 
       if (!close_sources.empty())
@@ -436,50 +398,46 @@ class PdtBalanceData {
 
   // Return a new balance data object representing the reversed balance
   // information.
-  PdtBalanceData<A> *Reverse(StateId num_states,
-                               StateId num_split,
-                               StateId state_id_shift) const;
+  PdtBalanceData<A> *Reverse(StateId num_states, StateId num_split,
+                             StateId state_id_shift) const;
 
  private:
-  OpenParenSet open_paren_set_;                      // open par. at dest?
+  OpenParenSet open_paren_set_;  // open par. at dest?
 
   OpenParenMap open_paren_map_;                      // open parens per state
   ParenState<A> open_dest_;                          // cur open dest. state
   typename OpenParenMap::const_iterator open_iter_;  // cur open parens/state
 
-  CloseParenMap close_paren_map_;                    // close states/open
-                                                     //  paren and state
+  CloseParenMap close_paren_map_;  // close states/open
+                                   //  paren and state
 
-  CloseSourceMap close_source_map_;                  // paren, state to set ID
+  CloseSourceMap close_source_map_;  // paren, state to set ID
   mutable Collection<ssize_t, StateId> close_source_sets_;
 };
 
 // Return a new balance data object representing the reversed balance
 // information.
 template <class A>
-PdtBalanceData<A> *PdtBalanceData<A>::Reverse(
-    StateId num_states,
-    StateId num_split,
-    StateId state_id_shift) const {
+PdtBalanceData<A> *PdtBalanceData<A>::Reverse(StateId num_states,
+                                              StateId num_split,
+                                              StateId state_id_shift) const {
   PdtBalanceData<A> *bd = new PdtBalanceData<A>;
-  unordered_set<StateId> close_sources;
+  std::unordered_set<StateId> close_sources;
   StateId split_size = num_states / num_split;
 
-  for (StateId i = 0; i < num_states; i+= split_size) {
+  for (StateId i = 0; i < num_states; i += split_size) {
     close_sources.clear();
 
-    for (typename CloseSourceMap::const_iterator
-             sit = close_source_map_.begin();
-         sit != close_source_map_.end();
-         ++sit) {
+    for (typename CloseSourceMap::const_iterator sit =
+             close_source_map_.begin();
+         sit != close_source_map_.end(); ++sit) {
       ParenState<A> okey = sit->first;
       StateId open_dest = okey.state_id;
       Label paren_id = okey.paren_id;
       for (SetIterator set_iter = close_source_sets_.FindSet(sit->second);
            !set_iter.Done(); set_iter.Next()) {
         StateId close_source = set_iter.Element();
-        if ((close_source < i) || (close_source >= i + split_size))
-          continue;
+        if ((close_source < i) || (close_source >= i + split_size)) continue;
         close_sources.insert(close_source + state_id_shift);
         bd->OpenInsert(paren_id, close_source + state_id_shift);
         bd->CloseInsert(paren_id, close_source + state_id_shift,
@@ -487,18 +445,15 @@ PdtBalanceData<A> *PdtBalanceData<A>::Reverse(
       }
     }
 
-    for (typename unordered_set<StateId>::const_iterator it
-             = close_sources.begin();
-         it != close_sources.end();
-         ++it) {
+    for (typename std::unordered_set<StateId>::const_iterator it =
+             close_sources.begin();
+         it != close_sources.end(); ++it) {
       bd->FinishInsert(*it);
     }
-
   }
   return bd;
 }
 
-
 }  // namespace fst
 
 #endif  // FST_EXTENSIONS_PDT_PAREN_H_
