@@ -1,43 +1,27 @@
-// fst.h
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
+//
+// FST abstract base class definition, state and arc iterator interface, and
+// suggested base implementation.
 
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
-// Finite-State Transducer (FST) - abstract base class definition,
-// state and arc iterator interface, and suggested base implementation.
-//
-
-#ifndef FST_LIB_FST_H__
-#define FST_LIB_FST_H__
+#ifndef FST_LIB_FST_H_
+#define FST_LIB_FST_H_
 
 #include <stddef.h>
 #include <sys/types.h>
 #include <cmath>
+#include <iostream>
+#include <sstream>
 #include <string>
 
 #include <fst/compat.h>
 #include <fst/types.h>
+#include <fstream>
 
 #include <fst/arc.h>
 #include <fst/memory.h>
 #include <fst/properties.h>
 #include <fst/register.h>
-#include <iostream>
-#include <fstream>
-#include <sstream>
 #include <fst/symbol-table.h>
 #include <fst/util.h>
 
@@ -46,12 +30,15 @@ DECLARE_bool(fst_align);
 
 namespace fst {
 
-bool IsFstHeader(istream &, const string &);
+bool IsFstHeader(std::istream &, const string &);
 
 class FstHeader;
-template <class A> class StateIteratorData;
-template <class A> class ArcIteratorData;
-template <class A> class MatcherBase;
+template <class A>
+struct StateIteratorData;
+template <class A>
+struct ArcIteratorData;
+template <class A>
+class MatcherBase;
 
 struct FstReadOptions {
   // FileReadMode(s) are advisory, there are many conditions than prevent a
@@ -62,40 +49,45 @@ struct FstReadOptions {
   string source;                // Where you're reading from
   const FstHeader *header;      // Pointer to Fst header. If non-zero, use
                                 // this info (don't read a stream header)
-  const SymbolTable* isymbols;  // Pointer to input symbols. If non-zero, use
+  const SymbolTable *isymbols;  // Pointer to input symbols. If non-zero, use
                                 // this info (read and skip stream isymbols)
-  const SymbolTable* osymbols;  // Pointer to output symbols. If non-zero, use
+  const SymbolTable *osymbols;  // Pointer to output symbols. If non-zero, use
                                 // this info (read and skip stream osymbols)
   FileReadMode mode;            // Read or map files (advisory, if possible)
   bool read_isymbols;           // Read isymbols, if any, default true
   bool read_osymbols;           // Read osymbols, if any, default true
 
-  explicit FstReadOptions(const string& src = "<unspecified>",
-                          const FstHeader *hdr = 0,
-                          const SymbolTable* isym = 0,
-                          const SymbolTable* osym = 0);
+  explicit FstReadOptions(const string &src = "<unspecified>",
+                          const FstHeader *hdr = 0, const SymbolTable *isym = 0,
+                          const SymbolTable *osym = 0);
 
-  explicit FstReadOptions(const string& src,
-                          const SymbolTable* isym,
-                          const SymbolTable* osym = 0);
+  explicit FstReadOptions(const string &src, const SymbolTable *isym,
+                          const SymbolTable *osym = 0);
 
   // Helper function to convert strings FileReadModes into their enum value.
   static FileReadMode ReadMode(const string &mode);
+
+  // Outputs a debug string for the FstReadOptions object.
+  string DebugString() const;
 };
 
 struct FstWriteOptions {
-  string source;                 // Where you're writing to
-  bool write_header;             // Write the header?
-  bool write_isymbols;           // Write input symbols?
-  bool write_osymbols;           // Write output symbols?
-  bool align;                    // Write data aligned where appropriate;
-                                 // this may fail on pipes
-
-  explicit FstWriteOptions(const string& src = "<unspecifed>",
-                           bool hdr = true, bool isym = true,
-                           bool osym = true, bool alig = FLAGS_fst_align)
-      : source(src), write_header(hdr),
-        write_isymbols(isym), write_osymbols(osym), align(alig) {}
+  string source;        // Where you're writing to
+  bool write_header;    // Write the header?
+  bool write_isymbols;  // Write input symbols?
+  bool write_osymbols;  // Write output symbols?
+  bool align;           // Write data aligned where appropriate;
+                        // this may fail on pipes
+  bool stream_write;    // Avoid seek operations in writing.
+  explicit FstWriteOptions(const string &src = "<unspecifed>", bool hdr = true,
+                           bool isym = true, bool osym = true,
+                           bool alig = FLAGS_fst_align, bool stm = false)
+      : source(src),
+        write_header(hdr),
+        write_isymbols(isym),
+        write_osymbols(osym),
+        align(alig),
+        stream_write(stm) {}
 };
 
 //
@@ -106,13 +98,18 @@ struct FstWriteOptions {
 class FstHeader {
  public:
   enum {
-    HAS_ISYMBOLS = 0x1,          // Has input symbol table
-    HAS_OSYMBOLS = 0x2,          // Has output symbol table
-    IS_ALIGNED   = 0x4,          // Memory-aligned (where appropriate)
+    HAS_ISYMBOLS = 0x1,  // Has input symbol table
+    HAS_OSYMBOLS = 0x2,  // Has output symbol table
+    IS_ALIGNED = 0x4,    // Memory-aligned (where appropriate)
   } Flags;
 
-  FstHeader() : version_(0), flags_(0), properties_(0), start_(-1),
-                numstates_(0), numarcs_(0) {}
+  FstHeader()
+      : version_(0),
+        flags_(0),
+        properties_(0),
+        start_(-1),
+        numstates_(0),
+        numarcs_(0) {}
   const string &FstType() const { return fsttype_; }
   const string &ArcType() const { return arctype_; }
   int32 Version() const { return version_; }
@@ -122,8 +119,8 @@ class FstHeader {
   int64 NumStates() const { return numstates_; }
   int64 NumArcs() const { return numarcs_; }
 
-  void SetFstType(const string& type) { fsttype_ = type; }
-  void SetArcType(const string& type) { arctype_ = type; }
+  void SetFstType(const string &type) { fsttype_ = type; }
+  void SetArcType(const string &type) { arctype_ = type; }
   void SetVersion(int32 version) { version_ = version; }
   void SetFlags(int32 flags) { flags_ = flags; }
   void SetProperties(uint64 properties) { properties_ = properties; }
@@ -131,28 +128,32 @@ class FstHeader {
   void SetNumStates(int64 numstates) { numstates_ = numstates; }
   void SetNumArcs(int64 numarcs) { numarcs_ = numarcs; }
 
-  bool Read(istream &strm, const string &source, bool rewind = false);
-  bool Write(ostream &strm, const string &source) const;
+  bool Read(std::istream &strm, const string &source,  // NOLINT
+            bool rewind = false);
+  bool Write(std::ostream &strm, const string &source) const;  // NOLINT
+
+  // Outputs a debug string for the FstHeader object.
+  string DebugString() const;
 
  private:
-
-  string fsttype_;                   // E.g. "vector"
-  string arctype_;                   // E.g. "standard"
-  int32 version_;                    // Type version #
-  int32 flags_;                      // File format bits
-  uint64 properties_;                // FST property bits
-  int64 start_;                      // Start state
-  int64 numstates_;                  // # of states
-  int64 numarcs_;                    // # of arcs
+  string fsttype_;     // E.g. "vector"
+  string arctype_;     // E.g. "standard"
+  int32 version_;      // Type version #
+  int32 flags_;        // File format bits
+  uint64 properties_;  // FST property bits
+  int64 start_;        // Start state
+  int64 numstates_;    // # of states
+  int64 numarcs_;      // # of arcs
 };
 
-
 // Specifies matcher action.
-enum MatchType { MATCH_INPUT,      // Match input label.
-                 MATCH_OUTPUT,     // Match output label.
-                 MATCH_BOTH,       // Match input or output label.
-                 MATCH_NONE,       // Match nothing.
-                 MATCH_UNKNOWN };  // Match type unknown.
+enum MatchType {
+  MATCH_INPUT,   // Match input label.
+  MATCH_OUTPUT,  // Match output label.
+  MATCH_BOTH,    // Match input or output label.
+  MATCH_NONE,    // Match nothing.
+  MATCH_UNKNOWN
+};  // Match type unknown.
 
 //
 // Fst INTERFACE CLASS DEFINITION
@@ -170,24 +171,23 @@ class Fst {
 
   virtual ~Fst() {}
 
-  virtual StateId Start() const = 0;          // Initial state
+  virtual StateId Start() const = 0;  // Initial state
 
-  virtual Weight Final(StateId) const = 0;    // State's final weight
+  virtual Weight Final(StateId) const = 0;  // State's final weight
 
   virtual size_t NumArcs(StateId) const = 0;  // State's arc count
 
-  virtual size_t NumInputEpsilons(StateId)
-      const = 0;                              // State's input epsilon count
+  virtual size_t NumInputEpsilons(
+      StateId) const = 0;  // State's input epsilon count
 
-  virtual size_t NumOutputEpsilons(StateId)
-      const = 0;                              // State's output epsilon count
+  virtual size_t NumOutputEpsilons(
+      StateId) const = 0;  // State's output epsilon count
 
   // If test=false, return stored properties bits for mask (some poss. unknown)
   // If test=true, return property bits for mask (computing o.w. unknown)
-  virtual uint64 Properties(uint64 mask, bool test)
-      const = 0;  // Property bits
+  virtual uint64 Properties(uint64 mask, bool test) const = 0;  // Property bits
 
-  virtual const string& Type() const = 0;    // Fst type name
+  virtual const string &Type() const = 0;  // Fst type name
 
   // Get a copy of this Fst. The copying behaves as follows:
   //
@@ -206,24 +206,23 @@ class Fst {
   virtual Fst<A> *Copy(bool safe = false) const = 0;
 
   // Read an Fst from an input stream; returns NULL on error
-  static Fst<A> *Read(istream &strm, const FstReadOptions &opts) {
+  static Fst<A> *Read(std::istream &strm,  // NOLINT
+                      const FstReadOptions &opts) {
     FstReadOptions ropts(opts);
     FstHeader hdr;
     if (ropts.header)
       hdr = *opts.header;
     else {
-      if (!hdr.Read(strm, opts.source))
-        return 0;
+      if (!hdr.Read(strm, opts.source)) return nullptr;
       ropts.header = &hdr;
     }
     FstRegister<A> *registr = FstRegister<A>::GetRegister();
     const typename FstRegister<A>::Reader reader =
-      registr->GetReader(hdr.FstType());
+        registr->GetReader(hdr.FstType());
     if (!reader) {
-      LOG(ERROR) << "Fst::Read: Unknown FST type \"" << hdr.FstType()
-                 << "\" (arc type = \"" << A::Type()
-                 << "\"): " << ropts.source;
-      return 0;
+      LOG(ERROR) << "Fst::Read: Unknown Fst type " << hdr.FstType()
+                 << " (arc type = " << A::Type() << "): " << ropts.source;
+      return nullptr;
     }
     return reader(strm, ropts);
   }
@@ -232,19 +231,21 @@ class Fst {
   // Empty filename reads from standard input
   static Fst<A> *Read(const string &filename) {
     if (!filename.empty()) {
-      ifstream strm(filename.c_str(), ifstream::in | ifstream::binary);
+      std::ifstream strm(filename.c_str(),
+                              std::ios_base::in | std::ios_base::binary);
       if (!strm) {
         LOG(ERROR) << "Fst::Read: Can't open file: " << filename;
-        return 0;
+        return nullptr;
       }
       return Read(strm, FstReadOptions(filename));
     } else {
-      return Read(cin, FstReadOptions("standard input"));
+      return Read(std::cin, FstReadOptions("standard input"));
     }
   }
 
   // Write an Fst to an output stream; return false on error
-  virtual bool Write(ostream &strm, const FstWriteOptions &opts) const {
+  virtual bool Write(std::ostream &strm,  // NOLINT
+                     const FstWriteOptions &opts) const {
     LOG(ERROR) << "Fst::Write: No write stream method for " << Type()
                << " Fst type";
     return false;
@@ -259,10 +260,10 @@ class Fst {
   }
 
   // Return input label symbol table; return NULL if not specified
-  virtual const SymbolTable* InputSymbols() const = 0;
+  virtual const SymbolTable *InputSymbols() const = 0;
 
   // Return output label symbol table; return NULL if not specified
-  virtual const SymbolTable* OutputSymbols() const = 0;
+  virtual const SymbolTable *OutputSymbols() const = 0;
 
   // For generic state iterator construction; not normally called
   // directly by users.
@@ -279,19 +280,21 @@ class Fst {
  protected:
   bool WriteFile(const string &filename) const {
     if (!filename.empty()) {
-      ofstream strm(filename.c_str(), ofstream::out | ofstream::binary);
+      std::ofstream strm(filename.c_str(),
+                               std::ios_base::out | std::ios_base::binary);
       if (!strm) {
         LOG(ERROR) << "Fst::Write: Can't open file: " << filename;
         return false;
       }
-      return Write(strm, FstWriteOptions(filename));
+      bool val = Write(strm, FstWriteOptions(filename));
+      if (!val) LOG(ERROR) << "Fst::Write failed: " << filename;
+      return val;
     } else {
-      return Write(cout, FstWriteOptions("standard output"));
+      return Write(std::cout, FstWriteOptions("standard output"));
     }
   }
 };
 
-
 //
 // STATE and ARC ITERATOR DEFINITIONS
 //
@@ -309,8 +312,8 @@ class StateIteratorBase {
 
   bool Done() const { return Done_(); }       // End of iterator?
   StateId Value() const { return Value_(); }  // Current state (when !Done)
-  void Next() { Next_(); }      // Advance to next state (when !Done)
-  void Reset() { Reset_(); }    // Return to initial condition
+  void Next() { Next_(); }    // Advance to next state (when !Done)
+  void Reset() { Reset_(); }  // Return to initial condition
 
  private:
   // This allows base class virtual access to non-virtual derived-
@@ -322,15 +325,14 @@ class StateIteratorBase {
   virtual void Reset_() = 0;
 };
 
-
 // StateIterator initialization data
 
-template <class A> struct StateIteratorData {
+template <class A>
+struct StateIteratorData {
   StateIteratorBase<A> *base;   // Specialized iterator if non-zero
   typename A::StateId nstates;  // O.w. total # of states
 };
 
-
 // Generic state iterator, templated on the FST definition
 // - a wrapper around pointer to specific one.
 // Here is a typical use: \code
@@ -351,7 +353,9 @@ class StateIterator {
     fst.InitStateIterator(&data_);
   }
 
-  ~StateIterator() { if (data_.base) delete data_.base; }
+  ~StateIterator() {
+    if (data_.base) delete data_.base;
+  }
 
   bool Done() const {
     return data_.base ? data_.base->Done() : s_ >= data_.nstates;
@@ -380,21 +384,18 @@ class StateIterator {
   DISALLOW_COPY_AND_ASSIGN(StateIterator);
 };
 
-
 // Flags to control the behavior on an arc iterator:
-static const uint32 kArcILabelValue    = 0x0001;  // Value() gives valid ilabel
-static const uint32 kArcOLabelValue    = 0x0002;  //  "       "     "    olabel
-static const uint32 kArcWeightValue    = 0x0004;  //  "       "     "    weight
+static const uint32 kArcILabelValue = 0x0001;     // Value() gives valid ilabel
+static const uint32 kArcOLabelValue = 0x0002;     //  "       "     "    olabel
+static const uint32 kArcWeightValue = 0x0004;     //  "       "     "    weight
 static const uint32 kArcNextStateValue = 0x0008;  //  "       "     " nextstate
-static const uint32 kArcNoCache   = 0x0010;       // No need to cache arcs
+static const uint32 kArcNoCache = 0x0010;         // No need to cache arcs
 
 static const uint32 kArcValueFlags =
-                  kArcILabelValue | kArcOLabelValue |
-                  kArcWeightValue | kArcNextStateValue;
+    kArcILabelValue | kArcOLabelValue | kArcWeightValue | kArcNextStateValue;
 
 static const uint32 kArcFlags = kArcValueFlags | kArcNoCache;
 
-
 // Arc iterator interface, templated on the Arc definition; used
 // for Arc iterator specializations that are returned by the InitArcIterator
 // Fst method.
@@ -406,13 +407,13 @@ class ArcIteratorBase {
 
   virtual ~ArcIteratorBase() {}
 
-  bool Done() const { return Done_(); }            // End of iterator?
-  const A& Value() const { return Value_(); }      // Current arc (when !Done)
-  void Next() { Next_(); }           // Advance to next arc (when !Done)
+  bool Done() const { return Done_(); }        // End of iterator?
+  const A &Value() const { return Value_(); }  // Current arc (when !Done)
+  void Next() { Next_(); }  // Advance to next arc (when !Done)
   size_t Position() const { return Position_(); }  // Return current position
-  void Reset() { Reset_(); }         // Return to initial condition
-  void Seek(size_t a) { Seek_(a); }  // Random arc access by position
-  uint32 Flags() const { return Flags_(); }  // Return current behavorial flags
+  void Reset() { Reset_(); }                  // Return to initial condition
+  void Seek(size_t a) { Seek_(a); }           // Random arc access by position
+  uint32 Flags() const { return Flags_(); }   // Return current behavorial flags
   void SetFlags(uint32 flags, uint32 mask) {  // Set behavorial flags
     SetFlags_(flags, mask);
   }
@@ -422,7 +423,7 @@ class ArcIteratorBase {
   // class members of the same name. It makes the derived class more
   // efficient to use but unsafe to further derive.
   virtual bool Done_() const = 0;
-  virtual const A& Value_() const = 0;
+  virtual const A &Value_() const = 0;
   virtual void Next_() = 0;
   virtual size_t Position_() const = 0;
   virtual void Reset_() = 0;
@@ -431,9 +432,9 @@ class ArcIteratorBase {
   virtual void SetFlags_(uint32 flags, uint32 mask) = 0;
 };
 
-
 // ArcIterator initialization data
-template <class A> struct ArcIteratorData {
+template <class A>
+struct ArcIteratorData {
   ArcIteratorBase<A> *base;  // Specialized iterator if non-zero
   const A *arcs;             // O.w. arcs pointer
   size_t narcs;              // ... and arc count
@@ -443,7 +444,7 @@ template <class A> struct ArcIteratorData {
 // Generic arc iterator, templated on the FST definition
 // - a wrapper around pointer to specific one.
 // Here is a typical use: \code
-//   for (ArcIterator<StdFst> aiter(fst, s));
+//   for (ArcIterator<StdFst> aiter(fst, s);
 //        !aiter.Done();
 //         aiter.Next()) {
 //     StdArc &arc = aiter.Value();
@@ -451,7 +452,7 @@ template <class A> struct ArcIteratorData {
 //   } \endcode
 template <class F>
 class ArcIterator {
-   public:
+ public:
   typedef F FST;
   typedef typename F::Arc Arc;
   typedef typename Arc::StateId StateId;
@@ -461,8 +462,7 @@ class ArcIterator {
   }
 
   explicit ArcIterator(const ArcIteratorData<Arc> &data) : data_(data), i_(0) {
-    if (data_.ref_count)
-      ++(*data_.ref_count);
+    if (data_.ref_count) ++(*data_.ref_count);
   }
 
   ~ArcIterator() {
@@ -473,10 +473,10 @@ class ArcIterator {
   }
 
   bool Done() const {
-    return data_.base ?  data_.base->Done() : i_ >= data_.narcs;
+    return data_.base ? data_.base->Done() : i_ >= data_.narcs;
   }
 
-  const Arc& Value() const {
+  const Arc &Value() const {
     return data_.base ? data_.base->Value() : data_.arcs[i_];
   }
 
@@ -501,9 +501,7 @@ class ArcIterator {
       i_ = a;
   }
 
-  size_t Position() const {
-    return data_.base ? data_.base->Position() : i_;
-  }
+  size_t Position() const { return data_.base ? data_.base->Position() : i_; }
 
   uint32 Flags() const {
     if (data_.base)
@@ -513,8 +511,7 @@ class ArcIterator {
   }
 
   void SetFlags(uint32 flags, uint32 mask) {
-    if (data_.base)
-      data_.base->SetFlags(flags, mask);
+    if (data_.base) data_.base->SetFlags(flags, mask);
   }
 
  private:
@@ -529,8 +526,8 @@ class ArcIterator {
 // new needs to be in the global namespace
 
 template <class F>
-void* operator new(size_t size,
-                   fst::MemoryPool< fst::ArcIterator<F> > *pool) {
+void *operator new(size_t size,
+                   fst::MemoryPool<fst::ArcIterator<F>> *pool) {
   return pool->Allocate();
 }
 
@@ -538,14 +535,13 @@ namespace fst {
 
 template <class F>
 inline void Destroy(fst::ArcIterator<F> *aiter,
-                    fst::MemoryPool< fst::ArcIterator<F> > *pool) {
+                    fst::MemoryPool<fst::ArcIterator<F>> *pool) {
   if (aiter) {
     aiter->~ArcIterator<F>();
     pool->Free(aiter);
   }
 }
 
-
 //
 // MATCHER DEFINITIONS
 //
@@ -555,7 +551,6 @@ MatcherBase<A> *Fst<A>::InitMatcher(MatchType match_type) const {
   return 0;  // Use the default matcher
 }
 
-
 //
 // FST ACCESSORS - Useful functions in high-performance cases.
 //
@@ -563,45 +558,44 @@ MatcherBase<A> *Fst<A>::InitMatcher(MatchType match_type) const {
 namespace internal {
 
 // General case - requires non-abstract, 'final' methods. Use for inlining.
-template <class F> inline
-typename F::Arc::Weight Final(const F &fst, typename F::Arc::StateId s) {
+template <class F>
+inline typename F::Arc::Weight Final(const F &fst, typename F::Arc::StateId s) {
   return fst.F::Final(s);
 }
 
-template <class F> inline
-ssize_t NumArcs(const F &fst, typename F::Arc::StateId s) {
+template <class F>
+inline ssize_t NumArcs(const F &fst, typename F::Arc::StateId s) {
   return fst.F::NumArcs(s);
 }
 
-template <class F> inline
-ssize_t NumInputEpsilons(const F &fst, typename F::Arc::StateId s) {
+template <class F>
+inline ssize_t NumInputEpsilons(const F &fst, typename F::Arc::StateId s) {
   return fst.F::NumInputEpsilons(s);
 }
 
-template <class F> inline
-ssize_t NumOutputEpsilons(const F &fst, typename F::Arc::StateId s) {
+template <class F>
+inline ssize_t NumOutputEpsilons(const F &fst, typename F::Arc::StateId s) {
   return fst.F::NumOutputEpsilons(s);
 }
 
-
 //  Fst<A> case - abstract methods.
-template <class A> inline
-typename A::Weight Final(const Fst<A> &fst, typename A::StateId s) {
+template <class A>
+inline typename A::Weight Final(const Fst<A> &fst, typename A::StateId s) {
   return fst.Final(s);
 }
 
-template <class A> inline
-ssize_t NumArcs(const Fst<A> &fst, typename A::StateId s) {
+template <class A>
+inline ssize_t NumArcs(const Fst<A> &fst, typename A::StateId s) {
   return fst.NumArcs(s);
 }
 
-template <class A> inline
-ssize_t NumInputEpsilons(const Fst<A> &fst, typename A::StateId s) {
+template <class A>
+inline ssize_t NumInputEpsilons(const Fst<A> &fst, typename A::StateId s) {
   return fst.NumInputEpsilons(s);
 }
 
-template <class A> inline
-ssize_t NumOutputEpsilons(const Fst<A> &fst, typename A::StateId s) {
+template <class A>
+inline ssize_t NumOutputEpsilons(const Fst<A> &fst, typename A::StateId s) {
   return fst.NumOutputEpsilons(s);
 }
 
@@ -610,13 +604,12 @@ ssize_t NumOutputEpsilons(const Fst<A> &fst, typename A::StateId s) {
 // A useful alias when using StdArc.
 typedef Fst<StdArc> StdFst;
 
-
 //
 //  CONSTANT DEFINITIONS
 //
 
-const int kNoStateId   =  -1;  // Not a valid state ID
-const int kNoLabel     =  -1;  // Not a valid label
+const int kNoStateId = -1;  // Not a valid state ID
+const int kNoLabel = -1;    // Not a valid label
 
 //
 // Fst IMPLEMENTATION BASE
@@ -625,16 +618,17 @@ const int kNoLabel     =  -1;  // Not a valid label
 // handle reference counts, property bits, type information and symbols.
 //
 
-template <class A> class FstImpl {
+template <class A>
+class FstImpl {
  public:
   typedef typename A::Weight Weight;
   typedef typename A::StateId StateId;
 
-  FstImpl()
-      : properties_(0), type_("null"), isymbols_(0), osymbols_(0) {}
+  FstImpl() : properties_(0), type_("null"), isymbols_(0), osymbols_(0) {}
 
   FstImpl(const FstImpl<A> &impl)
-      : properties_(impl.properties_), type_(impl.type_),
+      : properties_(impl.properties_),
+        type_(impl.type_),
         isymbols_(impl.isymbols_ ? impl.isymbols_->Copy() : 0),
         osymbols_(impl.osymbols_ ? impl.osymbols_->Copy() : 0) {}
 
@@ -643,7 +637,7 @@ template <class A> class FstImpl {
     delete osymbols_;
   }
 
-  const string& Type() const { return type_; }
+  const string &Type() const { return type_; }
 
   void SetType(const string &type) { type_ = type; }
 
@@ -652,7 +646,7 @@ template <class A> class FstImpl {
   virtual uint64 Properties(uint64 mask) const { return properties_ & mask; }
 
   void SetProperties(uint64 props) {
-    properties_ &= kError;          // kError can't be cleared
+    properties_ &= kError;  // kError can't be cleared
     properties_ |= props;
   }
 
@@ -664,52 +658,40 @@ template <class A> class FstImpl {
   // Allows (only) setting error bit on const FST impls
   void SetProperties(uint64 props, uint64 mask) const {
     if (mask != kError)
-      FSTERROR() << "FstImpl::SetProperties() const: can only set kError";
+      FSTERROR() << "FstImpl::SetProperties() const: Can only set kError";
     properties_ |= kError;
   }
 
-  const SymbolTable* InputSymbols() const { return isymbols_; }
+  const SymbolTable *InputSymbols() const { return isymbols_; }
 
-  const SymbolTable* OutputSymbols() const { return osymbols_; }
+  const SymbolTable *OutputSymbols() const { return osymbols_; }
 
-  SymbolTable* InputSymbols() { return isymbols_; }
+  SymbolTable *InputSymbols() { return isymbols_; }
 
-  SymbolTable* OutputSymbols() { return osymbols_; }
+  SymbolTable *OutputSymbols() { return osymbols_; }
 
-  void SetInputSymbols(const SymbolTable* isyms) {
+  void SetInputSymbols(const SymbolTable *isyms) {
     if (isymbols_) delete isymbols_;
     isymbols_ = isyms ? isyms->Copy() : 0;
   }
 
-  void SetOutputSymbols(const SymbolTable* osyms) {
+  void SetOutputSymbols(const SymbolTable *osyms) {
     if (osymbols_) delete osymbols_;
     osymbols_ = osyms ? osyms->Copy() : 0;
   }
 
-  int RefCount() const {
-    return ref_count_.count();
-  }
-
-  int IncrRefCount() {
-    return ref_count_.Incr();
-  }
-
-  int DecrRefCount() {
-    return ref_count_.Decr();
-  }
-
   // Read-in header and symbols from input stream, initialize Fst, and
   // return the header.  If opts.header is non-null, skip read-in and
   // use the option value.  If opts.[io]symbols is non-null, read-in
   // (if present), but use the option value.
-  bool ReadHeader(istream &strm, const FstReadOptions& opts,
+  bool ReadHeader(std::istream &strm, const FstReadOptions &opts,  // NOLINT
                   int min_version, FstHeader *hdr);
 
   // Write-out header and symbols from output stream.
   // If a opts.header is false, skip writing header.
   // If opts.[io]symbols is false, skip writing those symbols.
   // This method is needed for Impl's that implement Write methods.
-  void WriteHeader(ostream &strm, const FstWriteOptions& opts,
+  void WriteHeader(std::ostream &strm, const FstWriteOptions &opts,  // NOLINT
                    int version, FstHeader *hdr) const {
     if (opts.write_header) {
       hdr->SetFstType(type_);
@@ -721,8 +703,7 @@ template <class A> class FstImpl {
         file_flags |= FstHeader::HAS_ISYMBOLS;
       if (osymbols_ && opts.write_osymbols)
         file_flags |= FstHeader::HAS_OSYMBOLS;
-      if (opts.align)
-        file_flags |= FstHeader::IS_ALIGNED;
+      if (opts.align) file_flags |= FstHeader::IS_ALIGNED;
       hdr->SetFlags(file_flags);
       hdr->Write(strm, opts.source);
     }
@@ -735,8 +716,8 @@ template <class A> class FstImpl {
   // If opts.[io]symbols is false, skip writing those symbols.
   // type is the Fst type being written.
   // This method is used in the cross-type serialization methods Fst::WriteFst.
-  static void WriteFstHeader(const Fst<A> &fst, ostream &strm,
-                             const FstWriteOptions& opts, int version,
+  static void WriteFstHeader(const Fst<A> &fst, std::ostream &strm,  // NOLINT
+                             const FstWriteOptions &opts, int version,
                              const string &type, uint64 properties,
                              FstHeader *hdr) {
     if (opts.write_header) {
@@ -749,8 +730,7 @@ template <class A> class FstImpl {
         file_flags |= FstHeader::HAS_ISYMBOLS;
       if (fst.OutputSymbols() && opts.write_osymbols)
         file_flags |= FstHeader::HAS_OSYMBOLS;
-      if (opts.align)
-        file_flags |= FstHeader::IS_ALIGNED;
+      if (opts.align) file_flags |= FstHeader::IS_ALIGNED;
       hdr->SetFlags(file_flags);
       hdr->Write(strm, opts.source);
     }
@@ -767,43 +747,43 @@ template <class A> class FstImpl {
   // the beginning of the file an rewrite the header with updated fields.
   // It repositions the file pointer back at the end of the file.
   // returns true on success, false on failure.
-  static bool UpdateFstHeader(const Fst<A> &fst, ostream &strm,
-                              const FstWriteOptions& opts, int version,
+  static bool UpdateFstHeader(const Fst<A> &fst, std::ostream &strm,  // NOLINT
+                              const FstWriteOptions &opts, int version,
                               const string &type, uint64 properties,
                               FstHeader *hdr, size_t header_offset) {
     strm.seekp(header_offset);
     if (!strm) {
-      LOG(ERROR) << "Fst::UpdateFstHeader: write failed: " << opts.source;
+      LOG(ERROR) << "Fst::UpdateFstHeader: Write failed: " << opts.source;
       return false;
     }
     WriteFstHeader(fst, strm, opts, version, type, properties, hdr);
     if (!strm) {
-      LOG(ERROR) << "Fst::UpdateFstHeader: write failed: " << opts.source;
+      LOG(ERROR) << "Fst::UpdateFstHeader: Write failed: " << opts.source;
       return false;
     }
-    strm.seekp(0, ios_base::end);
+    strm.seekp(0, std::ios_base::end);
     if (!strm) {
-      LOG(ERROR) << "Fst::UpdateFstHeader: write failed: " << opts.source;
+      LOG(ERROR) << "Fst::UpdateFstHeader: Write failed: " << opts.source;
       return false;
     }
     return true;
   }
 
  protected:
-  mutable uint64 properties_;           // Property bits
+  mutable uint64 properties_;  // Property bits
 
  private:
-  string type_;                 // Unique name of Fst class
-  SymbolTable *isymbols_;       // Ilabel symbol table
-  SymbolTable *osymbols_;       // Olabel symbol table
-  RefCounter ref_count_;        // Reference count
+  string type_;            // Unique name of Fst class
+  SymbolTable *isymbols_;  // Ilabel symbol table
+  SymbolTable *osymbols_;  // Olabel symbol table
 
   void operator=(const FstImpl<A> &impl);  // disallow
 };
 
-template <class A> inline
-bool FstImpl<A>::ReadHeader(istream &strm, const FstReadOptions& opts,
-                            int min_version, FstHeader *hdr) {
+template <class A>
+inline bool FstImpl<A>::ReadHeader(std::istream &strm,  // NOLINT
+                                   const FstReadOptions &opts, int min_version,
+                                   FstHeader *hdr) {
   if (opts.header)
     *hdr = *opts.header;
   else if (!hdr->Read(strm, opts.source))
@@ -811,20 +791,19 @@ bool FstImpl<A>::ReadHeader(istream &strm, const FstReadOptions& opts,
 
   if (FLAGS_v >= 2) {
     LOG(INFO) << "FstImpl::ReadHeader: source: " << opts.source
-              << ", fst_type: " << hdr->FstType()
-              << ", arc_type: " << A::Type()
+              << ", fst_type: " << hdr->FstType() << ", arc_type: " << A::Type()
               << ", version: " << hdr->Version()
               << ", flags: " << hdr->GetFlags();
   }
 
   if (hdr->FstType() != type_) {
-    LOG(ERROR) << "FstImpl::ReadHeader: Fst not of type \"" << type_
-               << "\": " << opts.source;
+    LOG(ERROR) << "FstImpl::ReadHeader: Fst not of type " << type_
+               << ": " << opts.source;
     return false;
   }
   if (hdr->ArcType() != A::Type()) {
-    LOG(ERROR) << "FstImpl::ReadHeader: Arc not of type \"" << A::Type()
-               << "\": " << opts.source;
+    LOG(ERROR) << "FstImpl::ReadHeader: Arc not of type " << A::Type()
+               << ": " << opts.source;
     return false;
   }
   if (hdr->Version() < min_version) {
@@ -836,13 +815,11 @@ bool FstImpl<A>::ReadHeader(istream &strm, const FstReadOptions& opts,
   if (hdr->GetFlags() & FstHeader::HAS_ISYMBOLS)
     isymbols_ = SymbolTable::Read(strm, opts.source);
   // Input symbol table not wanted; delete
-  if (!opts.read_isymbols)
-    SetInputSymbols(0);
+  if (!opts.read_isymbols) SetInputSymbols(0);
   if (hdr->GetFlags() & FstHeader::HAS_OSYMBOLS)
     osymbols_ = SymbolTable::Read(strm, opts.source);
   // Output symbol table not wanted; delete
-  if (!opts.read_osymbols)
-    SetOutputSymbols(0);
+  if (!opts.read_osymbols) SetOutputSymbols(0);
   if (opts.isymbols) {
     delete isymbols_;
     isymbols_ = opts.isymbols->Copy();
@@ -854,37 +831,33 @@ bool FstImpl<A>::ReadHeader(istream &strm, const FstReadOptions& opts,
   return true;
 }
 
-
-template<class Arc>
+template <class Arc>
 uint64 TestProperties(const Fst<Arc> &fst, uint64 mask, uint64 *known);
 
-
 // This is a helper class template useful for attaching an Fst interface to
 // its implementation, handling reference counting.
-template < class I, class F = Fst<typename I::Arc> >
+template <class I, class F = Fst<typename I::Arc>>
 class ImplToFst : public F {
  public:
   typedef typename I::Arc Arc;
   typedef typename Arc::Weight Weight;
   typedef typename Arc::StateId StateId;
 
-  virtual ~ImplToFst() { if (!impl_->DecrRefCount()) delete impl_;  }
+  StateId Start() const override { return impl_->Start(); }
 
-  virtual StateId Start() const { return impl_->Start(); }
+  Weight Final(StateId s) const override { return impl_->Final(s); }
 
-  virtual Weight Final(StateId s) const { return impl_->Final(s); }
+  size_t NumArcs(StateId s) const override { return impl_->NumArcs(s); }
 
-  virtual size_t NumArcs(StateId s) const { return impl_->NumArcs(s); }
-
-  virtual size_t NumInputEpsilons(StateId s) const {
+  size_t NumInputEpsilons(StateId s) const override {
     return impl_->NumInputEpsilons(s);
   }
 
-  virtual size_t NumOutputEpsilons(StateId s) const {
+  size_t NumOutputEpsilons(StateId s) const override {
     return impl_->NumOutputEpsilons(s);
   }
 
-  virtual uint64 Properties(uint64 mask, bool test) const {
+  uint64 Properties(uint64 mask, bool test) const override {
     if (test) {
       uint64 knownprops, testprops = TestProperties(*this, mask, &knownprops);
       impl_->SetProperties(testprops, knownprops);
@@ -894,96 +867,82 @@ class ImplToFst : public F {
     }
   }
 
-  virtual const string& Type() const { return impl_->Type(); }
+  const string &Type() const override { return impl_->Type(); }
 
-  virtual const SymbolTable* InputSymbols() const {
+  const SymbolTable *InputSymbols() const override {
     return impl_->InputSymbols();
   }
 
-  virtual const SymbolTable* OutputSymbols() const {
+  const SymbolTable *OutputSymbols() const override {
     return impl_->OutputSymbols();
   }
 
  protected:
-  ImplToFst() : impl_(0) {}
-
-  ImplToFst(I *impl) : impl_(impl) {}
-
-  ImplToFst(const ImplToFst<I, F> &fst) {
-    impl_ = fst.impl_;
-    impl_->IncrRefCount();
-  }
+  explicit ImplToFst(std::shared_ptr<I> impl) : impl_(impl) {}
 
   // This constructor presumes there is a copy constructor for the
   // implementation.
   ImplToFst(const ImplToFst<I, F> &fst, bool safe) {
     if (safe) {
-      impl_ = new I(*(fst.impl_));
+      impl_ = std::make_shared<I>(*(fst.impl_));
     } else {
       impl_ = fst.impl_;
-      impl_->IncrRefCount();
     }
   }
 
-  I *GetImpl() const { return impl_; }
+  // GetImpl and GetMutableImpl return raw pointers to the shared object.
+  const I *GetImpl() const { return impl_.get(); }
+  I *GetMutableImpl() const { return impl_.get(); }
 
-  // Change Fst implementation pointer. If 'own_impl' is true,
-  // ownership of the input implementation is given to this
-  // object; otherwise, the input implementation's reference count
-  // should be incremented.
-  void SetImpl(I *impl, bool own_impl = true) {
-    if (!own_impl)
-      impl->IncrRefCount();
-    if (impl_ && !impl_->DecrRefCount()) delete impl_;
-    impl_ = impl;
-  }
+  // GetSharedImpl returns a refcounted smart poiner to the fst implementation.
+  std::shared_ptr<I> GetSharedImpl() const { return impl_; }
+
+  // Unique returns the refcount for the owned smartpointer.
+  bool Unique() const { return impl_.unique(); }
+
+  void SetImpl(std::shared_ptr<I> impl) { impl_ = impl; }
 
  private:
-  // Disallow
-  ImplToFst<I, F> &operator=(const ImplToFst<I, F> &fst);
+  template <class U, class G>
+  friend void Cast(const U &ifst, G *ofst);
 
-  ImplToFst<I, F> &operator=(const Fst<Arc> &fst) {
-    FSTERROR() << "ImplToFst: Assignment operator disallowed";
-    GetImpl()->SetProperties(kError, kError);
-    return *this;
-  }
-
-  I *impl_;
+  std::shared_ptr<I> impl_;
 };
 
-
 // Converts FSTs by casting their implementations, where this makes
 // sense (which excludes implementations with weight-dependent virtual
 // methods). Must be a friend of the Fst classes involved (currently
 // the concrete Fsts: VectorFst, ConstFst, CompactFst).
-template<class F, class G> void Cast(const F &ifst, G *ofst) {
-  ofst->SetImpl(reinterpret_cast<typename G::Impl *>(ifst.GetImpl()), false);
+template <class F, class G>
+void Cast(const F &ifst, G *ofst) {
+  // Uses an aliasing constructor, based on static_pointer_cast, can only
+  // be safely used for arc types that have identical storage characteristics.
+  ofst->impl_ = std::shared_ptr<typename G::Impl>(
+      ifst.impl_, reinterpret_cast<typename G::Impl *>(ifst.impl_.get()));
 }
 
 // Fst Serialization
 template <class A>
 void FstToString(const Fst<A> &fst, string *result) {
-  ostringstream ostrm;
+  std::ostringstream ostrm;
   fst.Write(ostrm, FstWriteOptions("FstToString"));
   *result = ostrm.str();
 }
 
 template <class A>
 void FstToString(const Fst<A> &fst, string *result,
-                 const FstWriteOptions& options) {
-  ostringstream ostrm;
+                 const FstWriteOptions &options) {
+  std::ostringstream ostrm;
   fst.Write(ostrm, options);
   *result = ostrm.str();
 }
 
 template <class A>
 Fst<A> *StringToFst(const string &s) {
-  istringstream istrm(s);
+  std::istringstream istrm(s);
   return Fst<A>::Read(istrm, FstReadOptions("StringToFst"));
 }
 
 }  // namespace fst
 
-
-
-#endif  // FST_LIB_FST_H__
+#endif  // FST_LIB_FST_H_
