@@ -1,21 +1,6 @@
-// fstlib.h
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \page FstLib FST - Weighted Finite State Transducers
 // This is a library for constructing, combining, optimizing, and
 // searching "weighted finite-state transducers" (FSTs). Weighted
 // finite-state transducers are automata where each transition has an
@@ -34,20 +19,15 @@
 // integral types but the weight can be an arbitrary type whose
 // members satisfy certain algebraic ("semiring") properties.
 //
-// For more information, see the FST Library Wiki page:
-// http://wiki.corp.google.com/twiki/bin/view/Main/FstLibrary
+// This convenience file includes all other FST header files.
 
-// \file
-// This convenience file includes all other FST inl.h files.
-//
-
-#ifndef FST_LIB_FSTLIB_H__
-#define FST_LIB_FSTLIB_H__
+#ifndef FST_LIB_FSTLIB_H_
+#define FST_LIB_FSTLIB_H_
 
 
 // Abstract FST classes
-#include <fst/fst.h>
 #include <fst/expanded-fst.h>
+#include <fst/fst.h>
 #include <fst/mutable-fst.h>
 
 // Concrete FST classes
@@ -57,8 +37,8 @@
 #include <fst/vector-fst.h>
 
 // FST algorithms and delayed FST classes
-#include <fst/arcsort.h>
 #include <fst/arc-map.h>
+#include <fst/arcsort.h>
 #include <fst/closure.h>
 #include <fst/compose.h>
 #include <fst/concat.h>
@@ -73,6 +53,7 @@
 #include <fst/factor-weight.h>
 #include <fst/intersect.h>
 #include <fst/invert.h>
+#include <fst/isomorphic.h>
 #include <fst/map.h>
 #include <fst/minimize.h>
 #include <fst/project.h>
@@ -90,8 +71,8 @@
 #include <fst/rmfinalepsilon.h>
 #include <fst/shortest-distance.h>
 #include <fst/shortest-path.h>
-#include <fst/statesort.h>
 #include <fst/state-map.h>
+#include <fst/statesort.h>
 #include <fst/synchronize.h>
 #include <fst/topsort.h>
 #include <fst/union.h>
@@ -99,7 +80,6 @@
 #include <fst/visit.h>
 
 // Weights
-#include <fst/weight.h>
 #include <fst/expectation-weight.h>
 #include <fst/float-weight.h>
 #include <fst/lexicographic-weight.h>
@@ -112,6 +92,7 @@
 #include <fst/sparse-tuple-weight.h>
 #include <fst/string-weight.h>
 #include <fst/tuple-weight.h>
+#include <fst/weight.h>
 
 // Auxiliary classes for composition
 #include <fst/compose-filter.h>
@@ -141,9 +122,6 @@
 #include <fst/properties.h>
 #include <fst/register.h>
 #include <fst/state-reachable.h>
-#include <iostream>
-#include <fstream>
-#include <sstream>
 #include <fst/string.h>
 #include <fst/symbol-table.h>
 #include <fst/symbol-table-ops.h>
@@ -151,4 +129,4 @@
 #include <fst/util.h>
 
 
-#endif  // FST_LIB_FSTLIB_H__
+#endif  // FST_LIB_FSTLIB_H_
