@@ -1,44 +1,27 @@
-// intersect.h
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
+//
+// Class to compute the intersection of two FSAs.
 
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
-// Class to compute the intersection of two FSAs
-
-#ifndef FST_LIB_INTERSECT_H__
-#define FST_LIB_INTERSECT_H__
+#ifndef FST_LIB_INTERSECT_H_
+#define FST_LIB_INTERSECT_H_
 
 #include <algorithm>
 #include <vector>
-using std::vector;
 
 #include <fst/cache.h>
 #include <fst/compose.h>
 
+
 namespace fst {
 
-template <class A,
-          class M = Matcher<Fst<A> >,
+template <class A, class M = Matcher<Fst<A>>,
           class F = SequenceComposeFilter<M>,
-          class T = GenericComposeStateTable<A, typename F::FilterState> >
+          class T = GenericComposeStateTable<A, typename F::FilterState>>
 struct IntersectFstOptions : public ComposeFstOptions<A, M, F, T> {
-  explicit IntersectFstOptions(const CacheOptions &opts,
-                               M *mat1 = 0, M *mat2 = 0,
-                               F *filt = 0, T *sttable= 0)
-      : ComposeFstOptions<A, M, F, T>(opts, mat1, mat2, filt, sttable) { }
+  explicit IntersectFstOptions(const CacheOptions &opts, M *mat1 = 0,
+                               M *mat2 = 0, F *filt = 0, T *sttable = 0)
+      : ComposeFstOptions<A, M, F, T>(opts, mat1, mat2, filt, sttable) {}
 
   IntersectFstOptions() {}
 };
@@ -59,70 +42,71 @@ class IntersectFst : public ComposeFst<A> {
   using ComposeFst<A>::CreateBase;
   using ComposeFst<A>::CreateBase1;
   using ComposeFst<A>::Properties;
-  using ImplToFst< ComposeFstImplBase<A> >::SetImpl;
 
   typedef A Arc;
   typedef typename A::Weight Weight;
   typedef typename A::StateId StateId;
 
   IntersectFst(const Fst<A> &fst1, const Fst<A> &fst2,
-               const CacheOptions opts = CacheOptions()) {
-    AcceptorCheck(fst1, fst2);
-    SetImpl(CreateBase(fst1, fst2, opts));
+               const CacheOptions opts = CacheOptions())
+      : ComposeFst<A>(CreateBase(fst1, fst2, opts)) {
+    bool acceptors =
+        fst1.Properties(kAcceptor, true) && fst2.Properties(kAcceptor, true);
+    if (!acceptors) {
+      FSTERROR() << "IntersectFst: Input Fsts are not acceptors";
+      GetMutableImpl()->SetProperties(kError);
+    }
   }
 
   template <class M, class F, class T>
   IntersectFst(const Fst<A> &fst1, const Fst<A> &fst2,
-               const IntersectFstOptions<A, M, F, T> &opts) {
-    AcceptorCheck(fst1, fst2);
-    SetImpl(CreateBase1(fst1, fst2, opts));
+               const IntersectFstOptions<A, M, F, T> &opts)
+      : ComposeFst<A>(CreateBase1(fst1, fst2, opts)) {
+    bool acceptors =
+        fst1.Properties(kAcceptor, true) && fst2.Properties(kAcceptor, true);
+    if (!acceptors) {
+      FSTERROR() << "IntersectFst: input Fsts are not acceptors";
+      GetMutableImpl()->SetProperties(kError);
+    }
   }
 
   // See Fst<>::Copy() for doc.
-  IntersectFst(const IntersectFst<A> &fst, bool safe = false) :
-      ComposeFst<A>(fst, safe) {}
+  IntersectFst(const IntersectFst<A> &fst, bool safe = false)
+      : ComposeFst<A>(fst, safe) {}
 
   // Get a copy of this IntersectFst. See Fst<>::Copy() for further doc.
-  virtual IntersectFst<A> *Copy(bool safe = false) const {
+  IntersectFst<A> *Copy(bool safe = false) const override {
     return new IntersectFst<A>(*this, safe);
   }
 
  private:
-  void AcceptorCheck(const Fst<Arc> &fst1, const Fst<Arc> &fst2) {
-    if (!fst1.Properties(kAcceptor, true) || !fst2.Properties(kAcceptor, true))
-      LOG(FATAL) << "IntersectFst: input FSTs are not acceptors";
-  }
+  using ImplToFst<ComposeFstImplBase<A>>::GetImpl;
+  using ImplToFst<ComposeFstImplBase<A>>::GetMutableImpl;
 };
 
-
 // Specialization for IntersectFst.
 template <class A>
-class StateIterator< IntersectFst<A> >
-    : public StateIterator< ComposeFst<A> > {
+class StateIterator<IntersectFst<A>> : public StateIterator<ComposeFst<A>> {
  public:
   explicit StateIterator(const IntersectFst<A> &fst)
-      : StateIterator< ComposeFst<A> >(fst) {}
+      : StateIterator<ComposeFst<A>>(fst) {}
 };
 
-
 // Specialization for IntersectFst.
 template <class A>
-class ArcIterator< IntersectFst<A> >
-    : public ArcIterator< ComposeFst<A> > {
+class ArcIterator<IntersectFst<A>> : public ArcIterator<ComposeFst<A>> {
  public:
   typedef typename A::StateId StateId;
 
   ArcIterator(const IntersectFst<A> &fst, StateId s)
-      : ArcIterator< ComposeFst<A> >(fst, s) {}
+      : ArcIterator<ComposeFst<A>>(fst, s) {}
 };
 
 // Useful alias when using StdArc.
 typedef IntersectFst<StdArc> StdIntersectFst;
 
-
 typedef ComposeOptions IntersectOptions;
 
-
 // Computes the intersection (Hadamard product) of two FSAs. This
 // version writes the intersection to an output MurableFst. Only
 // strings that are in both automata are retained in the result.
@@ -133,11 +117,11 @@ typedef ComposeOptions IntersectOptions;
 // Complexity: same as Compose.
 //
 // Caveats:  same as Compose.
-template<class Arc>
+template <class Arc>
 void Intersect(const Fst<Arc> &ifst1, const Fst<Arc> &ifst2,
-             MutableFst<Arc> *ofst,
-             const IntersectOptions &opts = IntersectOptions()) {
-  typedef Matcher< Fst<Arc> > M;
+               MutableFst<Arc> *ofst,
+               const IntersectOptions &opts = IntersectOptions()) {
+  typedef Matcher<Fst<Arc>> M;
 
   if (opts.filter_type == AUTO_FILTER) {
     CacheOptions nopts;
@@ -148,19 +132,18 @@ void Intersect(const Fst<Arc> &ifst1, const Fst<Arc> &ifst2,
     iopts.gc_limit = 0;  // Cache only the last state for fastest copy.
     *ofst = IntersectFst<Arc>(ifst1, ifst2, iopts);
   } else if (opts.filter_type == ALT_SEQUENCE_FILTER) {
-    IntersectFstOptions<Arc, M, AltSequenceComposeFilter<M> > iopts;
+    IntersectFstOptions<Arc, M, AltSequenceComposeFilter<M>> iopts;
     iopts.gc_limit = 0;  // Cache only the last state for fastest copy.
     *ofst = IntersectFst<Arc>(ifst1, ifst2, iopts);
   } else if (opts.filter_type == MATCH_FILTER) {
-    IntersectFstOptions<Arc, M, MatchComposeFilter<M> > iopts;
+    IntersectFstOptions<Arc, M, MatchComposeFilter<M>> iopts;
     iopts.gc_limit = 0;  // Cache only the last state for fastest copy.
     *ofst = IntersectFst<Arc>(ifst1, ifst2, iopts);
   }
 
-  if (opts.connect)
-    Connect(ofst);
+  if (opts.connect) Connect(ofst);
 }
 
 }  // namespace fst
 
-#endif  // FST_LIB_INTERSECT_H__
+#endif  // FST_LIB_INTERSECT_H_
