@@ -1,21 +1,6 @@
-// lexicographic-weight.h
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: rws@google.com (Richard Sproat)
-//
-// \file
 // Lexicographic weight set and associated semiring operation definitions.
 //
 // A lexicographic weight is a sequence of weights, each of which must have the
@@ -25,43 +10,48 @@
 // The + operation on two weights a and b is the lexicographically
 // prior of a and b.
 
-#ifndef FST_LIB_LEXICOGRAPHIC_WEIGHT_H__
-#define FST_LIB_LEXICOGRAPHIC_WEIGHT_H__
+#ifndef FST_LIB_LEXICOGRAPHIC_WEIGHT_H_
+#define FST_LIB_LEXICOGRAPHIC_WEIGHT_H_
 
 #include <string>
+
 #include <fst/pair-weight.h>
 #include <fst/weight.h>
 
+
 namespace fst {
 
-template<class W1, class W2>
+template <class W1, class W2>
 class LexicographicWeight : public PairWeight<W1, W2> {
  public:
   using PairWeight<W1, W2>::Value1;
   using PairWeight<W1, W2>::Value2;
+  using PairWeight<W1, W2>::SetValue1;
+  using PairWeight<W1, W2>::SetValue2;
   using PairWeight<W1, W2>::Zero;
   using PairWeight<W1, W2>::One;
+  using PairWeight<W1, W2>::NoWeight;
   using PairWeight<W1, W2>::Quantize;
   using PairWeight<W1, W2>::Reverse;
 
   typedef LexicographicWeight<typename W1::ReverseWeight,
-                              typename W2::ReverseWeight>
-  ReverseWeight;
+                              typename W2::ReverseWeight> ReverseWeight;
 
   LexicographicWeight() {}
 
-  LexicographicWeight(const PairWeight<W1, W2>& w)
-      : PairWeight<W1, W2>(w) {}
+  LexicographicWeight(const PairWeight<W1, W2> &w) : PairWeight<W1, W2>(w) {}
 
   LexicographicWeight(W1 w1, W2 w2) : PairWeight<W1, W2>(w1, w2) {
     uint64 props = kPath;
     if ((W1::Properties() & props) != props) {
-      LOG(ERROR) << "LexicographicWeight must "
+      FSTERROR() << "LexicographicWeight must "
                  << "have the path property: " << W1::Type();
+      SetValue1(W1::NoWeight());
     }
     if ((W2::Properties() & props) != props) {
-      LOG(ERROR) << "LexicographicWeight must "
+      FSTERROR() << "LexicographicWeight must "
                  << "have the path property: " << W2::Type();
+      SetValue2(W2::NoWeight());
     }
   }
 
@@ -75,6 +65,12 @@ class LexicographicWeight : public PairWeight<W1, W2> {
     return one;
   }
 
+  static const LexicographicWeight<W1, W2> &NoWeight() {
+    static const LexicographicWeight<W1, W2> no_weight(
+        PairWeight<W1, W2>::NoWeight());
+    return no_weight;
+  }
+
   static const string &Type() {
     static const string type = W1::Type() + "_LT_" + W2::Type();
     return type;
@@ -92,9 +88,7 @@ class LexicographicWeight : public PairWeight<W1, W2> {
     return PairWeight<W1, W2>::Quantize();
   }
 
-  ReverseWeight Reverse() const {
-    return PairWeight<W1, W2>::Reverse();
-  }
+  ReverseWeight Reverse() const { return PairWeight<W1, W2>::Reverse(); }
 
   static uint64 Properties() {
     uint64 props1 = W1::Properties();
@@ -107,6 +101,8 @@ class LexicographicWeight : public PairWeight<W1, W2> {
 template <class W1, class W2>
 inline LexicographicWeight<W1, W2> Plus(const LexicographicWeight<W1, W2> &w,
                                         const LexicographicWeight<W1, W2> &v) {
+  if (!w.Member() || !v.Member())
+    return LexicographicWeight<W1, W2>::NoWeight();
   NaturalLess<W1> less1;
   NaturalLess<W2> less2;
   if (less1(w.Value1(), v.Value1())) return w;
@@ -131,6 +127,6 @@ inline LexicographicWeight<W1, W2> Divide(const LexicographicWeight<W1, W2> &w,
                                      Divide(w.Value2(), v.Value2(), typ));
 }
 
-}  // namespace fst;
+}  // namespace fst
 
-#endif  // FST_LIB_LEXICOGRAPHIC_WEIGHT_H__
+#endif  // FST_LIB_LEXICOGRAPHIC_WEIGHT_H_
