@@ -20,8 +20,8 @@
 // Classes and functions here are no-ops (by design); proper locking requires
 // actual implementation.
 
-#ifndef FST_LIB_LOCK_H__
-#define FST_LIB_LOCK_H__
+#ifndef FST_LIB_LOCK_H_
+#define FST_LIB_LOCK_H_
 
 #include <fst/compat.h>  // for DISALLOW_COPY_AND_ASSIGN
 
@@ -72,21 +72,6 @@ class ReaderMutexLock {
   DISALLOW_COPY_AND_ASSIGN(ReaderMutexLock);
 };
 
-// Reference counting - single-thread implementation
-class RefCounter {
- public:
-  RefCounter() : count_(1) {}
-
-  int count() const { return count_; }
-  int Incr() const { return ++count_; }
-  int Decr() const {  return --count_; }
-
- private:
-  mutable int count_;
-
-  DISALLOW_COPY_AND_ASSIGN(RefCounter);
-};
-
 }  // namespace fst
 
-#endif  // FST_LIB_LOCK_H__
+#endif  // FST_LIB_LOCK_H_
