@@ -1,26 +1,11 @@
-// lookahead-matcher.h
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
 // Classes to add lookahead to FST matchers, useful e.g. for improving
 // composition efficiency with certain inputs.
 
-#ifndef FST_LIB_LOOKAHEAD_MATCHER_H__
-#define FST_LIB_LOOKAHEAD_MATCHER_H__
+#ifndef FST_LIB_LOOKAHEAD_MATCHER_H_
+#define FST_LIB_LOOKAHEAD_MATCHER_H_
 
 #include <fst/add-on.h>
 #include <fst/const-fst.h>
@@ -96,24 +81,24 @@ namespace fst {
 // LOOK-AHEAD FLAGS (see also kMatcherFlags in matcher.h):
 //
 // Matcher is a lookahead matcher when 'match_type' is MATCH_INPUT.
-const uint32 kInputLookAheadMatcher =     0x00000010;
+const uint32 kInputLookAheadMatcher = 0x00000010;
 
 // Matcher is a lookahead matcher when 'match_type' is MATCH_OUTPUT.
-const uint32 kOutputLookAheadMatcher =    0x00000020;
+const uint32 kOutputLookAheadMatcher = 0x00000020;
 
 // A non-trivial implementation of LookAheadWeight() method defined and
 // should be used?
-const uint32 kLookAheadWeight =           0x00000040;
+const uint32 kLookAheadWeight = 0x00000040;
 
 // A non-trivial implementation of LookAheadPrefix() method defined and
 // should be used?
-const uint32 kLookAheadPrefix =           0x00000080;
+const uint32 kLookAheadPrefix = 0x00000080;
 
 // Look-ahead of matcher FST non-epsilon arcs?
-const uint32 kLookAheadNonEpsilons =      0x00000100;
+const uint32 kLookAheadNonEpsilons = 0x00000100;
 
 // Look-ahead of matcher FST epsilon arcs?
-const uint32 kLookAheadEpsilons =         0x00000200;
+const uint32 kLookAheadEpsilons = 0x00000200;
 
 // Ignore epsilon paths for the lookahead prefix? Note this gives
 // correct results in composition only with an appropriate composition
@@ -121,10 +106,10 @@ const uint32 kLookAheadEpsilons =         0x00000200;
 const uint32 kLookAheadNonEpsilonPrefix = 0x00000400;
 
 // For LabelLookAheadMatcher, save relabeling data to file
-const uint32 kLookAheadKeepRelabelData =  0x00000800;
+const uint32 kLookAheadKeepRelabelData = 0x00000800;
 
 // Flags used for lookahead matchers.
-const uint32 kLookAheadFlags =            0x00000ff0;
+const uint32 kLookAheadFlags = 0x00000ff0;
 
 // LookAhead Matcher interface, templated on the Arc definition; used
 // for lookahead matcher specializations that are returned by the
@@ -138,10 +123,10 @@ class LookAheadMatcherBase : public MatcherBase<A> {
   typedef typename A::Weight Weight;
 
   LookAheadMatcherBase()
-  : weight_(Weight::One()),
-    prefix_arc_(kNoLabel, kNoLabel, Weight::One(), kNoStateId) {}
+      : weight_(Weight::One()),
+        prefix_arc_(kNoLabel, kNoLabel, Weight::One(), kNoStateId) {}
 
-  virtual ~LookAheadMatcherBase() {}
+  ~LookAheadMatcherBase() override {}
 
   bool LookAheadLabel(Label label) const { return LookAheadLabel_(label); }
 
@@ -160,7 +145,7 @@ class LookAheadMatcherBase : public MatcherBase<A> {
     }
   }
 
-  virtual void InitLookAheadFst(const Fst<Arc>& fst, bool copy = false) = 0;
+  virtual void InitLookAheadFst(const Fst<Arc> &fst, bool copy = false) = 0;
 
  protected:
   void SetLookAheadWeight(const Weight &w) { weight_ = w; }
@@ -178,7 +163,6 @@ class LookAheadMatcherBase : public MatcherBase<A> {
   Arc prefix_arc_;                            // Look-ahead prefix arc
 };
 
-
 // Don't really lookahead, just declare future looks good regardless.
 template <class M>
 class TrivialLookAheadMatcher
@@ -198,44 +182,48 @@ class TrivialLookAheadMatcher
       : matcher_(lmatcher.matcher_, safe) {}
 
   // General matcher methods
-  TrivialLookAheadMatcher<M> *Copy(bool safe = false) const {
+  TrivialLookAheadMatcher<M> *Copy(bool safe = false) const override {
     return new TrivialLookAheadMatcher<M>(*this, safe);
   }
 
-  MatchType Type(bool test) const { return matcher_.Type(test); }
+  MatchType Type(bool test) const override { return matcher_.Type(test); }
   void SetState(StateId s) { return matcher_.SetState(s); }
   bool Find(Label label) { return matcher_.Find(label); }
   bool Done() const { return matcher_.Done(); }
-  const Arc& Value() const { return matcher_.Value(); }
+  const Arc &Value() const { return matcher_.Value(); }
   void Next() { matcher_.Next(); }
+  Weight Final(StateId s) const { return matcher_.Final(s); }
   ssize_t Priority(StateId s) { return matcher_.Priority(s); }
-  virtual const FST &GetFst() const { return matcher_.GetFst(); }
-  uint64 Properties(uint64 props) const { return matcher_.Properties(props); }
-  uint32 Flags() const {
+  const FST &GetFst() const override { return matcher_.GetFst(); }
+  uint64 Properties(uint64 props) const override {
+    return matcher_.Properties(props);
+  }
+  uint32 Flags() const override {
     return matcher_.Flags() | kInputLookAheadMatcher | kOutputLookAheadMatcher;
   }
 
   // Look-ahead methods.
-  bool LookAheadLabel(Label label) const { return true;  }
-  bool LookAheadFst(const Fst<Arc> &fst, StateId s) {return true; }
+  bool LookAheadLabel(Label label) const { return true; }
+  bool LookAheadFst(const Fst<Arc> &fst, StateId s) { return true; }
   Weight LookAheadWeight() const { return Weight::One(); }
   bool LookAheadPrefix(Arc *arc) const { return false; }
-  void InitLookAheadFst(const Fst<Arc>& fst, bool copy = false) {}
+  void InitLookAheadFst(const Fst<Arc> &fst, bool copy = false) override {}
 
  private:
   // This allows base class virtual access to non-virtual derived-
   // class members of the same name. It makes the derived class more
   // efficient to use but unsafe to further derive.
-  virtual void SetState_(StateId s) { SetState(s); }
-  virtual bool Find_(Label label) { return Find(label); }
-  virtual bool Done_() const { return Done(); }
-  virtual const Arc& Value_() const { return Value(); }
-  virtual void Next_() { Next(); }
-  virtual ssize_t Priority_(StateId s) { return Priority(s); }
+  void SetState_(StateId s) override { SetState(s); }
+  bool Find_(Label label) override { return Find(label); }
+  bool Done_() const override { return Done(); }
+  const Arc &Value_() const override { return Value(); }
+  void Next_() override { Next(); }
+  Weight Final_(StateId s) const override { return Final(s); }
+  ssize_t Priority_(StateId s) override { return Priority(s); }
 
-  bool LookAheadLabel_(Label l) const { return LookAheadLabel(l); }
+  bool LookAheadLabel_(Label l) const override { return LookAheadLabel(l); }
 
-  bool LookAheadFst_(const Fst<Arc> &fst, StateId s) {
+  bool LookAheadFst_(const Fst<Arc> &fst, StateId s) override {
     return LookAheadFst(fst, s);
   }
 
@@ -248,9 +236,8 @@ class TrivialLookAheadMatcher
 // Look-ahead of one transition. Template argument F accepts flags to
 // control behavior.
 template <class M, uint32 F = kLookAheadNonEpsilons | kLookAheadEpsilons |
-          kLookAheadWeight | kLookAheadPrefix>
-class ArcLookAheadMatcher
-    : public LookAheadMatcherBase<typename M::FST::Arc> {
+                              kLookAheadWeight | kLookAheadPrefix>
+class ArcLookAheadMatcher : public LookAheadMatcherBase<typename M::FST::Arc> {
  public:
   typedef typename M::FST FST;
   typedef typename M::Arc Arc;
@@ -264,8 +251,9 @@ class ArcLookAheadMatcher
   using LookAheadMatcherBase<Arc>::SetLookAheadWeight;
   using LookAheadMatcherBase<Arc>::ClearLookAheadPrefix;
 
-  ArcLookAheadMatcher(const FST &fst, MatchType match_type,
-                      MatcherData *data = 0)
+  ArcLookAheadMatcher(
+      const FST &fst, MatchType match_type,
+      std::shared_ptr<MatcherData> data = std::shared_ptr<MatcherData>())
       : matcher_(fst, match_type),
         fst_(matcher_.GetFst()),
         lfst_(0),
@@ -279,11 +267,11 @@ class ArcLookAheadMatcher
         s_(kNoStateId) {}
 
   // General matcher methods
-  ArcLookAheadMatcher<M, F> *Copy(bool safe = false) const {
+  ArcLookAheadMatcher<M, F> *Copy(bool safe = false) const override {
     return new ArcLookAheadMatcher<M, F>(*this, safe);
   }
 
-  MatchType Type(bool test) const { return matcher_.Type(test); }
+  MatchType Type(bool test) const override { return matcher_.Type(test); }
 
   void SetState(StateId s) {
     s_ = s;
@@ -292,18 +280,24 @@ class ArcLookAheadMatcher
 
   bool Find(Label label) { return matcher_.Find(label); }
   bool Done() const { return matcher_.Done(); }
-  const Arc& Value() const { return matcher_.Value(); }
+  const Arc &Value() const { return matcher_.Value(); }
   void Next() { matcher_.Next(); }
+  Weight Final(StateId s) const { return matcher_.Final(s); }
   ssize_t Priority(StateId s) { return matcher_.Priority(s); }
-  const FST &GetFst() const { return fst_; }
-  uint64 Properties(uint64 props) const { return matcher_.Properties(props); }
-  uint32 Flags() const {
-    return matcher_.Flags() | kInputLookAheadMatcher |
-        kOutputLookAheadMatcher | F;
+  const FST &GetFst() const override { return fst_; }
+  uint64 Properties(uint64 props) const override {
+    return matcher_.Properties(props);
+  }
+  uint32 Flags() const override {
+    return matcher_.Flags() | kInputLookAheadMatcher | kOutputLookAheadMatcher |
+           F;
   }
 
-  // Writable matcher methods
-  MatcherData *GetData() const { return 0; }
+  const MatcherData *GetData() const { return nullptr; }
+
+  std::shared_ptr<MatcherData> GetSharedData() const {
+    return std::shared_ptr<MatcherData>();
+  }
 
   // Look-ahead methods.
   bool LookAheadLabel(Label label) const { return matcher_.Find(label); }
@@ -312,7 +306,7 @@ class ArcLookAheadMatcher
   // at (s_, s).
   bool LookAheadFst(const Fst<Arc> &fst, StateId s);
 
-  void InitLookAheadFst(const Fst<Arc>& fst, bool copy = false) {
+  void InitLookAheadFst(const Fst<Arc> &fst, bool copy = false) override {
     lfst_ = &fst;
   }
 
@@ -320,57 +314,50 @@ class ArcLookAheadMatcher
   // This allows base class virtual access to non-virtual derived-
   // class members of the same name. It makes the derived class more
   // efficient to use but unsafe to further derive.
-  virtual void SetState_(StateId s) { SetState(s); }
-  virtual bool Find_(Label label) { return Find(label); }
-  virtual bool Done_() const { return Done(); }
-  virtual const Arc& Value_() const { return Value(); }
-  virtual void Next_() { Next(); }
-  virtual ssize_t Priority_(StateId s) { return Priority(s); }
+  void SetState_(StateId s) override { SetState(s); }
+  bool Find_(Label label) override { return Find(label); }
+  bool Done_() const override { return Done(); }
+  const Arc &Value_() const override { return Value(); }
+  void Next_() override { Next(); }
+  Weight Final_(StateId s) const override { return Final(s); }
+  ssize_t Priority_(StateId s) override { return Priority(s); }
 
-  bool LookAheadLabel_(Label l) const { return LookAheadLabel(l); }
-  bool LookAheadFst_(const Fst<Arc> &fst, StateId s) {
+  bool LookAheadLabel_(Label l) const override { return LookAheadLabel(l); }
+  bool LookAheadFst_(const Fst<Arc> &fst, StateId s) override {
     return LookAheadFst(fst, s);
   }
 
   mutable M matcher_;
-  const FST &fst_;         // Matcher FST
-  const Fst<Arc> *lfst_;   // Look-ahead FST
-  StateId s_;              // Matcher state
+  const FST &fst_;        // Matcher FST
+  const Fst<Arc> *lfst_;  // Look-ahead FST
+  StateId s_;             // Matcher state
 };
 
 template <class M, uint32 F>
 bool ArcLookAheadMatcher<M, F>::LookAheadFst(const Fst<Arc> &fst, StateId s) {
-  if (&fst != lfst_)
-    InitLookAheadFst(fst);
+  if (&fst != lfst_) InitLookAheadFst(fst);
 
   bool ret = false;
   ssize_t nprefix = 0;
-  if (F & kLookAheadWeight)
-    SetLookAheadWeight(Weight::Zero());
-  if (F & kLookAheadPrefix)
-    ClearLookAheadPrefix();
-  if (fst_.Final(s_) != Weight::Zero() &&
-      lfst_->Final(s) != Weight::Zero()) {
-    if (!(F & (kLookAheadWeight | kLookAheadPrefix)))
-      return true;
+  if (F & kLookAheadWeight) SetLookAheadWeight(Weight::Zero());
+  if (F & kLookAheadPrefix) ClearLookAheadPrefix();
+  if (fst_.Final(s_) != Weight::Zero() && lfst_->Final(s) != Weight::Zero()) {
+    if (!(F & (kLookAheadWeight | kLookAheadPrefix))) return true;
     ++nprefix;
     if (F & kLookAheadWeight)
-      SetLookAheadWeight(Plus(LookAheadWeight(),
-                              Times(fst_.Final(s_), lfst_->Final(s))));
+      SetLookAheadWeight(
+          Plus(LookAheadWeight(), Times(fst_.Final(s_), lfst_->Final(s))));
     ret = true;
   }
   if (matcher_.Find(kNoLabel)) {
-    if (!(F & (kLookAheadWeight | kLookAheadPrefix)))
-      return true;
+    if (!(F & (kLookAheadWeight | kLookAheadPrefix))) return true;
     ++nprefix;
     if (F & kLookAheadWeight)
       for (; !matcher_.Done(); matcher_.Next())
         SetLookAheadWeight(Plus(LookAheadWeight(), matcher_.Value().weight));
     ret = true;
   }
-  for (ArcIterator< Fst<Arc> > aiter(*lfst_, s);
-       !aiter.Done();
-       aiter.Next()) {
+  for (ArcIterator<Fst<Arc>> aiter(*lfst_, s); !aiter.Done(); aiter.Next()) {
     const Arc &arc = aiter.Value();
     Label label = kNoLabel;
     switch (matcher_.Type(false)) {
@@ -381,28 +368,23 @@ bool ArcLookAheadMatcher<M, F>::LookAheadFst(const Fst<Arc> &fst, StateId s) {
         label = arc.ilabel;
         break;
       default:
-        FSTERROR() << "ArcLookAheadMatcher::LookAheadFst: bad match type";
+        FSTERROR() << "ArcLookAheadMatcher::LookAheadFst: Bad match type";
         return true;
     }
     if (label == 0) {
-      if (!(F & (kLookAheadWeight | kLookAheadPrefix)))
-        return true;
-      if (!(F & kLookAheadNonEpsilonPrefix))
-        ++nprefix;
+      if (!(F & (kLookAheadWeight | kLookAheadPrefix))) return true;
+      if (!(F & kLookAheadNonEpsilonPrefix)) ++nprefix;
       if (F & kLookAheadWeight)
         SetLookAheadWeight(Plus(LookAheadWeight(), arc.weight));
       ret = true;
     } else if (matcher_.Find(label)) {
-      if (!(F & (kLookAheadWeight | kLookAheadPrefix)))
-        return true;
+      if (!(F & (kLookAheadWeight | kLookAheadPrefix))) return true;
       for (; !matcher_.Done(); matcher_.Next()) {
         ++nprefix;
         if (F & kLookAheadWeight)
           SetLookAheadWeight(Plus(LookAheadWeight(),
-                                  Times(arc.weight,
-                                        matcher_.Value().weight)));
-        if ((F & kLookAheadPrefix) && nprefix == 1)
-          SetLookAheadPrefix(arc);
+                                  Times(arc.weight, matcher_.Value().weight)));
+        if ((F & kLookAheadPrefix) && nprefix == 1) SetLookAheadPrefix(arc);
       }
       ret = true;
     }
@@ -416,15 +398,14 @@ bool ArcLookAheadMatcher<M, F>::LookAheadFst(const Fst<Arc> &fst, StateId s) {
   return ret;
 }
 
-
 // Template argument F accepts flags to control behavior.
 // It must include precisely one of KInputLookAheadMatcher or
 // KOutputLookAheadMatcher.
-template <class M, uint32 F = kLookAheadEpsilons | kLookAheadWeight |
-          kLookAheadPrefix | kLookAheadNonEpsilonPrefix |
-          kLookAheadKeepRelabelData,
+template <class M,
+          uint32 F = kLookAheadEpsilons | kLookAheadWeight | kLookAheadPrefix |
+                     kLookAheadNonEpsilonPrefix | kLookAheadKeepRelabelData,
           class S = DefaultAccumulator<typename M::Arc>,
-          class R = LabelReachable<typename M::Arc, S> >
+          class R = LabelReachable<typename M::Arc, S>>
 class LabelLookAheadMatcher
     : public LookAheadMatcherBase<typename M::FST::Arc> {
  public:
@@ -440,25 +421,26 @@ class LabelLookAheadMatcher
   using LookAheadMatcherBase<Arc>::SetLookAheadWeight;
   using LookAheadMatcherBase<Arc>::ClearLookAheadPrefix;
 
-  LabelLookAheadMatcher(const FST &fst, MatchType match_type,
-                        MatcherData *data = 0, S *s = 0)
+  LabelLookAheadMatcher(
+      const FST &fst, MatchType match_type,
+      std::shared_ptr<MatcherData> data = std::shared_ptr<MatcherData>(),
+      S *s = 0)
       : matcher_(fst, match_type),
         lfst_(0),
         label_reachable_(0),
         s_(kNoStateId),
         error_(false) {
     if (!(F & (kInputLookAheadMatcher | kOutputLookAheadMatcher))) {
-      FSTERROR() << "LabelLookaheadMatcher: bad matcher flags: " << F;
+      FSTERROR() << "LabelLookaheadMatcher: Bad matcher flags: " << F;
       error_ = true;
     }
     bool reach_input = match_type == MATCH_INPUT;
     if (data) {
-      if (reach_input == data->ReachInput())
-        label_reachable_ = new R(data, s);
+      if (reach_input == data->ReachInput()) label_reachable_ = new R(data, s);
     } else if ((reach_input && (F & kInputLookAheadMatcher)) ||
                (!reach_input && (F & kOutputLookAheadMatcher))) {
-      label_reachable_ = new R(
-          fst, reach_input, s, F & kLookAheadKeepRelabelData);
+      label_reachable_ =
+          new R(fst, reach_input, s, F & kLookAheadKeepRelabelData);
     }
   }
 
@@ -466,26 +448,23 @@ class LabelLookAheadMatcher
                         bool safe = false)
       : matcher_(lmatcher.matcher_, safe),
         lfst_(lmatcher.lfst_),
-        label_reachable_(
-            lmatcher.label_reachable_ ?
-            new R(*lmatcher.label_reachable_, safe) : 0),
+        label_reachable_(lmatcher.label_reachable_
+                             ? new R(*lmatcher.label_reachable_, safe)
+                             : 0),
         s_(kNoStateId),
         error_(lmatcher.error_) {}
 
-  ~LabelLookAheadMatcher() {
-    delete label_reachable_;
-  }
+  ~LabelLookAheadMatcher() override { delete label_reachable_; }
 
   // General matcher methods
-  LabelLookAheadMatcher<M, F, S, R> *Copy(bool safe = false) const {
+  LabelLookAheadMatcher<M, F, S, R> *Copy(bool safe = false) const override {
     return new LabelLookAheadMatcher<M, F, S, R>(*this, safe);
   }
 
-  MatchType Type(bool test) const { return matcher_.Type(test); }
+  MatchType Type(bool test) const override { return matcher_.Type(test); }
 
   void SetState(StateId s) {
-    if (s_ == s)
-      return;
+    if (s_ == s) return;
     s_ = s;
     match_set_state_ = false;
     reach_set_state_ = false;
@@ -500,19 +479,20 @@ class LabelLookAheadMatcher
   }
 
   bool Done() const { return matcher_.Done(); }
-  const Arc& Value() const { return matcher_.Value(); }
+  const Arc &Value() const { return matcher_.Value(); }
   void Next() { matcher_.Next(); }
   ssize_t Priority(StateId s) { return matcher_.Priority(s); }
-  const FST &GetFst() const { return matcher_.GetFst(); }
+  Weight Final(StateId s) const { return matcher_.Final(s); }
+  const FST &GetFst() const override { return matcher_.GetFst(); }
 
-  uint64 Properties(uint64 inprops) const {
+  uint64 Properties(uint64 inprops) const override {
     uint64 outprops = matcher_.Properties(inprops);
     if (error_ || (label_reachable_ && label_reachable_->Error()))
       outprops |= kError;
     return outprops;
   }
 
-  uint32 Flags() const {
+  uint32 Flags() const override {
     if (label_reachable_ && label_reachable_->GetData()->ReachInput())
       return matcher_.Flags() | F | kInputLookAheadMatcher;
     else if (label_reachable_ && !label_reachable_->GetData()->ReachInput())
@@ -521,15 +501,18 @@ class LabelLookAheadMatcher
       return matcher_.Flags();
   }
 
-  // Writable matcher methods
-  MatcherData *GetData() const {
+  const MatcherData *GetData() const {
     return label_reachable_ ? label_reachable_->GetData() : 0;
   };
 
+  std::shared_ptr<MatcherData> GetSharedData() const {
+    return label_reachable_ ? label_reachable_->GetSharedData()
+                            : std::shared_ptr<MatcherData>();
+  }
+
   // Look-ahead methods.
   bool LookAheadLabel(Label label) const {
-    if (label == 0)
-      return true;
+    if (label == 0) return true;
 
     if (label_reachable_) {
       if (!reach_set_state_) {
@@ -547,7 +530,7 @@ class LabelLookAheadMatcher
   template <class L>
   bool LookAheadFst(const L &fst, StateId s);
 
-  void InitLookAheadFst(const Fst<Arc>& fst, bool copy = false) {
+  void InitLookAheadFst(const Fst<Arc> &fst, bool copy = false) override {
     lfst_ = &fst;
     if (label_reachable_) {
       bool reach_input = Type(false) == MATCH_OUTPUT;
@@ -556,7 +539,7 @@ class LabelLookAheadMatcher
   }
 
   template <class L>
-  void InitLookAheadFst(const L& fst, bool copy = false) {
+  void InitLookAheadFst(const L &fst, bool copy = false) {
     lfst_ = static_cast<const Fst<Arc> *>(&fst);
     if (label_reachable_) {
       bool reach_input = Type(false) == MATCH_OUTPUT;
@@ -568,38 +551,38 @@ class LabelLookAheadMatcher
   // This allows base class virtual access to non-virtual derived-
   // class members of the same name. It makes the derived class more
   // efficient to use but unsafe to further derive.
-  virtual void SetState_(StateId s) { SetState(s); }
-  virtual bool Find_(Label label) { return Find(label); }
-  virtual bool Done_() const { return Done(); }
-  virtual const Arc& Value_() const { return Value(); }
-  virtual void Next_() { Next(); }
-  virtual ssize_t Priority_(StateId s) { return Priority(s); }
+  void SetState_(StateId s) override { SetState(s); }
+  bool Find_(Label label) override { return Find(label); }
+  bool Done_() const override { return Done(); }
+  const Arc &Value_() const override { return Value(); }
+  void Next_() override { Next(); }
+  Weight Final_(StateId s) const override { return Final(s); }
+  ssize_t Priority_(StateId s) override { return Priority(s); }
 
-  bool LookAheadLabel_(Label l) const { return LookAheadLabel(l); }
-  bool LookAheadFst_(const Fst<Arc> &fst, StateId s) {
+  bool LookAheadLabel_(Label l) const override { return LookAheadLabel(l); }
+  bool LookAheadFst_(const Fst<Arc> &fst, StateId s) override {
     return LookAheadFst(fst, s);
   }
 
   mutable M matcher_;
-  const Fst<Arc> *lfst_;                     // Look-ahead FST
-  R *label_reachable_;                       // Label reachability info
-  StateId s_;                                // Matcher state
-  bool match_set_state_;                     // matcher_.SetState called?
-  mutable bool reach_set_state_;             // reachable_.SetState called?
+  const Fst<Arc> *lfst_;          // Look-ahead FST
+  R *label_reachable_;            // Label reachability info
+  StateId s_;                     // Matcher state
+  bool match_set_state_;          // matcher_.SetState called?
+  mutable bool reach_set_state_;  // reachable_.SetState called?
   bool error_;
 };
 
 template <class M, uint32 F, class S, class R>
-template <class L> inline
-bool LabelLookAheadMatcher<M, F, S, R>::LookAheadFst(const L &fst, StateId s) {
-  if (static_cast<const Fst<Arc> *>(&fst) != lfst_)
-    InitLookAheadFst(fst);
+template <class L>
+inline bool LabelLookAheadMatcher<M, F, S, R>::LookAheadFst(const L &fst,
+                                                            StateId s) {
+  if (static_cast<const Fst<Arc> *>(&fst) != lfst_) InitLookAheadFst(fst);
 
   SetLookAheadWeight(Weight::One());
   ClearLookAheadPrefix();
 
-  if (!label_reachable_)
-    return true;
+  if (!label_reachable_) return true;
 
   label_reachable_->SetState(s_, s);
   reach_set_state_ = true;
@@ -609,9 +592,8 @@ bool LabelLookAheadMatcher<M, F, S, R>::LookAheadFst(const L &fst, StateId s) {
 
   ArcIterator<L> aiter(fst, s);
   aiter.SetFlags(kArcNoCache, kArcNoCache);  // make caching optional;
-  bool reach_arc = label_reachable_->Reach(&aiter, 0,
-                                           internal::NumArcs(*lfst_, s),
-                                           compute_weight);
+  bool reach_arc = label_reachable_->Reach(
+      &aiter, 0, internal::NumArcs(*lfst_, s), compute_weight);
   Weight lfinal = internal::Final(*lfst_, s);
   bool reach_final = lfinal != Weight::Zero() && label_reachable_->ReachFinal();
   if (reach_arc) {
@@ -626,15 +608,13 @@ bool LabelLookAheadMatcher<M, F, S, R>::LookAheadFst(const L &fst, StateId s) {
     }
   }
   if (reach_final && compute_weight)
-    SetLookAheadWeight(reach_arc ?
-                       Plus(LookAheadWeight(), lfinal) : lfinal);
+    SetLookAheadWeight(reach_arc ? Plus(LookAheadWeight(), lfinal) : lfinal);
 
   return reach_arc || reach_final;
 }
 
-
 // Label-lookahead relabeling class.
-template <class A, class R = LabelReachableData<typename A::Label> >
+template <class A, class R = LabelReachableData<typename A::Label>>
 class LabelLookAheadRelabeler {
  public:
   typedef typename A::Label Label;
@@ -644,15 +624,14 @@ class LabelLookAheadRelabeler {
 
   // Relabels matcher Fst - initialization function object.
   template <typename I>
-  LabelLookAheadRelabeler(I **impl);
+  explicit LabelLookAheadRelabeler(std::shared_ptr<I> *impl);
 
   // Relabels arbitrary Fst. Class L should be a label-lookahead Fst.
   template <class L>
-  static void Relabel(MutableFst<A> *fst, const L &mfst,
-                      bool relabel_input) {
-    typename L::Impl *impl = mfst.GetImpl();
-    D *data = impl->GetAddOn();
-    Reachable reachable(data->First() ? data->First() : data->Second());
+  static void Relabel(MutableFst<A> *fst, const L &mfst, bool relabel_input) {
+    const D *data = mfst.GetAddOn();
+    Reachable reachable(data->First() ? data->SharedFirst()
+                                      : data->SharedSecond());
     reachable.Relabel(fst, relabel_input);
   }
 
@@ -662,56 +641,53 @@ class LabelLookAheadRelabeler {
   // ensure no collisions when relabeling automata that have
   // labels unseen here.
   template <class L>
-  static void RelabelPairs(const L &mfst, vector<pair<Label, Label> > *pairs,
+  static void RelabelPairs(const L &mfst,
+                           std::vector<std::pair<Label, Label>> *pairs,
                            bool avoid_collisions = false) {
-    typename L::Impl *impl = mfst.GetImpl();
-    D *data = impl->GetAddOn();
-    Reachable reachable(data->First() ? data->First() : data->Second());
+    const D *data = mfst.GetAddOn();
+    Reachable reachable(data->First() ? data->SharedFirst()
+                                      : data->SharedSecond());
     reachable.RelabelPairs(pairs, avoid_collisions);
   }
 };
 
 template <class A, class R>
-template <typename I> inline
-LabelLookAheadRelabeler<A, R>::LabelLookAheadRelabeler(I **impl) {
+template <typename I>
+inline LabelLookAheadRelabeler<A, R>::LabelLookAheadRelabeler(
+    std::shared_ptr<I> *impl) {
   Fst<A> &fst = (*impl)->GetFst();
-  D *data = (*impl)->GetAddOn();
+  std::shared_ptr<D> data = (*impl)->GetSharedAddOn();
   const string name = (*impl)->Type();
   bool is_mutable = fst.Properties(kMutable, false);
-  MutableFst<A> *mfst = 0;
+  std::unique_ptr<MutableFst<A>> mfst;
   if (is_mutable) {
-    mfst = static_cast<MutableFst<A> *>(&fst);
+    mfst.reset(static_cast<MutableFst<A> *>(&fst));
   } else {
-    mfst = new VectorFst<A>(fst);
-    data->IncrRefCount();
-    delete *impl;
+    mfst.reset(new VectorFst<A>(fst));
   }
   if (data->First()) {  // reach_input
-    Reachable reachable(data->First());
-    reachable.Relabel(mfst, true);
+    Reachable reachable(data->SharedFirst());
+    reachable.Relabel(mfst.get(), true);
     if (!FLAGS_save_relabel_ipairs.empty()) {
-      vector<pair<Label, Label> > pairs;
+      std::vector<std::pair<Label, Label>> pairs;
       reachable.RelabelPairs(&pairs, true);
       WriteLabelPairs(FLAGS_save_relabel_ipairs, pairs);
     }
   } else {
-    Reachable reachable(data->Second());
-    reachable.Relabel(mfst, false);
+    Reachable reachable(data->SharedSecond());
+    reachable.Relabel(mfst.get(), false);
     if (!FLAGS_save_relabel_opairs.empty()) {
-      vector<pair<Label, Label> > pairs;
+      std::vector<std::pair<Label, Label>> pairs;
       reachable.RelabelPairs(&pairs, true);
       WriteLabelPairs(FLAGS_save_relabel_opairs, pairs);
     }
   }
   if (!is_mutable) {
-    *impl = new I(*mfst, name);
+    *impl = std::make_shared<I>(*mfst, name);
     (*impl)->SetAddOn(data);
-    delete mfst;
-    data->DecrRefCount();
   }
 }
 
-
 // Generic lookahead matcher, templated on the FST definition
 // - a wrapper around pointer to specific one.
 template <class F>
@@ -726,11 +702,14 @@ class LookAheadMatcher {
 
   LookAheadMatcher(const F &fst, MatchType match_type) {
     base_ = fst.InitMatcher(match_type);
-    if (!base_)
-      base_ = new SortedMatcher<F>(fst, match_type);
+    if (!base_) base_ = new SortedMatcher<F>(fst, match_type);
     lookahead_ = false;
   }
 
+  // Takes ownership of base.
+  explicit LookAheadMatcher(MatcherBase<Arc> *base)
+      : base_(base), lookahead_(false) {}
+
   LookAheadMatcher(const LookAheadMatcher<F> &matcher, bool safe = false) {
     base_ = matcher.base_->Copy(safe);
     lookahead_ = matcher.lookahead_;
@@ -740,15 +719,16 @@ class LookAheadMatcher {
 
   // General matcher methods
   LookAheadMatcher<F> *Copy(bool safe = false) const {
-      return new LookAheadMatcher<F>(*this, safe);
+    return new LookAheadMatcher<F>(*this, safe);
   }
 
   MatchType Type(bool test) const { return base_->Type(test); }
   void SetState(StateId s) { base_->SetState(s); }
   bool Find(Label label) { return base_->Find(label); }
   bool Done() const { return base_->Done(); }
-  const Arc& Value() const { return base_->Value(); }
+  const Arc &Value() const { return base_->Value(); }
   void Next() { base_->Next(); }
+  Weight Final(StateId s) const { return base_->Final(s); }
   ssize_t Priority(StateId s) { return base_->Priority(s); }
   const F &GetFst() const { return static_cast<const F &>(base_->GetFst()); }
 
@@ -793,7 +773,7 @@ class LookAheadMatcher {
     }
   }
 
-  void InitLookAheadFst(const Fst<Arc>& fst, bool copy = false) {
+  void InitLookAheadFst(const Fst<Arc> &fst, bool copy = false) {
     if (LookAheadCheck()) {
       LBase *lbase = static_cast<LBase *>(base_);
       lbase->InitLookAheadFst(fst, copy);
@@ -803,8 +783,8 @@ class LookAheadMatcher {
  private:
   bool LookAheadCheck() const {
     if (!lookahead_) {
-      lookahead_ = base_->Flags() &
-          (kInputLookAheadMatcher | kOutputLookAheadMatcher);
+      lookahead_ =
+          base_->Flags() & (kInputLookAheadMatcher | kOutputLookAheadMatcher);
       if (!lookahead_) {
         FSTERROR() << "LookAheadMatcher: No look-ahead matcher defined";
       }
@@ -820,4 +800,4 @@ class LookAheadMatcher {
 
 }  // namespace fst
 
-#endif  // FST_LIB_LOOKAHEAD_MATCHER_H__
+#endif  // FST_LIB_LOOKAHEAD_MATCHER_H_
