@@ -1,25 +1,10 @@
-// product-weight.h
-
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
 // Product weight set and associated semiring operation definitions.
 
-#ifndef FST_LIB_PRODUCT_WEIGHT_H__
-#define FST_LIB_PRODUCT_WEIGHT_H__
+#ifndef FST_LIB_PRODUCT_WEIGHT_H_
+#define FST_LIB_PRODUCT_WEIGHT_H_
 
 #include <stack>
 #include <string>
@@ -31,7 +16,7 @@
 namespace fst {
 
 // Product semiring: W1 * W2
-template<class W1, class W2>
+template <class W1, class W2>
 class ProductWeight : public PairWeight<W1, W2> {
  public:
   using PairWeight<W1, W2>::Zero;
@@ -41,11 +26,11 @@ class ProductWeight : public PairWeight<W1, W2> {
   using PairWeight<W1, W2>::Reverse;
 
   typedef ProductWeight<typename W1::ReverseWeight, typename W2::ReverseWeight>
-  ReverseWeight;
+      ReverseWeight;
 
   ProductWeight() {}
 
-  ProductWeight(const PairWeight<W1, W2>& w) : PairWeight<W1, W2>(w) {}
+  ProductWeight(const PairWeight<W1, W2> &w) : PairWeight<W1, W2>(w) {}
 
   ProductWeight(W1 w1, W2 w2) : PairWeight<W1, W2>(w1, w2) {}
 
@@ -73,17 +58,15 @@ class ProductWeight : public PairWeight<W1, W2> {
   static uint64 Properties() {
     uint64 props1 = W1::Properties();
     uint64 props2 = W2::Properties();
-    return props1 & props2 & (kLeftSemiring | kRightSemiring |
-                              kCommutative | kIdempotent);
+    return props1 & props2 &
+           (kLeftSemiring | kRightSemiring | kCommutative | kIdempotent);
   }
 
   ProductWeight<W1, W2> Quantize(float delta = kDelta) const {
     return PairWeight<W1, W2>::Quantize(delta);
   }
 
-  ReverseWeight Reverse() const {
-    return PairWeight<W1, W2>::Reverse();
-  }
+  ReverseWeight Reverse() const { return PairWeight<W1, W2>::Reverse(); }
 };
 
 template <class W1, class W2>
@@ -110,4 +93,4 @@ inline ProductWeight<W1, W2> Divide(const ProductWeight<W1, W2> &w,
 
 }  // namespace fst
 
-#endif  // FST_LIB_PRODUCT_WEIGHT_H__
+#endif  // FST_LIB_PRODUCT_WEIGHT_H_
