@@ -1,26 +1,11 @@
-// randequivalent.h
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
+//
+// Tests if two FSTS are equivalent by checking if random strings from one FST
+// are transduced the same by both FSTs.
 
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: allauzen@google.com (Cyril Allauzen)
-//
-// \file
-// Tests if two FSTS are equivalent by checking if random
-// strings from one FST are transduced the same by both FSTs.
-
-#ifndef FST_RANDEQUIVALENT_H__
-#define FST_RANDEQUIVALENT_H__
+#ifndef FST_LIB_RANDEQUIVALENT_H_
+#define FST_LIB_RANDEQUIVALENT_H_
 
 #include <fst/arcsort.h>
 #include <fst/compose.h>
@@ -29,6 +14,7 @@
 #include <fst/shortest-distance.h>
 #include <fst/vector-fst.h>
 
+
 namespace fst {
 
 // Test if two FSTs are equivalent by randomly generating 'num_paths'
@@ -38,19 +24,24 @@ namespace fst {
 // of the two FSTs the sum of the weights of all the successful paths
 // sharing the same input and output labels as the considered randomly
 // generated path and checks that these two values are within
-// 'delta'.
-template<class Arc, class ArcSelector>
+// 'delta'. Returns optional error value (when FLAGS_error_fatal = false).
+template <class Arc, class ArcSelector>
 bool RandEquivalent(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
-                    ssize_t num_paths, float delta,
-                    const RandGenOptions<ArcSelector> &opts) {
+                    int32 num_paths, float delta,
+                    const RandGenOptions<ArcSelector> &opts,
+                    bool *error = nullptr) {
   typedef typename Arc::Weight Weight;
+  if (error) *error = false;
 
   // Check that the symbol table are compatible
   if (!CompatSymbols(fst1.InputSymbols(), fst2.InputSymbols()) ||
-      !CompatSymbols(fst1.OutputSymbols(), fst2.OutputSymbols()))
-      LOG(FATAL) << "RandEquivalent: input/output symbol tables of 1st "
-                 << "argument do not match input/output symbol tables of 2nd "
-                 << "argument";
+      !CompatSymbols(fst1.OutputSymbols(), fst2.OutputSymbols())) {
+    FSTERROR() << "RandEquivalent: Input/output symbol tables of 1st "
+               << "argument do not match input/output symbol tables of 2nd "
+               << "argument";
+    if (error) *error = true;
+    return false;
+  }
 
   ILabelCompare<Arc> icomp;
   OLabelCompare<Arc> ocomp;
@@ -61,7 +52,8 @@ bool RandEquivalent(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
   ArcSort(&sfst1, icomp);
   ArcSort(&sfst2, icomp);
 
-  for (ssize_t n = 0; n < num_paths; ++n) {
+  bool ret = true;
+  for (auto n = 0; n < num_paths; ++n) {
     VectorFst<Arc> path;
     const Fst<Arc> &fst = rand() % 2 ? sfst1 : sfst2;
     RandGen(fst, &path, opts);
@@ -92,28 +84,35 @@ bool RandEquivalent(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
     Weight sum2 = ShortestDistance(pfst2);
 
     if (!ApproxEqual(sum1, sum2, delta)) {
-        VLOG(1) << "Sum1 = " << sum1;
-        VLOG(1) << "Sum2 = " << sum2;
-        return false;
+      VLOG(1) << "Sum1 = " << sum1;
+      VLOG(1) << "Sum2 = " << sum2;
+      ret = false;
+      break;
     }
   }
-  return true;
-}
 
+  if (fst1.Properties(kError, false) || fst2.Properties(kError, false)) {
+    if (error) *error = true;
+    return false;
+  }
+
+  return ret;
+}
 
 // Test if two FSTs are equivalent by randomly generating 'num_paths' paths
 // of length no more than 'path_length' using the seed 'seed' in these FSTs.
+// Returns optional error value (when FLAGS_error_fatal = false).
 template <class Arc>
 bool RandEquivalent(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
-                    ssize_t num_paths, float delta = kDelta,
-                    int seed = time(0), int path_length = INT_MAX) {
+                    int32 num_paths, float delta = kDelta,
+                    time_t seed = time(nullptr),
+                    int32 path_length = std::numeric_limits<int32>::max(),
+                    bool *error = nullptr) {
   UniformArcSelector<Arc> uniform_selector(seed);
-  RandGenOptions< UniformArcSelector<Arc> >
-      opts(uniform_selector, path_length);
-  return RandEquivalent(fst1, fst2, num_paths, delta, opts);
+  RandGenOptions<UniformArcSelector<Arc>> opts(uniform_selector, path_length);
+  return RandEquivalent(fst1, fst2, num_paths, delta, opts, error);
 }
 
-
 }  // namespace fst
 
-#endif  // FST_LIB_RANDEQUIVALENT_H__
+#endif  // FST_LIB_RANDEQUIVALENT_H_
