@@ -1,58 +1,48 @@
-// rational.h
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
+//
+// An FST implementation and base interface for delayed unions, concatenations,
+// and closures.
 
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
-//
-// Copyright 2005-2010 Google, Inc.
-// Author: riley@google.com (Michael Riley)
-//
-// \file
-// An Fst implementation and base interface for delayed unions,
-// concatenations and closures.
-
-#ifndef FST_LIB_RATIONAL_H__
-#define FST_LIB_RATIONAL_H__
+#ifndef FST_LIB_RATIONAL_H_
+#define FST_LIB_RATIONAL_H_
 
 #include <algorithm>
 #include <string>
 #include <vector>
-using std::vector;
-#include <fst/map.h>
+
 #include <fst/mutable-fst.h>
 #include <fst/replace.h>
 #include <fst/test-properties.h>
 
+
 namespace fst {
 
 typedef CacheOptions RationalFstOptions;
 
 // This specifies whether to add the empty string.
-enum ClosureType { CLOSURE_STAR = 0,    // T* -> add the empty string
-                   CLOSURE_PLUS = 1 };  // T+ -> don't add the empty string
-
-template <class A> class RationalFst;
-template <class A> void Union(RationalFst<A> *fst1, const Fst<A> &fst2);
-template <class A> void Concat(RationalFst<A> *fst1, const Fst<A> &fst2);
-template <class A> void Concat(const Fst<A> &fst1, RationalFst<A> *fst2);
-template <class A> void Closure(RationalFst<A> *fst, ClosureType closure_type);
+enum ClosureType {
+  CLOSURE_STAR = 0,  // T* -> add the empty string
+  CLOSURE_PLUS = 1
+};  // T+ -> don't add the empty string
 
+template <class A>
+class RationalFst;
+template <class A>
+void Union(RationalFst<A> *fst1, const Fst<A> &fst2);
+template <class A>
+void Concat(RationalFst<A> *fst1, const Fst<A> &fst2);
+template <class A>
+void Concat(const Fst<A> &fst1, RationalFst<A> *fst2);
+template <class A>
+void Closure(RationalFst<A> *fst, ClosureType closure_type);
 
 // Implementation class for delayed unions, concatenations and closures.
-template<class A>
+template <class A>
 class RationalFstImpl : public FstImpl<A> {
  public:
   using FstImpl<A>::SetType;
   using FstImpl<A>::SetProperties;
-  using FstImpl<A>::Properties;
   using FstImpl<A>::WriteHeader;
   using FstImpl<A>::SetInputSymbols;
   using FstImpl<A>::SetOutputSymbols;
@@ -63,11 +53,9 @@ class RationalFstImpl : public FstImpl<A> {
   typedef typename A::Label Label;
 
   explicit RationalFstImpl(const RationalFstOptions &opts)
-      : nonterminals_(0),
-        replace_(0),
-        replace_options_(opts, 0) {
+      : nonterminals_(0), replace_(0), replace_options_(opts, 0) {
     SetType("rational");
-    fst_tuples_.push_back(make_pair<Label, const Fst<A>*>(0, 0));
+    fst_tuples_.push_back(std::pair<Label, const Fst<A> *>(0, 0));
   }
 
   RationalFstImpl(const RationalFstImpl<A> &impl)
@@ -79,18 +67,16 @@ class RationalFstImpl : public FstImpl<A> {
     SetType("rational");
     fst_tuples_.reserve(impl.fst_tuples_.size());
     for (size_t i = 0; i < impl.fst_tuples_.size(); ++i)
-      fst_tuples_.push_back(make_pair(impl.fst_tuples_[i].first,
-                                      impl.fst_tuples_[i].second
-                                      ? impl.fst_tuples_[i].second->Copy(true)
-                                      : 0));
+      fst_tuples_.push_back(std::make_pair(
+          impl.fst_tuples_[i].first,
+          impl.fst_tuples_[i].second ? impl.fst_tuples_[i].second->Copy(true)
+                                     : 0));
   }
 
-  virtual ~RationalFstImpl() {
+  ~RationalFstImpl() override {
     for (size_t i = 0; i < fst_tuples_.size(); ++i)
-      if (fst_tuples_[i].second)
-        delete fst_tuples_[i].second;
-    if (replace_)
-      delete replace_;
+      if (fst_tuples_[i].second) delete fst_tuples_[i].second;
+    if (replace_) delete replace_;
   }
 
   StateId Start() { return Replace()->Start(); }
@@ -99,18 +85,24 @@ class RationalFstImpl : public FstImpl<A> {
 
   size_t NumArcs(StateId s) { return Replace()->NumArcs(s); }
 
-  size_t NumInputEpsilons(StateId s) {
-    return Replace()->NumInputEpsilons(s);
-  }
+  size_t NumInputEpsilons(StateId s) { return Replace()->NumInputEpsilons(s); }
 
   size_t NumOutputEpsilons(StateId s) {
     return Replace()->NumOutputEpsilons(s);
   }
 
+  uint64 Properties() const override { return Properties(kFstProperties); }
+
+  // Set error if found; return FST impl properties.
+  uint64 Properties(uint64 mask) const override {
+    if ((mask & kError) && Replace()->Properties(kError, false))
+      SetProperties(kError, kError);
+    return FstImpl<Arc>::Properties(mask);
+  }
+
   // Implementation of UnionFst(fst1,fst2)
   void InitUnion(const Fst<A> &fst1, const Fst<A> &fst2) {
-    if (replace_)
-      delete replace_;
+    if (replace_) delete replace_;
     uint64 props1 = fst1.Properties(kFstProperties, false);
     uint64 props2 = fst2.Properties(kFstProperties, false);
     SetInputSymbols(fst1.InputSymbols());
@@ -124,15 +116,14 @@ class RationalFstImpl : public FstImpl<A> {
     nonterminals_ = 2;
     rfst_.AddArc(0, A(0, -1, Weight::One(), 1));
     rfst_.AddArc(0, A(0, -2, Weight::One(), 1));
-    fst_tuples_.push_back(make_pair(-1, fst1.Copy()));
-    fst_tuples_.push_back(make_pair(-2, fst2.Copy()));
+    fst_tuples_.push_back(std::make_pair(-1, fst1.Copy()));
+    fst_tuples_.push_back(std::make_pair(-2, fst2.Copy()));
     SetProperties(UnionProperties(props1, props2, true), kCopyProperties);
   }
 
   // Implementation of ConcatFst(fst1,fst2)
   void InitConcat(const Fst<A> &fst1, const Fst<A> &fst2) {
-    if (replace_)
-      delete replace_;
+    if (replace_) delete replace_;
     uint64 props1 = fst1.Properties(kFstProperties, false);
     uint64 props2 = fst2.Properties(kFstProperties, false);
     SetInputSymbols(fst1.InputSymbols());
@@ -147,15 +138,14 @@ class RationalFstImpl : public FstImpl<A> {
     nonterminals_ = 2;
     rfst_.AddArc(0, A(0, -1, Weight::One(), 1));
     rfst_.AddArc(1, A(0, -2, Weight::One(), 2));
-    fst_tuples_.push_back(make_pair(-1, fst1.Copy()));
-    fst_tuples_.push_back(make_pair(-2, fst2.Copy()));
+    fst_tuples_.push_back(std::make_pair(-1, fst1.Copy()));
+    fst_tuples_.push_back(std::make_pair(-2, fst2.Copy()));
     SetProperties(ConcatProperties(props1, props2, true), kCopyProperties);
   }
 
   // Implementation of ClosureFst(fst, closure_type)
   void InitClosure(const Fst<A> &fst, ClosureType closure_type) {
-    if (replace_)
-      delete replace_;
+    if (replace_) delete replace_;
     uint64 props = fst.Properties(kFstProperties, false);
     SetInputSymbols(fst.InputSymbols());
     SetOutputSymbols(fst.OutputSymbols());
@@ -174,7 +164,7 @@ class RationalFstImpl : public FstImpl<A> {
     }
     rfst_.SetInputSymbols(fst.InputSymbols());
     rfst_.SetOutputSymbols(fst.OutputSymbols());
-    fst_tuples_.push_back(make_pair(-1, fst.Copy()));
+    fst_tuples_.push_back(std::make_pair(-1, fst.Copy()));
     nonterminals_ = 1;
     SetProperties(ClosureProperties(props, closure_type == CLOSURE_STAR, true),
                   kCopyProperties);
@@ -182,9 +172,8 @@ class RationalFstImpl : public FstImpl<A> {
 
   // Implementation of Union(Fst &, RationalFst *)
   void AddUnion(const Fst<A> &fst) {
-    if (replace_)
-      delete replace_;
-    uint64 props1 = Properties();
+    if (replace_) delete replace_;
+    uint64 props1 = FstImpl<A>::Properties();
     uint64 props2 = fst.Properties(kFstProperties, false);
     VectorFst<A> afst;
     afst.AddState();
@@ -194,15 +183,14 @@ class RationalFstImpl : public FstImpl<A> {
     ++nonterminals_;
     afst.AddArc(0, A(0, -nonterminals_, Weight::One(), 1));
     Union(&rfst_, afst);
-    fst_tuples_.push_back(make_pair(-nonterminals_, fst.Copy()));
+    fst_tuples_.push_back(std::make_pair(-nonterminals_, fst.Copy()));
     SetProperties(UnionProperties(props1, props2, true), kCopyProperties);
   }
 
   // Implementation of Concat(Fst &, RationalFst *)
   void AddConcat(const Fst<A> &fst, bool append) {
-    if (replace_)
-      delete replace_;
-    uint64 props1 = Properties();
+    if (replace_) delete replace_;
+    uint64 props1 = FstImpl<A>::Properties();
     uint64 props2 = fst.Properties(kFstProperties, false);
     VectorFst<A> afst;
     afst.AddState();
@@ -215,31 +203,25 @@ class RationalFstImpl : public FstImpl<A> {
       Concat(&rfst_, afst);
     else
       Concat(afst, &rfst_);
-    fst_tuples_.push_back(make_pair(-nonterminals_, fst.Copy()));
+    fst_tuples_.push_back(std::make_pair(-nonterminals_, fst.Copy()));
     SetProperties(ConcatProperties(props1, props2, true), kCopyProperties);
   }
 
   // Implementation of Closure(RationalFst *, closure_type)
   void AddClosure(ClosureType closure_type) {
-    if (replace_)
-      delete replace_;
-    uint64 props = Properties();
+    if (replace_) delete replace_;
+    uint64 props = FstImpl<A>::Properties();
     Closure(&rfst_, closure_type);
     SetProperties(ClosureProperties(props, closure_type == CLOSURE_STAR, true),
                   kCopyProperties);
   }
 
-  // Create the underlying ReplaceFst. No mutation can be performed
-  // after that.
-  void InitReplace() {
-    fst_tuples_[0].second = rfst_.Copy();
-    replace_ = new ReplaceFst<A>(fst_tuples_, replace_options_);
-  }
-
   // Returns the underlying ReplaceFst.
-  ReplaceFst<A> *Replace() {
-    if (!replace_)
-      InitReplace();
+  ReplaceFst<A> *Replace() const {
+    if (!replace_) {
+      fst_tuples_[0].second = rfst_.Copy();
+      replace_ = new ReplaceFst<A>(fst_tuples_, replace_options_);
+    }
     return replace_;
   }
 
@@ -247,11 +229,11 @@ class RationalFstImpl : public FstImpl<A> {
   VectorFst<A> rfst_;   // rational topology machine; uses neg. nonterminals
   Label nonterminals_;  // # of nonterminals used
   // Contains the nonterminals and their corresponding FSTs.
-  vector<pair<Label, const Fst<A>*> > fst_tuples_;
-  ReplaceFst<A> *replace_;             // Underlying ReplaceFst
+  mutable std::vector<std::pair<Label, const Fst<A> *>> fst_tuples_;
+  mutable ReplaceFst<A> *replace_;        // Underlying ReplaceFst
   ReplaceFstOptions<A> replace_options_;  // Options for creating 'replace_'
 
-  void operator=(const RationalFstImpl<A> &impl);    // disallow
+  void operator=(const RationalFstImpl<A> &impl);  // disallow
 };
 
 // Parent class for the delayed rational operations - delayed union,
@@ -260,10 +242,10 @@ class RationalFstImpl : public FstImpl<A> {
 // This class attaches interface to implementation and handles
 // reference counting, delegating most methods to ImplToFst.
 template <class A>
-class RationalFst : public ImplToFst< RationalFstImpl<A> > {
+class RationalFst : public ImplToFst<RationalFstImpl<A>> {
  public:
-  friend class StateIterator< RationalFst<A> >;
-  friend class ArcIterator< RationalFst<A> >;
+  friend class StateIterator<RationalFst<A>>;
+  friend class ArcIterator<RationalFst<A>>;
   friend void Union<>(RationalFst<A> *fst1, const Fst<A> &fst2);
   friend void Concat<>(RationalFst<A> *fst1, const Fst<A> &fst2);
   friend void Concat<>(const Fst<A> &fst1, RationalFst<A> *fst2);
@@ -273,54 +255,49 @@ class RationalFst : public ImplToFst< RationalFstImpl<A> > {
   typedef typename A::StateId StateId;
   typedef RationalFstImpl<A> Impl;
 
-  virtual void InitStateIterator(StateIteratorData<A> *data) const {
+  void InitStateIterator(StateIteratorData<A> *data) const override {
     GetImpl()->Replace()->InitStateIterator(data);
   }
 
-  virtual void InitArcIterator(StateId s, ArcIteratorData<A> *data) const {
+  void InitArcIterator(StateId s, ArcIteratorData<A> *data) const override {
     GetImpl()->Replace()->InitArcIterator(s, data);
   }
 
  protected:
+  using ImplToFst<Impl>::GetImpl;
+
   RationalFst()
-      : ImplToFst<Impl>(new Impl(RationalFstOptions())) {}
+      : ImplToFst<Impl>(std::make_shared<Impl>(RationalFstOptions())) {}
 
   explicit RationalFst(const RationalFstOptions &opts)
-      : ImplToFst<Impl>(new Impl(opts)) {}
+      : ImplToFst<Impl>(std::make_shared<Impl>(opts)) {}
 
   // See Fst<>::Copy() for doc.
-  RationalFst(const RationalFst<A> &fst , bool safe = false)
+  RationalFst(const RationalFst<A> &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
  private:
-  // Makes visible to friends.
-  Impl *GetImpl() const { return ImplToFst<Impl>::GetImpl(); }
-
   void operator=(const RationalFst<A> &fst);  // disallow
 };
 
-
 // Specialization for RationalFst.
 template <class A>
-class StateIterator< RationalFst<A> >
-    : public StateIterator< ReplaceFst<A> > {
+class StateIterator<RationalFst<A>> : public StateIterator<ReplaceFst<A>> {
  public:
   explicit StateIterator(const RationalFst<A> &fst)
-      : StateIterator< ReplaceFst<A> >(*(fst.GetImpl()->Replace())) {}
+      : StateIterator<ReplaceFst<A>>(*(fst.GetImpl()->Replace())) {}
 };
 
-
 // Specialization for RationalFst.
 template <class A>
-class ArcIterator< RationalFst<A> >
-    : public CacheArcIterator< ReplaceFst<A> > {
+class ArcIterator<RationalFst<A>> : public CacheArcIterator<ReplaceFst<A>> {
  public:
   typedef typename A::StateId StateId;
 
   ArcIterator(const RationalFst<A> &fst, StateId s)
-      : ArcIterator< ReplaceFst<A> >(*(fst.GetImpl()->Replace()), s) {}
+      : ArcIterator<ReplaceFst<A>>(*(fst.GetImpl()->Replace()), s) {}
 };
 
 }  // namespace fst
 
-#endif  // FST_LIB_RATIONAL_H__
+#endif  // FST_LIB_RATIONAL_H_
