@@ -17,15 +17,19 @@
 // \file
 // Google-style flag handling definitions
 
+#include <cstdlib>
 #include <cstring>
 
 #include <fst/compat.h>
 #include <fst/flags.h>
 
+static const char *private_tmpdir = getenv("TMPDIR");
+
 DEFINE_int32(v, 0, "verbose level");
 DEFINE_bool(help, false, "show usage information");
 DEFINE_bool(helpshort, false, "show brief usage information");
-DEFINE_string(tmpdir, "/tmp/", "temporary directory");
+DEFINE_string(tmpdir, private_tmpdir ? private_tmpdir : "/tmp",
+              "temporary directory");
 
 using namespace std;
 
@@ -49,7 +53,7 @@ void SetFlags(const char *usage, int *argc, char ***argv,
     string val = "";
 
     // split argval (arg=val) into arg and val
-    int pos = argval.find("=");
+    size_t pos = argval.find("=");
     if (pos != string::npos) {
       arg = argval.substr(0, pos);
       val = argval.substr(pos + 1);
@@ -118,7 +122,7 @@ ShowUsageRestrict(const std::set< pair<string, string> > &usage_set,
     if (file != old_file) {
       if (show_file) {
         if (file_out) cout << "\n";
-	cout << "Flags from: " << file << "\n";
+	      cout << "Flags from: " << file << "\n";
         file_out = true;
       }
       old_file = file;
